#!/bin/bash
#
# postinst.sh
#
# Postinstall script
#
#######################################################################################
#
export PATH=/usr/local/bin:/usr/bin:$PATH
#
ULSDIR="$1"
#
if [[ ! -d "$ULSDIR" ]]
 then
  echo "ERROR: ulsdir not set"
  echo "usage: `basename $0` <ULS-BASE-PATH>"
  exit 1
fi
#
MYSQLDEFAULTS="$2"
MYSQLO="mysql --defaults-extra-file=$MYSQLDEFAULTS -B -N logging"
#
# 1.9.5 -> 1.9.6
if [[ -f /etc/cron.d/ulsserver ]]
 then
  if ! grep -q 'send_utt_reminder' /etc/cron.d/ulsserver
   then
    echo "#
# UTT Reminder
*/2 * * * * uls $ULSDIR/jobs/send_utt_reminder &>/dev/null" >>/etc/cron.d/ulsserver
  fi
else
  echo "Damit die UTT-Reminder-E-Mails versendet werden, muss ggf.
in der entsprechenden crontab-Datei folgender Eintag ergänzt werden:
# UTT Reminder
*/2 * * * * uls $ULSDIR/jobs/send_utt_reminder &>/dev/null"
fi
#
# U2W-Configuration
#
for f in $(grep -l '^SWITCHUSER=wwwuls' /etc/unix2web/*)
 do
  if ! grep -q ' *\. /etc/ulsserver.conf' $f
   then
    sed -i 's|^SWITCHUSER=wwwuls|[[ -f /etc/ulsserver.conf ]] \&\& . /etc/ulsserver.conf\n&|' $f
  fi
  if ! grep -q -- '-Xd ulsurl=' $f
   then
    sed -i 's|U2WPARS="|&-Xd ulsurl=${ULSURL%/} |' $f
  fi
done
if ! grep -q '^U2WPORT=80$' /etc/unix2web/*
 then
  f=$(grep -l '^U2WPORT=11975$' /etc/unix2web/uls.conf)
  if [[ -z "$f" ]]
   then
    f=$(grep -l '^U2WPORT=11975$' /etc/unix2web/uls.conf.*)
    if [[ -z "$f" ]]
     then
      f=$(grep -l '^U2WPORT=11975$' /etc/unix2web/*)
    fi
  fi
  if [[ -n "$f" ]]
   then
    sed 's/^U2WPORT=11975$/U2WPORT=80/;/^U2WHOME=/s|\(U2WHOME=.*\)/html$|\1/html/user|;/^U2WPARS=/s| -V -r ULS -sh '"$ULSDIR"'/passwd.dat | |' "$f" >/etc/unix2web/uls80.conf
  fi
else
  for f in $(grep -l '^U2WPORT=80$' /etc/unix2web/*)
   do
    sed -i '/^U2WHOME=/s|\(U2WHOME=.*\)/html$|\1/html/user|;/^U2WPARS=/s| -V -r ULS -sh '"$ULSDIR"'/passwd.dat | |' "$f"
  done
fi
if ! grep -q '^U2WPORT=443$' /etc/unix2web/*
 then
  f=$(grep -l '^U2WPORT=11976$' /etc/unix2web/uls.conf)
  if [[ -z "$f" ]]
   then
    f=$(grep -l '^U2WPORT=11976$' /etc/unix2web/uls.conf.*)
    if [[ -z "$f" ]]
     then
      f=$(grep -l '^U2WPORT=11976$' /etc/unix2web/*)
    fi
  fi
  if [[ -n "$f" ]]
   then
    sed 's/^U2WPORT=11976$/U2WPORT=443/;/^U2WHOME=/s|\(U2WHOME=.*\)/html-https$|\1/html/user|;/^U2WPARS=/s| -V -r ULS -sh '"$ULSDIR"'/passwd.dat | |' "$f" >/etc/unix2web/uls443.conf
  fi
else
  for f in $(grep -l '^U2WPORT=443$' /etc/unix2web/*)
   do
    sed -i '/^U2WHOME=/s|\(U2WHOME=.*\)/html$|\1/html/user|;/^U2WPARS=/s| -V -r ULS -sh '"$ULSDIR"'/passwd.dat | |' "$f"
  done
fi
for f in $(grep -l '^U2WPORT=11975$' /etc/unix2web/*) $(grep -l '^U2WPORT=11976$' /etc/unix2web/*)
 do
  if ! grep -q -- "-Xp /grafana:$ULSDIR/html/rest/grafana.s3w" "$f"
   then
    sed -i '/^U2WPARS=/s|"$| -Xp /grafana:'"$ULSDIR"'/html/rest/grafana.s3w"|' "$f"
  fi
done
/etc/init.d/unix2web restart
/etc/init.d/unix2web start
#
rm "$ULSDIR/html/nouser/index.u2w"
#
# ULS-Report Passwort aendern, wenn noch nicht geschehen
#
if grep -q "^ULSREPPWD='getreport'" "$ULSDIR/ulsreport.conf"
 then
  if which u2w_passwd &>/dev/null
   then
    NEWPWD=$(tr -Cd '[:alnum:],._+=-' </dev/urandom | head -c 12)
    sed -i "s@^ULSREPPWD='getreport'@ULSREPPWD='$NEWPWD'@" "$ULSDIR/ulsreport.conf"
    sed -i 's/r ZG9yZXBvcnQ6Z2V0cmVwb3J0 /r doreport:29a68f48ecd905f57b97c3d863901107 /' "$ULSDIR/passwd.dat"
    u2w_passwd -p "$NEWPWD" "$ULSDIR/passwd.dat" doreport
  fi
fi
#
# Gnuplot mit pngcairo
#
if echo "set terminal" | gnuplot 2>&1 | fgrep -q pngcairo
 then
  if fgrep -q 'set terminal png font ' "$ULSDIR/html/user/loggingout_grafik.s3w"
   then
    echo "pngcairo is supportet by gnuplot - patch loggingout_grafik.s3w to pngcairo"
    sed -i 's/set terminal png font /set terminal pngcairo font /' "$ULSDIR/html/user/loggingout_grafik.s3w"
    sed -i 's/^#.*#set encoding utf8/set encoding utf8/;s/^set encoding iso_8859/#pngcairo is available#set encoding iso_8859/' "$ULSDIR/wwwbin/loggingout_grafik_prescript.plt"
    sed -i 's/ | recode UTF-8..ISO-8859-15 | gnuplot / | gnuplot /' "$ULSDIR/wwwbin/loggingout_grafik.sh"
  fi
else
  if fgrep -q 'set terminal pngcairo font ' "$ULSDIR/html/user/loggingout_grafik.s3w"
   then
    echo "pngcairo is not supportet by gnuplot - patch loggingout_grafik.s3w to png"
    sed -i 's/set terminal pngcairo font /set terminal png font /' "$ULSDIR/html/user/loggingout_grafik.s3w"
    sed -i 's/^set encoding utf8/#has no pngcairo#set encoding utf8/;s/^#.*#set encoding iso_8859/set encoding iso_8859/' "$ULSDIR/wwwbin/loggingout_grafik_prescript.plt"
    if ! fgrep -q 'recode UTF-8..ISO-8859-15' "$ULSDIR/wwwbin/loggingout_grafik.sh"
     then
      sed -i 's/ | gnuplot / | recode UTF-8..ISO-8859-15 | gnuplot /' "$ULSDIR/wwwbin/loggingout_grafik.sh"
    fi
  fi
fi
#
# MariaDB regex
#
if cd $ULSDIR/ulsmeld
 then
  REGEXPFLAGS=`$MYSQLO -e "show variables like '%regex%'"`
  if [[ -z "$REGEXPFLAGS" ]]
   then
    grep -q "^ *%mysqlwrite[(]\"set default_regex_flags='DOTALL'\"[)]" test_limits.s2w && sed -i "s/^\( *%mysqlwrite[(]\"set default_regex_flags='DOTALL'\"[)]\)/%%\1/" ../html/user/login.i2w test_isalive.s2w test_klimits.s2w test_kombilimits.s2w test_limits.s2w utt_escalation.s2w
  else
    grep -q "^%% *%mysqlwrite[(]\"set default_regex_flags='DOTALL'\"[)]" test_limits.s2w && sed -i "s/^%%\( *%mysqlwrite[(]\"set default_regex_flags='DOTALL'\"[)]\)/\1/" ../html/user/login.i2w test_isalive.s2w test_klimits.s2w test_kombilimits.s2w test_limits.s2w utt_escalation.s2w
  fi
fi
#
# send_messages
#
if [[ -f "$1/ulsmeld/send_message.preinst" ]]
 then
  sed -n '/case/,$p' "$1/ulsmeld/send_message.preinst" >/tmp/ulsmsgpreinst.tail
  sed -n '/case/,$p' "$1/ulsmeld/send_message" >/tmp/ulsmsgpostinst.tail
  if ! diff -q /tmp/ulsmsgpreinst.tail /tmp/ulsmsgpostinst.tail 2>/dev/null
   then
    cp -a "$1/ulsmeld/send_message" "$1/ulsmeld/send_message.sample"
    sed '/case/,$d' "$1/ulsmeld/send_message.sample" >"$1/ulsmeld/send_message"
    cat /tmp/ulsmsgpreinst.tail >>"$1/ulsmeld/send_message"
    echo "**********************************************************"
    echo " Note: The script $1/ulsmeld/send_message"
    echo "       was changed during the installation."
    echo "       Compare with $1/ulsmeld/send_message.preinst"
    echo "       and $1/ulsmeld/send_message.sample"
    echo "**********************************************************"
  fi
  rm /tmp/ulsmsgpreinst.tail /tmp/ulsmsgpostinst.tail
fi
#
if [[ -f /etc/ulsserver.conf ]]
 then
  if ! grep -q '^ULSURL=.*http' /etc/ulsserver.conf
   then
    echo "**********************************************************"
    echo " Note: You have to add the ULSURL into /etc/ulsserver.conf"
    echo " Sample: ULSURL='https://$(hostname)/index.u2w'"
    echo "**********************************************************"
  fi
else
  echo "**********************************************************"
  echo " Note: You have to create the file /etc/ulsserver.conf"
  echo "       with at least one line for the ULSURL"
  echo " Sample: ULSURL='https://$(hostname)/index.u2w'"
  echo "**********************************************************"
fi
