-- a2ipstartend.sql
--
-- procedure a2ipstartend(ipset, ipstart, ipend)
-- ipset: set of ip-addressranges
-- ipstart: first ip address (HEX)
-- ipend: last ip address (HEX)
-- get first and last ipaddress from range ipset

set @@character_set_client = 'utf8';

delimiter //
DROP PROCEDURE IF EXISTS a2ipstartend//
create procedure a2ipstartend(ipset varchar(50), OUT ipstart varchar(32), OUT ipend varchar(32))
  deterministic
  sql security invoker
  begin
    declare i, b integer;
    declare htip varchar(32);
    declare hts, hte varchar(16);
    declare ht bigint unsigned;

    set i = instr(ipset, '/');
    if i = 0 then
      if inet6_aton(ipset) is not null then
        set ipstart = hex(inet6_aton(ipset));
        set ipend = hex(inet6_aton(ipset));
      else
        set ipstart = NULL;
        set ipend = NULL;
      end if;
    else
      set b = substring(ipset, i+1);
      set ipset = left(ipset, i-1);
      if is_ipv6(ipset) then
        set htip = hex(inet6_aton(ipset));
        if b > 64 then
          set ht = conv(substr(htip, 17), 16, 10);
          set hts = conv(ht & (-1 << 128-b), 10, 16);
          set hte = conv(ht | ~(-1 << 128-b), 10, 16);
          set ipstart = concat(left(htip, 16), concat(repeat('0', 16 - length(hts)), hts));
          set ipend = concat(left(htip, 16), concat(repeat('0', 16 - length(hte)), hte));
        else
          set ht = conv(left(htip, 16), 16, 10);
          set hts = conv(ht & (-1 << 64-b), 10, 16);
          set hte = conv(ht | ~(-1 << 64-b), 10, 16);
          set ipstart = concat(concat(repeat('0', 16 - length(hts)), hts), '0000000000000000');
          set ipend = concat(concat(repeat('0', 16 - length(hte)), hte), 'FFFFFFFFFFFFFFFF');
        end if;
      else
        set hts = conv(inet_aton(ipset) & (-1 << 32-b), 10, 16);
        if hts is null then
          set ipstart = NULL;
          set ipend = NULL;
        else
          set hte = conv(inet_aton(ipset) | ~(-1 << 32-b), 10, 16);
          set ipstart = concat(repeat('0', 8 - length(hts)), hts);
          set ipend = concat(repeat('0', 8 - length(hte)), hte);
        end if;
      end if;
    end if;
end//
