-- accell.sql
--
-- function accell(lwid, von, bis, zvon, zbis, wtf, vg1, wf1, m2, vg2, wf2, m3, vg3, wf3, m4, vg4, wf4) returns varchar
-- wid: werteindizesID
-- von: datetime from
-- bis: datetime to
-- zvon: time from
-- zbis: time to
-- wtf: set of weekdays
-- vg1: comparevalue 1
-- wf1: comparefunction 1
-- m2: OR|AND
-- vg2: comparevalue 2
-- wf2: comparefunction 2
-- m3: OR|AND
-- vg3: comparevalue 3
-- wf3: comparefunction 3
-- m4: OR|AND
-- vg4: comparevalue 4
-- wf4: comparefunction 4
-- return: acceleration of values with extended filter for compressed values table lwerte

set @@character_set_client = 'utf8';

delimiter //

DROP FUNCTION IF EXISTS accell//
create function accell(lwid integer, von datetime, bis datetime,
  zvon time, zbis time, wtf varchar(20),
  vg1 varchar(3), wf1 varchar(250),
  m2 varchar(3), vg2 varchar(3), wf2 varchar(250),
  m3 varchar(3), vg3 varchar(3), wf3 varchar(250),
  m4 varchar(3), vg4 varchar(3), wf4 varchar(250)) returns varchar(30)
deterministic
sql security invoker
begin
  declare u1, u2, u3, n, z integer;
  declare asum double;
  declare w1, w2, w3 double;
  declare va, ba, vo, bo time;

  declare selcur cursor for select wert, UNIX_TIMESTAMP(datum) from lwerte
    where lwerteindizesID = lwid and von <= datum and datum <= bis
          and ((va <= TIME(datum) and TIME(datum) <= ba)
               or vo <= TIME(datum) or TIME(datum) <= bo)
          and (wtf = '' or find_in_set(DAYOFWEEK(datum), wtf))
          and extfilter(wert, vg1, wf1, m2, vg2, wf2, m3, vg3, wf3, m4, vg4, wf4)
    group by datum
    order by datum;
  declare continue handler for not found set z = 1;

  if TIME(zvon) < TIME(zbis)
   then
    set va = TIME(zvon);
    set ba = TIME(zbis);
    set vo = '24:00:00';
    set bo = '-00:00:01';
  else
    set va = '24:00:00';
    set ba = '-00:00:01';
    set vo = TIME(zvon);
    set bo = TIME(zbis);
  end if;

  set n = 0;
  set z = 0;
  set asum = 0.0;

  open selcur;

  fetch selcur into w1, u1;
  if z = 1
   then
    close selcur;
    return '';
  end if;

  fetch selcur into w2, u2;
  if z = 1
   then
    close selcur;
    return '';
  end if;

  repeat
    fetch selcur into w3, u3;
    if z != 1
     then
      set asum = asum + ((w3-w1)/(u3-u1)-(w2-w1)/(u2-u1))/(u3-u2);
      set n = n + 1;
      set w1 = w2;
      set w2 = w3;
      set u1 = u2;
      set u2 = u3;
    end if;
  until z = 1 end repeat;
  close selcur;

  if n > 0
   then
    return 25920000.0*asum/n;
  else
     return '';
  end if;
end//

GRANT EXECUTE on FUNCTION accell to logginglesen@'localhost', loggingtest@'localhost',
 loggingadmin@'localhost'//
