set @@character_set_client = 'utf8';

delimiter //

drop trigger if exists autoisalive_au//
create trigger autoisalive_au after update on autoisalive
  for each row
begin
  insert into changeprot
    (domainID, accessID, changedtable, tableID, userID, username, action,
     classes, server, section, teststep, details, unit, newvalues)
  select verfahrenID, accessID, 'autoisalive', autoisalive.ID,
    coalesce(@userid,0), coalesce(@username,'SYSTEM'), 'update',
    showreg(classes, classes_v), showreg(server, server_v), showreg(section, section_v),
    showreg(teststep, teststep_v), showreg(details, details_v),
    showreg(einheit, einheit_v),
    concat('period=', maxperiods, '\nperiodunit=', intervalle.name, '\nmsglevel=', msglevel,
           '\nnotifymode=', notifymode, '\nautoclose=', autoclose, '\nmessage=', REPLACE(REPLACE(meldung, '\\', '\\\\'), '\n', '\\n'),
           '\ndestination1=', notifdestname(meldungszielID), '\ndestination2=', notifdestname(meldungsziel2ID),
           '\nremark=', REPLACE(REPLACE(bemerkungen, '\\', '\\\\'), '\n', '\\n'), '\nactive=', aktiv)
  from autoisalive
  join intervalle on intervalle.ID = intervalleID
  where autoisalive.ID = NEW.ID and NEW.TS = autoisalive.TS;
end//


drop trigger if exists autoisalive_ai//
create trigger autoisalive_ai after insert on autoisalive
  for each row
begin
  insert into changeprot
    (domainID, accessID, changedtable, tableID, userID, username, action,
     classes, server, section, teststep, details, unit, newvalues)
  select NEW.verfahrenID, NEW.accessID, 'autoisalive', NEW.ID,
    coalesce(@userid,0), coalesce(@username,'SYSTEM'), 'insert',
    showreg(NEW.classes, NEW.classes_v), showreg(NEW.server, NEW.server_v), showreg(NEW.section, NEW.section_v),
    showreg(NEW.teststep, NEW.teststep_v), showreg(NEW.details, NEW.details_v),
    showreg(NEW.einheit, NEW.einheit_v),
    concat('period=', NEW.maxperiods, '\nperiodunit=', intervalle.name, '\nmsglevel=', NEW.msglevel,
           '\nnotifymode=', NEW.notifymode, '\nautoclose=', NEW.autoclose, '\nmessage=', REPLACE(REPLACE(NEW.meldung, '\\', '\\\\'), '\n', '\\n'),
           '\ndestination1=', notifdestname(NEW.meldungszielID), '\ndestination2=', notifdestname(NEW.meldungsziel2ID),
           '\nremark=', REPLACE(REPLACE(NEW.bemerkungen, '\\', '\\\\'), '\n', '\\n'), '\nactive=', NEW.aktiv)
  from intervalle
  where intervalle.ID = NEW.intervalleID;
end//


drop trigger if exists autoisalive_bd//
create trigger autoisalive_bd before delete on autoisalive
  for each row
begin
  insert into changeprot
    (domainID, accessID, changedtable, tableID, userID, username, action,
     classes, server, section, teststep, details, unit, newvalues)
  select OLD.verfahrenID, OLD.accessID, 'autoisalive', OLD.ID,
    coalesce(@userid,0), coalesce(@username,'SYSTEM'), 'delete',
    showreg(OLD.classes, OLD.classes_v), showreg(OLD.server, OLD.server_v), showreg(OLD.section, OLD.section_v),
    showreg(OLD.teststep, OLD.teststep_v), showreg(OLD.details, OLD.details_v),
    showreg(OLD.einheit, OLD.einheit_v),
    coalesce(if(OLD.bemerkungen = @isaliveremark, '', concat('remark=', REPLACE(REPLACE(@isaliveremark, '\\', '\\\\'), '\n', '\\n'))),'');
end//

delimiter ;
