-- cleanchangeprot.sql
--
-- procedure cleanchangeprot()
-- 
-- remove old entries in changeprot

set @@character_set_client = 'utf8';

delimiter //

DROP procedure IF EXISTS cleanchangeprot//
create procedure cleanchangeprot()
  deterministic
  sql security invoker
  begin

    declare storedays integer unsigned;
    declare deldate date;

    select wert into storedays from systemvars where name = 'StoretimeChangeprot';
    select curdate() - interval storedays day into deldate;

    -- ältesten Datensatz bestimmen, der aufbewahrt werden muss
    create temporary table loggingtmp.delids
      (primary key(changedtable, tableID, mts))
      select changedtable, tableID, max(TS) as mts
      from changeprot
      where TS < curdate() - interval storedays day
      group by changedtable, tableID;

    -- Tabelle / ID mit Daten, die aufbewahrt werden müssen
    create temporary table loggingtmp.storeids
      (primary key(changedtable, tableID))
      select changedtable, tableID
      from changeprot
      where TS >= curdate() - interval storedays day
      group by changedtable, tableID;


    -- zuerst alle Daten löschen, bei denen der geänderte Tabelleneintrag gelöscht wurde
    -- und keine Daten mehr vorhanden sind, die aufbewahrt werden müssen

    -- wertelimits
    delete changeprot
      from changeprot
      join loggingtmp.delids using(changedtable, tableID)
      left join loggingtmp.storeids using(changedtable, tableID)
      left join wertelimits as t on t.ID = changeprot.tableID
      where changeprot.changedtable = 'wertelimits'
        and storeids.tableID is NULL and t.ID is NULL;

    -- autolimits
    delete changeprot
      from changeprot
      join loggingtmp.delids using(changedtable, tableID)
      left join loggingtmp.storeids using(changedtable, tableID)
      left join autolimits as t on t.ID = changeprot.tableID
      where changeprot.changedtable = 'autowertelimits'
        and storeids.tableID is NULL and t.ID is NULL;

    -- gautolimits
    delete changeprot
      from changeprot
      join loggingtmp.delids using(changedtable, tableID)
      left join loggingtmp.storeids using(changedtable, tableID)
      left join gautolimits as t on t.ID = changeprot.tableID
      where changeprot.changedtable = 'gautolimits'
        and storeids.tableID is NULL and t.ID is NULL;

    -- kombilimits
    delete changeprot
      from changeprot
      join loggingtmp.delids using(changedtable, tableID)
      left join loggingtmp.storeids using(changedtable, tableID)
      left join kombilimits as t on t.ID = changeprot.tableID
      where changeprot.changedtable = 'kombilimits'
        and storeids.tableID is NULL and t.ID is NULL;

    -- autokombilimits
    delete changeprot
      from changeprot
      join loggingtmp.delids using(changedtable, tableID)
      left join loggingtmp.storeids using(changedtable, tableID)
      left join autokombilimits as t on t.ID = changeprot.tableID
      where changeprot.changedtable = 'autokombilimits'
        and storeids.tableID is NULL and t.ID is NULL;

    -- gautocombilimits
    delete changeprot
      from changeprot
      join loggingtmp.delids using(changedtable, tableID)
      left join loggingtmp.storeids using(changedtable, tableID)
      left join gautocombilimits as t on t.ID = changeprot.tableID
      where changeprot.changedtable = 'gautocombilimits'
        and storeids.tableID is NULL and t.ID is NULL;

    -- isalive
    delete changeprot
      from changeprot
      join loggingtmp.delids using(changedtable, tableID)
      left join loggingtmp.storeids using(changedtable, tableID)
      left join isalive as t on t.ID = changeprot.tableID
      where changeprot.changedtable = 'isalive'
        and storeids.tableID is NULL and t.ID is NULL;

    -- autoisalive
    delete changeprot
      from changeprot
      join loggingtmp.delids using(changedtable, tableID)
      left join loggingtmp.storeids using(changedtable, tableID)
      left join autoisalive as t on t.ID = changeprot.tableID
      where changeprot.changedtable = 'autoisalive'
        and storeids.tableID is NULL and t.ID is NULL;

    -- gautoisalive
    delete changeprot
      from changeprot
      join loggingtmp.delids using(changedtable, tableID)
      left join loggingtmp.storeids using(changedtable, tableID)
      left join gautoisalive as t on t.ID = changeprot.tableID
      where changeprot.changedtable = 'gautoisalive'
        and storeids.tableID is NULL and t.ID is NULL;

    -- lwertelimits
    delete changeprot
      from changeprot
      join loggingtmp.delids using(changedtable, tableID)
      left join loggingtmp.storeids using(changedtable, tableID)
      left join lwertelimits as t on t.ID = changeprot.tableID
      where changeprot.changedtable = 'lwertelimits'
        and storeids.tableID is NULL and t.ID is NULL;

    -- autollimits
    delete changeprot
      from changeprot
      join loggingtmp.delids using(changedtable, tableID)
      left join loggingtmp.storeids using(changedtable, tableID)
      left join autollimits as t on t.ID = changeprot.tableID
      where changeprot.changedtable = 'autollimits'
        and storeids.tableID is NULL and t.ID is NULL;

    -- gautoclimits
    delete changeprot
      from changeprot
      join loggingtmp.delids using(changedtable, tableID)
      left join loggingtmp.storeids using(changedtable, tableID)
      left join gautoclimits as t on t.ID = changeprot.tableID
      where changeprot.changedtable = 'gautoclimits'
        and storeids.tableID is NULL and t.ID is NULL;

    -- cclimits
    delete changeprot
      from changeprot
      join loggingtmp.delids using(changedtable, tableID)
      left join loggingtmp.storeids using(changedtable, tableID)
      left join cclimits as t on t.ID = changeprot.tableID
      where changeprot.changedtable = 'cclimits'
        and storeids.tableID is NULL and t.ID is NULL;

    -- autocclimits
    delete changeprot
      from changeprot
      join loggingtmp.delids using(changedtable, tableID)
      left join loggingtmp.storeids using(changedtable, tableID)
      left join autocclimits as t on t.ID = changeprot.tableID
      where changeprot.changedtable = 'autocclimits'
        and storeids.tableID is NULL and t.ID is NULL;

    -- gautocclimits
    delete changeprot
      from changeprot
      join loggingtmp.delids using(changedtable, tableID)
      left join loggingtmp.storeids using(changedtable, tableID)
      left join gautocclimits as t on t.ID = changeprot.tableID
      where changeprot.changedtable = 'gautocclimits'
        and storeids.tableID is NULL and t.ID is NULL;

    -- ueberwachungspausen
    delete changeprot
      from changeprot
      join loggingtmp.delids using(changedtable, tableID)
      left join loggingtmp.storeids using(changedtable, tableID)
      left join ueberwachungspausen as t on t.ID = changeprot.tableID
      where changeprot.changedtable = 'ueberwachungspausen'
        and storeids.tableID is NULL and t.ID is NULL;

    -- clientsuspensions
    delete changeprot
      from changeprot
      join loggingtmp.delids using(changedtable, tableID)
      left join loggingtmp.storeids using(changedtable, tableID)
      left join clientsuspensions as t on t.ID = changeprot.tableID
      where changeprot.changedtable = 'clientsuspensions'
        and storeids.tableID is NULL and t.ID is NULL;

    -- kombidetails
    delete changeprot
      from changeprot
      join loggingtmp.delids using(changedtable, tableID)
      left join loggingtmp.storeids using(changedtable, tableID)
      left join kombidetails as t on t.ID = changeprot.tableID
      where changeprot.changedtable = 'kombidetails'
        and storeids.tableID is NULL and t.ID is NULL;

    -- autokombidetails
    delete changeprot
      from changeprot
      join loggingtmp.delids using(changedtable, tableID)
      left join loggingtmp.storeids using(changedtable, tableID)
      left join autokombidetails as t on t.ID = changeprot.tableID
      where changeprot.changedtable = 'autokombidetails'
        and storeids.tableID is NULL and t.ID is NULL;

    -- gautocombidetails
    delete changeprot
      from changeprot
      join loggingtmp.delids using(changedtable, tableID)
      left join loggingtmp.storeids using(changedtable, tableID)
      left join gautocombidetails as t on t.ID = changeprot.tableID
      where changeprot.changedtable = 'gautocombidetails'
        and storeids.tableID is NULL and t.ID is NULL;

    -- ccdetails
    delete changeprot
      from changeprot
      join loggingtmp.delids using(changedtable, tableID)
      left join loggingtmp.storeids using(changedtable, tableID)
      left join ccdetails as t on t.ID = changeprot.tableID
      where changeprot.changedtable = 'ccdetails'
        and storeids.tableID is NULL and t.ID is NULL;

    -- autoccdetails
    delete changeprot
      from changeprot
      join loggingtmp.delids using(changedtable, tableID)
      left join loggingtmp.storeids using(changedtable, tableID)
      left join autoccdetails as t on t.ID = changeprot.tableID
      where changeprot.changedtable = 'autoccdetails'
        and storeids.tableID is NULL and t.ID is NULL;

    -- gautoccdetails
    delete changeprot
      from changeprot
      join loggingtmp.delids using(changedtable, tableID)
      left join loggingtmp.storeids using(changedtable, tableID)
      left join gautoccdetails as t on t.ID = changeprot.tableID
      where changeprot.changedtable = 'gautoccdetails'
        and storeids.tableID is NULL and t.ID is NULL;

    -- lwerteindizes
    delete changeprot
      from changeprot
      join loggingtmp.delids using(changedtable, tableID)
      left join loggingtmp.storeids using(changedtable, tableID)
      left join lwerteindizes as t on t.ID = changeprot.tableID
      where changeprot.changedtable = 'lwerteindizes'
        and storeids.tableID is NULL and t.ID is NULL;

    -- autokomprimierung
    delete changeprot
      from changeprot
      join loggingtmp.delids using(changedtable, tableID)
      left join loggingtmp.storeids using(changedtable, tableID)
      left join autokomprimierung as t on t.ID = changeprot.tableID
      where changeprot.changedtable = 'autokomprimierung'
        and storeids.tableID is NULL and t.ID is NULL;

    -- gautocompression
    delete changeprot
      from changeprot
      join loggingtmp.delids using(changedtable, tableID)
      left join loggingtmp.storeids using(changedtable, tableID)
      left join gautocompression as t on t.ID = changeprot.tableID
      where changeprot.changedtable = 'gautocompression'
        and storeids.tableID is NULL and t.ID is NULL;

    -- werteindizes
    delete changeprot
      from changeprot
      join loggingtmp.delids using(changedtable, tableID)
      left join loggingtmp.storeids using(changedtable, tableID)
      left join werteindizes as t on t.ID = changeprot.tableID
      where changeprot.changedtable = 'werteindizes'
        and storeids.tableID is NULL and t.ID is NULL;

    -- autoaufbewahrung
    delete changeprot
      from changeprot
      join loggingtmp.delids using(changedtable, tableID)
      left join loggingtmp.storeids using(changedtable, tableID)
      left join autoaufbewahrung as t on t.ID = changeprot.tableID
      where changeprot.changedtable = 'autoaufbewahrung'
        and storeids.tableID is NULL and t.ID is NULL;

    -- gautorettime
    delete changeprot
      from changeprot
      join loggingtmp.delids using(changedtable, tableID)
      left join loggingtmp.storeids using(changedtable, tableID)
      left join gautorettime as t on t.ID = changeprot.tableID
      where changeprot.changedtable = 'gautorettime'
        and storeids.tableID is NULL and t.ID is NULL;

    -- kautoaufbewahrung
    delete changeprot
      from changeprot
      join loggingtmp.delids using(changedtable, tableID)
      left join loggingtmp.storeids using(changedtable, tableID)
      left join kautoaufbewahrung as t on t.ID = changeprot.tableID
      where changeprot.changedtable = 'kautoaufbewahrung'
        and storeids.tableID is NULL and t.ID is NULL;

    -- gautocrettime
    delete changeprot
      from changeprot
      join loggingtmp.delids using(changedtable, tableID)
      left join loggingtmp.storeids using(changedtable, tableID)
      left join gautocrettime as t on t.ID = changeprot.tableID
      where changeprot.changedtable = 'gautocrettime'
        and storeids.tableID is NULL and t.ID is NULL;

    -- mailreports
    delete changeprot
      from changeprot
      join loggingtmp.delids using(changedtable, tableID)
      left join loggingtmp.storeids using(changedtable, tableID)
      left join mailreports as t on t.ID = changeprot.tableID
      where changeprot.changedtable = 'mailreports'
        and storeids.tableID is NULL and t.ID is NULL;

    -- mailreportsended
    delete changeprot
      from changeprot
      join loggingtmp.delids using(changedtable, tableID)
      left join loggingtmp.storeids using(changedtable, tableID)
      left join mailreports as t on t.ID = changeprot.tableID
      where changeprot.changedtable = 'mailreportsended'
        and storeids.tableID is NULL and t.ID is NULL;


    -- nun alle Einträge bis auf den neuesten Eintrag, der nicht mehr aufbewahrt werden
    -- muss, löschen, damit immer der vorherige Eintrag zum nicht zu löschenden vorhanden ist.
    delete changeprot
      from changeprot
      join loggingtmp.delids using(changedtable, tableID)
      where changeprot.TS < delids.mts;

    drop temporary table loggingtmp.delids, loggingtmp.storeids;

  end//

GRANT EXECUTE on PROCEDURE cleanchangeprot to loggingadmin@'localhost'//
