-- cleanvalues.sql
--
-- procedure cleanvalues()
-- 
-- remove old values, calculate retention time and call clean[cpstz]wertepart

set @@character_set_client = 'utf8';

delimiter //

DROP procedure IF EXISTS cleanvalues//
create procedure cleanvalues()
  deterministic
  sql security invoker
  begin

    create temporary table kwi
      ( primary key (ID) )
      select werteindizes.ID,
        if(lkdate = '0000-00-00', '0000-00-00', min(lkdate) + interval 1 day) as minlkdate
      from lwerteindizes
      join werteindizes using (serverID, sectionID, teststepID, teststepdetailsID, accessID)
      left join komprimierungids on komprimierungids.lwerteindizesID = lwerteindizes.ID
      where lwerteindizes.single = '1' OR komprimierungids.lwerteindizesID is not NULL
      group by werteindizes.ID;

    create temporary table vs (primary key(ID,verfahrenID))
      select ID, verfahrenID, servername from server;

    create temporary table winames (primary key(ID), key vid (verfahrenID,accessID))
      select w.ID, s.verfahrenID, w.accessID,
        s.servername, e.name as ename, t.name as tname,
        d.name as dname, u.name as uname
      from werteindizes as w
      join vs as s on s.ID = w.serverID
      join section as e on e.ID = w.sectionID
      join teststep as t on t.ID = w.teststepID
      join teststepdetails as d on d.ID = w.teststepdetailsID
      join einheiten as u on u.ID = d.einheitID;

    create temporary table va (primary key(ID))
      select w.ID, max(a.mindesttage) as mt
      from winames as w
      join autoaufbewahrung as a
        on a.verfahrenID = w.verfahrenID and a.accessID = w.accessID
          and a.mindesttage > 0 and a.aktiv = '1'
      where testreg(servername, a.server, a.server_v)
        and testreg(ename, a.section, a.section_v)
        and testreg(tname, a.teststep, a.teststep_v)
        and testreg(dname, a.details, a.details_v)
        and testreg(uname, a.einheit, a.einheit_v)
      group by w.ID;

    create temporary table ga (primary key(ID))
      select w.ID, max(g.mindesttage) as mt
      from winames as w
      join gruppenrechte as gr
        on gr.admin = '1' and gr.verfahrenID = w.verfahrenID and gr.accessID = w.accessID
      join verfahren on verfahren.ID = w.verfahrenID
      join gautorettime as g
        on g.groupID = gr.gruppenID and g.accessID = w.accessID
          and g.mindesttage > 0 and g.aktiv = '1'
      where testreg(verfahren.name, g.domain, g.domain_v)
        and testreg(servername, g.server, g.server_v)
        and testreg(ename, g.section, g.section_v)
        and testreg(tname, g.teststep, g.teststep_v)
        and testreg(dname, g.details, g.details_v)
        and testreg(uname, g.einheit, g.einheit_v)
      group by w.ID;

    drop temporary table vs, winames;

    create temporary table wi (primary key(id), key td(testdatum,ID))
      select w.ID, wtab,
        case when w.mindesttage = 0 then curdate() + interval 1 day
          when w.mtset = '1' then
            least(date(now() + interval 6 hour)
                  - interval greatest(w.mindesttage, coalesce(va.mt, 0), coalesce(ga.mt, 0)) day,
                  coalesce(kwi.minlkdate, curdate()))
          else
            least(date(now() + interval 6 hour)
                  - interval greatest(coalesce(va.mt, ga.mt, w.mindesttage), coalesce(va.mt, 0), coalesce(ga.mt, 0)) day,
                  coalesce(kwi.minlkdate, curdate()))
          end as testdatum
      from werteindizes as w
      join teststepdetails on teststepdetailsID = teststepdetails.ID
      join einheiten on einheitID = einheiten.ID
      left join kwi on kwi.ID = w.ID
      left join va on va.ID = w.ID
      left join ga on ga.ID = w.ID;

    set @gfid = (select ID from groupfunctions where code = 'diffto');
    create temporary table difftos (primary key(ID, dt))
      select werteindizesID as ID, wertelimits.anzmittel as dt, date(wertelimits.anzmittel) as d
      from wertelimits
      where groupfunctionsID = @gfid
      group by werteindizesID, wertelimits.anzmittel;

    call cleandatetables();

    call cleancwertepart();
    call cleanpwertepart();
    call cleanswertepart();
    call cleantwertepart();
    call cleanzwertepart();

    call cleanbwerte();

    drop temporary table wi, kwi, va, ga;
  end//

GRANT EXECUTE on PROCEDURE cleanvalues to loggingadmin@'localhost'//
