-- convert_ccombi.sql
--
-- procedure convert_ccombi(srvid integer, secid integer, tstid integer, aid integer, s, gwb, gwd, out q, out fdwid integer, out ctype enum('num','str','dat','blob'), out constvars, out lwids)
-- srvid: serverID
-- secid: sectionID
-- tstid: teststepID
-- aid: accessID
-- s: expression
-- gwb: 'b' for kombidetails else ''
-- gwd: ',datum' for kombidetails else ''
-- q: converted expression as sql
-- fdwid: first werteindizesID
-- ctype: type of result
-- constvars: needed constant variables (aggregatet values for the whole timerange)
-- lwids: used lwerteindizesID
--
-- convert compressed combidetail or compressed combilimit expression

set @@character_set_client = 'utf8';

delimiter //

DROP procedure IF EXISTS convert_ccombi//
create procedure convert_ccombi(srvid integer unsigned, secid integer unsigned,
    tstid integer unsigned, aid integer unsigned, s varchar(3000), gwb varchar(1),
    gwd varchar(6), OUT q varchar(5000), OUT fdwid integer unsigned,
    OUT ctype enum('num','str','dat','blob'), OUT constvars varchar(3000), OUT lwids varchar(2000))
  deterministic
  sql security invoker
 begin

  declare di, d, e, l, pro, back, i, slen, rsrvid, rsecid, rtstid integer;
  declare code, sdf, ADN varchar(20);
  declare p1, p2, p3, p, srv varchar(50);
  declare rp1, rp2, rp3 integer;
  declare tst varchar(200);
  declare ds varchar(600);
  declare rds varchar(310);
  declare sec, eh, neh, df varchar(100);
  declare ftype, utype enum('num','str','dat','blob');
  declare sel varchar(150);

  set constvars = '';
  set lwids = '';

  if s REGEXP 'into |create |drop |alter |insert |select|delete |update |from |call |;|:='
   then
    set q = '';
  else
    set fdwid = 0;
    set ctype = 'num';
    set q = '';
    set i = 1;
    set e = 1;
    set slen = char_length(s);
    while e <= slen
     do
      set d = locate('$', s, e);
      if d > 0 then
        set q = concat(q, substring(s, e, d-e));
        set e = locate('(', s, d);
        if e > 0 then
          set code = substring(s, d+1, e-d-1);
          set e = e+1;
          call getpar(s, e, ds, i);
          if i > 0 then
            call getpar(s, e, p1, i);
          else
            set p1 = '';
          end if;
          if i > 0 then
            call getpar(s, e, p2, i);
          else
            set p2 = '';
          end if;
          if i > 0 then
            call getpar(s, e, p3, i);
          else
            set p3 = '';
          end if;
          while i > 0 do
            call getpar(s, e, p, i);
          end while;
          if ds REGEXP '^\\|' then
            set rp1 = locate('|', ds, 2);
            set srv = substring(ds, 2, rp1-2);
            set rp2 = locate('|', ds, rp1+1);
            set sec = substring(ds, rp1+1, rp2-rp1-1);
            set rp3 = locate('|', ds, rp2+1);
            set tst = substring(ds, rp2+1, rp3-rp2-1);
            set rds = substring(ds, rp3+1);
            if srv = '' then
              set rsrvid = srvid;
            else
              set rsrvid = (select ss.ID from server ss, server so
                            where so.ID = srvid and ss.verfahrenID = so.verfahrenID
                              and ss.servername = srv);
            end if;
            if sec = '' then
              set rsecid = secid;
            else
              set rsecid = (select ID from section where bname = sec OR name = sec
                            order by bname != sec limit 1);
            end if;
            if tst = '' then
              set rtstid = tstid;
            else
              set rtstid = (select ID from teststep where bname = tst OR name = tst
                            order by bname != tst limit 1);
            end if;
          else
            set rsrvid = srvid;
            set rsecid = secid;
            set rtstid = tstid;
            set rds = ds;
          end if;
          set l = locate('@', rds);
          if l > 0 then
            set neh = substring(rds, l+1);
            set rds = left(rds, l-1);
          else
            set neh = '';
          end if;
          set l = locate('/', rds);
          set di = 0;
          if l > 0 then
            set eh = substring(rds, l+1);
            set rds = left(rds, l-1);
            select lwerteindizes.ID, type into di, utype
              from lwerteindizes, teststepdetails, einheiten, groupfunctions
              where serverID = rsrvid and sectionID = rsecid
                and teststepID = rtstid and teststepdetailsID = teststepdetails.ID
                and concat(groupfunctions.code, '[', left(period,1), '](',
                     teststepdetails.name, ')') = rds
                and einheiten.ID = coalesce(lwerteindizes.unitID, einheitID)
                and einheiten.name = eh and accessID = aid
                and groupfunctions.ID = lwerteindizes.groupfunctionsID
              order by einheiten.bname != eh
              limit 1;
          else
            select lwerteindizes.ID, type into di, utype
              from lwerteindizes, teststepdetails, einheiten, groupfunctions
              where serverID = rsrvid and sectionID = rsecid
                and teststepID = rtstid and teststepdetailsID = teststepdetails.ID
                and concat(groupfunctions.code, '[', left(period,1), '](',
                     teststepdetails.name, ')') = rds
                and einheiten.ID = coalesce(lwerteindizes.unitID, einheitID)
                and accessID = aid and groupfunctions.ID = lwerteindizes.groupfunctionsID
              limit 1;
          end if;
          if di > 0 then
            if fdwid = 0 then
              set fdwid = di;
            end if;
            if NOT find_in_set(di, lwids) then
              set lwids = concat(lwids, ',', di);
            end if;
            if utype != 'num' then
              set ADN = '';
            else
              set ADN = '0+';
            end if;
            if neh != '' then
              if (select einheiten.name = '[E]'
                    from lwerteindizes, teststepdetails, einheiten
                    where lwerteindizes.ID = di and teststepdetails.ID = teststepdetailsID
                      and einheiten.ID = einheitID)
               then
                set df = concat(", '", neh, "')");
                set sdf = "ehwconv(";
              else
                select concat('*',eho.sifaktor/ehn.sifaktor) into df
                  from lwerteindizes, teststepdetails, einheiten as eho, einheiten as ehn
                  where lwerteindizes.ID = di and teststepdetails.ID = teststepdetailsID and eho.ID=einheitID
                    and eho.sifaktor > 0 and ehn.name = neh
                    and eho.einheitenID = ehn.einheitenID
                    and ehn.sifaktor > 0;
                set sdf = '';
              end if;
            else
              set df = '';
              set sdf = '';
            end if;
            set sel = '';
            case code
            when 'ACCEL_OF' then
              set ftype = 'num';
              if p1 != '' AND (gwb = '' OR p1 REGEXP '[hmds]') then
                set sel = concat("(0+grouplwert",gwb,"('",di,"'",gwd,",'",p1,"','accel')",df,")");
              elseif gwb != '' AND p1 = '' then
                set sel = concat(" @caccel",di,df," ");
                if constvars not like concat('%caccel:' , di, '%') then
                  set constvars = concat('caccel:', di, ' ', constvars);
                end if;
              end if;
            when 'AVERAGE_OF' then
              set ftype = 'num';
              if p1 != '' AND (gwb = '' OR p1 REGEXP '[hmds]') then
                set sel = concat("(0+grouplwert",gwb,"('",di,"'",gwd,",'",p1,"','avg')",df,")");
              elseif gwb != '' AND p1 = '' then
                set sel = concat(" @cavg",di,df," ");
                if constvars not like concat('%cavg:' , di, '%') then
                  set constvars = concat('cavg:', di, ' ', constvars);
                end if;
              end if;
            when 'RATIO_LAST2AVG' then
              set ftype = 'num';
              if p1 != '' AND (gwb = '' OR p1 REGEXP '[hmds]') then
                set sel = concat("(0+grouplwert",gwb,"('",di,"'",gwd,",'",p1,"','last/avg')",df,")");
              end if;
            when 'RATIO_LAST2AVG-1' then
              set ftype = 'num';
              if p1 != '' AND (gwb = '' OR p1 REGEXP '[hmds]') then
                set sel = concat("(0+grouplwert",gwb,"('",di,"'",gwd,",'",p1,"','last/avg-1')",df,")");
              end if;
            when 'COUNT_OF' then
              set ftype = 'num';
              if p1 != '' AND (gwb = '' OR p1 REGEXP '[hmds]') then
                set sel = concat("(0+grouplwert",gwb,"('",di,"'",gwd,",'",p1,"','count'))");
              elseif gwb != '' AND p1 = '' then
                set sel = concat(" @ccount",di,df," ");
                if constvars not like concat('%ccount:' , di, '%') then
                  set constvars = concat('ccount:', di, ' ', constvars);
                end if;
              end if;
            when 'DATETIME_OF' then
              set ftype = 'dat';
              if gwb != '' then
                set sel = 'datum';
              end if;
            when 'DEV_OF' then
              set ftype = 'num';
              if p1 != '' AND (gwb = '' OR p1 REGEXP '[hmds]') then
                set sel = concat("(0+grouplwert",gwb,"('",di,"'",gwd,",'",p1,"','dev')",df,")");
              elseif gwb != '' AND p1 = '' then
                set sel = concat(" @cdevi",di,df," ");
                if constvars not like concat('%cdevi:' , di, '%') then
                  set constvars = concat('cdevi:', di, ' ', constvars);
                end if;
              end if;
            when 'DIFFER_OF' then
              set ftype = 'num';
              if p1 != '' AND (gwb = '' OR p1 REGEXP '[hmds]') then
                set sel = concat("(0+grouplwert",gwb,"('",di,"'",gwd,",'",p1,"','differ'))");
              else
                set sel = concat("(0+modewert",gwb,"('",di,"'",gwd,",2,'differ'))");

              end if;
            when 'FIRST_OF' then
              set ftype = utype;
              if p1 != '' AND (gwb = '' OR p1 REGEXP '[hmds]') then
                set sel = concat("(",ADN,sdf,"grouplwert",gwb,"('",di,"'",gwd,",'",p1,"','first')",df,")");
              elseif gwb != '' AND p1 = '' then
                set sel = concat(" @cfirst",di,df," ");
                if constvars not like concat('%cfirst:' , di, '%') then
                  set constvars = concat('cfirst:', di, ' ', constvars);
                end if;
              end if;
            when 'FIRST_SIZE_OF' then
              set ftype = 'num';
              if p1 != '' AND (gwb = '' OR p1 REGEXP '[hmds]') then
                set sel = concat("(0+grouplwert",gwb,"('",di,"'",gwd,",'",p1,"','firstsize')",df,")");
              elseif gwb != '' AND p1 = '' then
                set sel = concat(" @cfirstsize",di,df," ");
                if constvars not like concat('%cfirstsize:' , di, '%') then
                  set constvars = concat('cfirstsize:', di, ' ', constvars);
                end if;
              end if;
            when 'GRAD_OF' then
              set ftype = 'num';
              if p1 != '' AND (gwb = '' OR p1 REGEXP '[hmds]') then
                set sel = concat("(0+grouplwert",gwb,"('",di,"'",gwd,",'",p1,"','grad')",df,")");
              elseif gwb != '' AND p1 = '' then
                set sel = concat(" @cgrad",di,df," ");
                if constvars not like concat('%cgrad:' , di, '%') then
                  set constvars = concat('cgrad:', di, ' ', constvars);
                end if;
              end if;
            when 'GRADIENT' then
              set ftype = 'num';
              if p1 != '' then
                if p1 like '%h%' then
                  set pro = p1+0;
                else
                  set pro = (p1+0)*24;
                end if;
                if p2 != '' then
                  if p2 like '%h%' then
                    set back = p2+0;
                  else
                    set back = (p2+0)*24;
                  end if;
                else
                  set back = 24;
                end if;
              end if;
              set sel = concat("(gradient('",di,"',",back,",",pro,"))");
            when 'LAST_OF' then
              set ftype = utype;
              if p1 != '' AND (gwb = '' OR p1 REGEXP '[hmds]') then
                set sel = concat("(",ADN,sdf,"grouplwert",gwb,"('",di,"'",gwd,",'",p1,"','last')",df,")");
              elseif gwb != '' AND p1 = '' then
                set sel = concat(" @clast",di,df," ");
                if constvars not like concat('%clast:' , di, '%') then
                  set constvars = concat('clast:', di, ' ', constvars);
                end if;
              end if;
            when 'LAST-FIRST_OF' then
              set ftype = 'num';
              if p1 != '' AND (gwb = '' OR p1 REGEXP '[hmds]') then
                set sel = concat("(0+grouplwert",gwb,"('",di,"'",gwd,",'",p1,"','last-first')",df,")");
              elseif gwb != '' AND p1 = '' then
                set sel = concat(" @clmf",di,df," ");
                if constvars not like concat('%clmf:' , di, '%') then
                  set constvars = concat('clmf:', di, ' ', constvars);
                end if;
              end if;
            when 'MAX_OF' then
              set ftype = utype;
              if p1 != '' AND (gwb = '' OR p1 REGEXP '[hmds]') then
                set sel = concat("(",ADN,"grouplwert",gwb,"('",di,"'",gwd,",'",p1,"','max')",df,")");
              elseif gwb != '' AND p1 = '' then
                set sel = concat(" @cmax",di,df," ");
                if constvars not like concat('%cmax:' , di, '%') then
                  set constvars = concat('cmax:', di, ' ', constvars);
                end if;
              end if;
            when 'MIN_OF' then
              set ftype = utype;
              if p1 != '' AND (gwb = '' OR p1 REGEXP '[hmds]') then
                set sel = concat("(",ADN,"grouplwert",gwb,"('",di,"'",gwd,",'",p1,"','min')",df,")");
              elseif gwb != '' AND p1 = '' then
                set sel = concat(" @cmin",di,df," ");
                if constvars not like concat('%cmin:' , di, '%') then
                  set constvars = concat('cmin:', di, ' ', constvars);
                end if;
              end if;
            when 'MAX-MIN_OF' then
              set ftype = 'num';
              if p1 != '' AND (gwb = '' OR p1 REGEXP '[hmds]') then
                set sel = concat("(0+grouplwert",gwb,"('",di,"'",gwd,",'",p1,"','max-min')",df,")");
              elseif gwb != '' AND p1 = '' then
                set sel = concat(" @cmmm",di,df," ");
                if constvars not like concat('%cmmm:' , di, '%') then
                  set constvars = concat('cmmm:', di, ' ', constvars);
                end if;
              end if;
            when 'PREDICTION' then
              set ftype = 'num';
              if p1 != '' then
                if p1 like '%h%' then
                  set pro = p1+0;
                else
                  set pro = (p1+0)*24;
                end if;
                if p2 != '' then
                  if p2 like '%h%' then
                    set back = p2+0;
                  else
                    set back = (p2+0)*24;
                  end if;
                else
                  set back = 24;
                end if;
              end if;
              set sel = concat("(predictionl',gwb,'('",di,"'",gwd,",",back,",",pro,")",df,")");
            when 'PREV_OF' then
              set ftype = utype;
              set sel = concat("(",ADN,sdf,"modelwert",gwb,"('",di,"'",gwd,",2,'first')",df,")");
            when 'PREV_SIZE_OF' then
              set ftype = 'num';
              set sel = concat("(0+modelwert",gwb,"('",di,"'",gwd,",2,'firstsize')",df,")");
            when 'REGCOUNT' then
              set ftype = 'num';
              if p2 != '' && gwb = '' then
                set sel = concat("(regcountl('",di,"','",p1,"','",p2,"'))");
              end if;
            when 'SIZE_OF' then
              set ftype = 'num';
              set sel = concat("(0+modelwert",gwb,"('",di,"'",gwd,",1,'size')",df,")");
            when 'SUM_OF' then
              set ftype = 'num';
              if p1 != '' AND (gwb = '' OR p1 REGEXP '[hmds]') then
                set sel = concat("(0+grouplwert",gwb,"('",di,"'",gwd,",'",p1,"','sum')",df,")");
              elseif gwb != '' AND p1 = '' then
                set sel = concat(" @csum",di,df," ");
                if constvars not like concat('%csum:' , di, '%') then
                  set constvars = concat('csum:', di, ' ', constvars);
                end if;
              end if;
            when 'VALUE_OF' then
              set ftype = utype;
              if gwb != '' then
                set sel = concat("(",ADN,sdf,"getlwertvb('",di,"',datum,datum,'last')",df,")");
              else
                set sel = concat("(",ADN,sdf,"grouplwert",gwb,"('",di,"'",gwd,",'",p1,"','last')",df,")");
              end if;
            when 'ACTUAL_OF' then
              set ftype = utype;
              set sel = concat("(",ADN,sdf,"grouplwert",gwb,"('",di,"'",gwd,",'",p1,"','last')",df,")");
            else
              set sel = '';
            end case;
            if sel != '' then
              set q = concat(q, sel);
              if ftype != 'num' then
                set ctype = 'str';
              end if;
            end if;
          else
            set slen = 0;
          end if;
        else
          set slen = 0;
        end if;
      else
        set q = concat(q, substring(s, e));
        set e = slen+1;
      end if;
    end while;
    if slen = 0 then
      set q = '';
    end if;
  end if;
  set constvars = rtrim(constvars);
end//

GRANT EXECUTE on PROCEDURE convert_ccombi
  to logginglesen@'localhost', loggingadmin@'localhost', logging@'localhost'//
