-- createleveltable.sql
--
-- procedure createleveltable(wid, fromdt, todt, relation, infov, warnv, errorv, excludev)
-- wid: werteindizesID
-- fromdt: datetime from
-- todt: datetime to
-- relation: relation for compare
-- infov: info limit
-- warnv: warn limit
-- errorv: error limit
-- excludev: exclude

set @@character_set_client = 'utf8';

delimiter //

DROP procedure IF EXISTS createleveltable//
create procedure createleveltable(wid integer, fromdt datetime, todt datetime, relation varchar(15),
                                  infov varchar(2000), warnv varchar(2000), errorv varchar(2000),
                                  excludev varchar(4000))
deterministic
sql security invoker
begin
  declare wt, w, f, fs varchar(20);
  declare sel varchar(500);

  select wtab into wt from werteindizes, teststepdetails, einheiten
    where werteindizes.ID = wid and teststepdetailsID = teststepdetails.ID and einheitID = einheiten.ID;

  if fromdt = '' then
    set fromdt = '0000-00-00 00:00:00';
  end if;
  if todt = '' then
    set todt = now() + interval 1 year;
  end if;

  if find_in_set(wt, 'swerte,twerte') then
    set f = ', id';
    set fs = '';
    if relation like '%|' then
      set w = 'wert ';
      set relation = replace(relation, '|', '');
    else
      set w = 'binary wert ';
    end if;
  else
    set f = '';
    set fs = '0 as ';
    if relation regexp '==|~' OR find_in_set(wt, 'swerte,twerte') then
      if relation like '%|' then
        set w = 'wert ';
        set relation = replace(relation, '|', '');
      else
        set w = 'binary wert ';
      end if;
    else
      if relation like '%|' then
        set w = 'abs(0+wert) ';
        set relation = replace(relation, '|', '');
      else
        set w = '0+wert ';
      end if;
    end if;
  end if;

  if relation = '==' then
    set relation = '=';
  elseif relation = '!==' then
    set relation = '!=';
  elseif relation = '~' then
    set relation = 'regexp';
  elseif relation = '!~' then
    set relation = 'not regexp';
  elseif relation = '~~' then
    set relation = 'like';
  elseif relation = '!~~' then
    set relation = 'not like';
  end if;

  if relation = 'like' OR relation = 'not like' then
    set sel = concat('create temporary table loggingtmp.limitlevels (primary key(werteindizesID, datum', f, ')) select werteindizesID, datum', f, ', max(case when ? != \'\' AND ', w, relation,
      ' concat(\'%\', ?, \'%\') then 0 when ? != \'\' AND ', w, relation, ' concat(\'%\', ?, \'%\') then 3 when ? != \'\' AND ', w, relation,
      ' concat(\'%\', ?, \'%\') then 2 when ? != \'\' AND ', w, relation, ' concat(\'%\', ?, \'%\') then 1 else 0 end) as limlevel from ', wt,
      ' where werteindizesID = ? and ? <= datum and datum <= ? group by werteindizesID, datum', f);
  else
    set sel = concat('create temporary table loggingtmp.limitlevels (primary key(werteindizesID, datum, id)) select werteindizesID, datum, ', fs, 'id, max(case when ? != \'\' AND ', w, relation,
      ' ? then 0 when ? != \'\' AND ', w, relation, ' ? then 3 when ? != \'\' AND ', w, relation,
      ' ? then 2 when ? != \'\' AND ', w, relation, ' ? then 1 else 0 end) as limlevel from ', wt,
      ' where werteindizesID = ? and ? <= datum and datum <= ? group by werteindizesID, datum', f);
  end if;

  prepare prepsel from sel;
  execute prepsel using excludev, excludev, errorv, errorv, warnv, warnv, infov, infov, wid, fromdt, todt;
  drop prepare prepsel;

end//
