-- ehwconv.sql
--
-- function ehwconv(ehw, neweh) returns varchar
-- ehw: value with unit (<value> <unit>) ULS unit [E]
-- neweh: target unit
-- return: <value> in new unit without unitname

-- function cehwconv(ehw, neweh) returns varchar
-- ehw: value with unit (<value> <unit>) ULS unit [E]
-- neweh: target unit
-- return: <value> in new unit without unitname
--
-- function for compressed values


set @@character_set_client = 'utf8';

delimiter //
DROP FUNCTION IF EXISTS ehwconv//
create function ehwconv(ehw varbinary(200), neweh varbinary(100)) returns varchar(30)
  deterministic
  sql security invoker
  begin
    declare w, fv, ov, fz, oz double;
    declare e varchar(100);
    declare i integer;

    select instr(ehw, ' ') into i;
    if i = 0
     then
      return '';
    else
      select left(ehw, i-1) + 0.0 into w;
      select substr(ehw, i+1) into e;
      select sifaktor, sioffset into fv, ov from einheiten where bname = e and sifaktor > 0;
      select sifaktor, sioffset into fz, oz from einheiten where bname = neweh and sifaktor > 0;

      if fv is NULL OR fz is NULL
       then
        return '';
      else
        return (w*fv+ov-oz)/fz;
      end if;
    end if;
  end//

GRANT EXECUTE on FUNCTION ehwconv
  to logginglesen@'localhost', loggingtest@'localhost', 'loggingadmin'@'localhost'//


--
-- cehwconv: Einheitenkonvertierung bei komprimierten Werten,
--           bei grad oder accel müssen /h oder /h² beachtet werden.

DROP FUNCTION IF EXISTS cehwconv//
create function cehwconv(ehw varbinary(200), neweh varbinary(100)) returns varchar(30)
  deterministic
  sql security invoker
  begin
    declare w, fv, ov, fz, oz double;
    declare e varbinary(100);
    declare i integer;

    set i = instr(ehw, ' ');
    if i = 0
     then
      return '';
    else
      set w = left(ehw, i-1) + 0.0;
      set e = substr(ehw, i+1);
      
      select sifaktor, sioffset into fv, ov from einheiten
        where sifaktor > 0
          and (bname = e OR concat(bname, '/h') = e OR concat(bname, '/h²') = e)
        order by bname != e, concat(bname, '/h') != e
        limit 1;
      select sifaktor, sioffset into fz, oz from einheiten
        where sifaktor > 0
          and (bname = neweh OR concat(bname, '/h') = neweh OR concat(bname, '/h²') = neweh)
        order by bname != neweh, concat(bname, '/h') != neweh
        limit 1;

      if fv is NULL OR fz is NULL
       then
        return '';
      else
        return (w*fv+ov-oz)/fz;
      end if;
    end if;
  end//

GRANT EXECUTE on FUNCTION cehwconv
  to logginglesen@'localhost', loggingtest@'localhost', 'loggingadmin'@'localhost'//
