set @@character_set_client = 'utf8';

delimiter //

drop trigger if exists gautocrettime_au//
create trigger gautocrettime_au after update on gautocrettime
  for each row 
begin
  insert into changeprot
    (groupID, accessID, changedtable, tableID, userID, username, action,
     domain, server, section, teststep, details, unit, newvalues)
  select NEW.groupID, NEW.accessID, 'gautocrettime', NEW.ID,
    coalesce(@userid,0), coalesce(@username,'SYSTEM'), 'update',
    showreg(NEW.domain, NEW.domain_v), showreg(NEW.server, NEW.server_v),
    showreg(NEW.section, NEW.section_v), showreg(NEW.teststep, NEW.teststep_v),
    showreg(NEW.details, NEW.details_v), showreg(NEW.einheit, NEW.einheit_v),
    concat('compression=',
      coalesce((select group_concat(code order by g.ID separator ',')
                from groupfunctions as g
                where find_in_set(g.ID, NEW.groupfunctionIDs)),''),
      '\nperiod=', NEW.period, '\ndays=', NEW.mindesttage, '\nremark=',
      REPLACE(REPLACE(NEW.remark, '\\', '\\\\'), '\n', '\\n'))
  from dual
  where NEW.domain != OLD.domain OR NEW.domain_v != OLD.domain_v
    OR NEW.server != OLD.server OR NEW.server_v != OLD.server_v
    OR NEW.section != OLD.section OR NEW.section_v != OLD.section_v
    OR NEW.teststep != OLD.teststep OR NEW.teststep_v != OLD.teststep_v
    OR NEW.details != OLD.details OR NEW.details_v != OLD.details_v
    OR NEW.einheit != OLD.einheit OR NEW.einheit_v != OLD.einheit_v
    OR NEW.groupfunctionIDs != OLD.groupfunctionIDs OR NEW.period != OLD.period
    OR NEW.mindesttage != OLD.mindesttage OR NEW.accessID != OLD.accessID
    OR NEW.remark != OLD.remark;
end//


drop trigger if exists gautocrettime_ai//
create trigger gautocrettime_ai after insert on gautocrettime
  for each row 
begin
  insert into changeprot
    (groupID, accessID, changedtable, tableID, userID, username, action,
     domain, server, section, teststep, details, unit, newvalues)
  values
    (NEW.groupID, NEW.accessID, 'gautocrettime', NEW.ID,
     coalesce(@userid,0), coalesce(@username,'SYSTEM'), 'insert',
     showreg(NEW.domain, NEW.domain_v), showreg(NEW.server, NEW.server_v),
     showreg(NEW.section, NEW.section_v), showreg(NEW.teststep, NEW.teststep_v),
     showreg(NEW.details, NEW.details_v), showreg(NEW.einheit, NEW.einheit_v),
     concat('compression=',
       coalesce((select group_concat(code order by g.ID separator ',')
                 from groupfunctions as g
                 where find_in_set(g.ID, NEW.groupfunctionIDs)),''),
       '\nperiod=', NEW.period, '\ndays=', NEW.mindesttage, '\nremark=',
       REPLACE(REPLACE(NEW.remark, '\\', '\\\\'), '\n', '\\n')));
end//


drop trigger if exists gautocrettime_bd//
create trigger gautocrettime_bd before delete on gautocrettime
  for each row 
begin
  insert into changeprot
    (groupID, accessID, changedtable, tableID, userID, username, action,
     domain, server, section, teststep, details, unit, newvalues)
  values
    (OLD.groupID, OLD.accessID, 'gautocrettime', OLD.ID,
     coalesce(@userid,0), coalesce(@username,'SYSTEM'), 'delete',
     showreg(OLD.domain, OLD.domain_v), showreg(OLD.server, OLD.server_v),
     showreg(OLD.section, OLD.section_v), showreg(OLD.teststep, OLD.teststep_v),
     showreg(OLD.details, OLD.details_v), showreg(OLD.einheit, OLD.einheit_v),
     coalesce(if(OLD.remark = @comprettimeremark, '', concat('remark=', REPLACE(REPLACE(@comprettimeremark, '\\', '\\\\'), '\n', '\\n'))),''));
end//

delimiter ;
