set @@character_set_client = 'utf8';

delimiter //

drop trigger if exists gautolimits_au//
create trigger gautolimits_au after update on gautolimits
  for each row
begin
  insert into changeprot
    (groupID, accessID, changedtable, tableID, userID, username, action,
     domain, classes, server, section, teststep, details, unit, newvalues)
  select groupID, accessID, 'gautolimits', gautolimits.ID,
    coalesce(@userid,0), coalesce(@username,'SYSTEM'), 'update',
    showreg(domain, domain_v), showreg(classes, classes_v), showreg(server, server_v),
    showreg(section, section_v), showreg(teststep, teststep_v),
    showreg(details, details_v), showreg(einheit, einheit_v),
    concat('function=', g.code, '\nover=', anzmittel, '\nrelation=', vergleich,
           '\ninfolevel=', infowert, '\nwarnlevel=', warnwert, '\nerrorlevel=',
           errorwert, '\nexclude=', exclude, '\nnotifymode=', notifymode, '\nmessage=',
           REPLACE(REPLACE(meldung, '\\', '\\\\'), '\n', '\\n'), '\ndestination1=',
           notifdestname(meldungszielID), '\ndest1levels=',
           if(find_in_set('INFO', ziel1level), 'I', ''),
           if(find_in_set('WARN', ziel1level), 'W', ''),
           if(find_in_set('ERROR', ziel1level), 'F', ''), '\ndestination2=',
           notifdestname(meldungsziel2ID), '\ndest2levels=',
           if(find_in_set('INFO', ziel2level), 'I', ''),
           if(find_in_set('WARN', ziel2level), 'W', ''),
           if(find_in_set('ERROR', ziel2level), 'F', ''), '\nremark=',
           REPLACE(REPLACE(bemerkungen, '\\', '\\\\'), '\n', '\\n'), '\nonlyifnoother=', nootherflag, '\nactive=', aktiv)
  from gautolimits
  join groupfunctions as g on g.ID = groupfunctionsID
  where gautolimits.ID = NEW.ID and gautolimits.TS = NEW.TS;
end//


drop trigger if exists gautolimits_ai//
create trigger gautolimits_ai after insert on gautolimits
  for each row
begin
  insert into changeprot
    (groupID, accessID, changedtable, tableID, userID, username, action,
     domain, classes, server, section, teststep, details, unit, newvalues)
  select NEW.groupID, NEW.accessID, 'gautolimits', NEW.ID,
    coalesce(@userid,0), coalesce(@username,'SYSTEM'), 'insert',
    showreg(NEW.domain, NEW.domain_v), showreg(NEW.classes, NEW.classes_v), showreg(NEW.server, NEW.server_v),
    showreg(NEW.section, NEW.section_v), showreg(NEW.teststep, NEW.teststep_v),
    showreg(NEW.details, NEW.details_v), showreg(NEW.einheit, NEW.einheit_v),
    concat('function=', g.code, '\nover=', NEW.anzmittel, '\nrelation=', NEW.vergleich,
           '\ninfolevel=', NEW.infowert, '\nwarnlevel=', NEW.warnwert, '\nerrorlevel=',
           NEW.errorwert, '\nexclude=', NEW.exclude, '\nnotifymode=', NEW.notifymode, '\nmessage=',
           REPLACE(REPLACE(NEW.meldung, '\\', '\\\\'), '\n', '\\n'), '\ndestination1=',
           notifdestname(NEW.meldungszielID), '\ndest1levels=',
           if(find_in_set('INFO', NEW.ziel1level), 'I', ''),
           if(find_in_set('WARN', NEW.ziel1level), 'W', ''),
           if(find_in_set('ERROR', NEW.ziel1level), 'F', ''), '\ndestination2=',
           notifdestname(NEW.meldungsziel2ID), '\ndest2levels=',
           if(find_in_set('INFO', NEW.ziel2level), 'I', ''),
           if(find_in_set('WARN', NEW.ziel2level), 'W', ''),
           if(find_in_set('ERROR', NEW.ziel2level), 'F', ''), '\nremark=',
           REPLACE(REPLACE(NEW.bemerkungen, '\\', '\\\\'), '\n', '\\n'), '\nonlyifnoother=', NEW.nootherflag, '\nactive=', NEW.aktiv)
  from groupfunctions as g
  where g.ID = NEW.groupfunctionsID;
end//


drop trigger if exists gautolimits_bd//
create trigger gautolimits_bd before delete on gautolimits
  for each row
begin
  insert into changeprot
    (groupID, accessID, changedtable, tableID, userID, username, action,
     domain, classes, server, section, teststep, details, unit, newvalues)
  values
   (OLD.groupID, OLD.accessID, 'gautolimits', OLD.ID,
    coalesce(@userid,0), coalesce(@username,'SYSTEM'), 'delete',
    showreg(OLD.domain, OLD.domain_v), showreg(OLD.classes, OLD.classes_v), showreg(OLD.server, OLD.server_v),
    showreg(OLD.section, OLD.section_v), showreg(OLD.teststep, OLD.teststep_v),
    showreg(OLD.details, OLD.details_v), showreg(OLD.einheit, OLD.einheit_v),
    coalesce(if(OLD.bemerkungen = @limitsremark, '', concat('remark=', REPLACE(REPLACE(@limitsremark, '\\', '\\\\'), '\n', '\\n'))),''));
end//

delimiter ;
