-- getbwertvb.sql
--
-- function getbwertvb(wid, von, bis, mode) returns varchar
-- wid: werteindizesID
-- von: datetime from
-- bis: datetime to
-- mode: aggregation function
-- return: aggragated value for binary data (table bwerte)

set @@character_set_client = 'utf8';

delimiter //

DROP FUNCTION IF EXISTS getbwertvb//
create function getbwertvb(wid integer, von datetime, bis datetime, mode varchar(12))
                  returns varchar(255)
deterministic
sql security invoker
begin
  case mode
    when 'count' then
      return (select count(*) from bwerte
              where werteindizesID = wid and von <= datum and datum <= bis);
    when 'size' then
      return (select length(daten) from bwerte, daten
              where werteindizesID = wid and von <= datum and datum <= bis
                and daten.ID=datenID
              order by datum desc, bwerte.ID desc limit 1);
    when 'firstsize' then
      return (select length(daten) from bwerte, daten
              where werteindizesID = wid and von <= datum and datum <= bis
                and daten.ID=datenID
              order by datum, bwerte.ID limit 1);
    when 'lastsize' then
      return (select length(daten) from bwerte, daten
              where werteindizesID = wid and von <= datum and datum <= bis
                and daten.ID=datenID
              order by datum desc, bwerte.ID desc limit 1);
    when 'differ' then
      return (select datenID from bwerte
              where werteindizesID = wid and von <= datum and datum <= bis
              order by datum desc, ID desc limit 1)
          != (select datenID from bwerte
              where werteindizesID = wid and von <= datum and datum <= bis
              order by datum, ID limit 1);
    when 'last' then
      return (select dateiname from bwerte
              where werteindizesID = wid and von <= datum and datum <= bis
              order by datum desc, ID desc limit 1);
    else
      return '';
  end case;
end//

GRANT EXECUTE on FUNCTION getbwertvb to logginglesen@'localhost', loggingtest@'localhost', loggingadmin@'localhost'//
