-- getbwertvbz.sql
--
-- function getbwertvbz(wid, von, bis, mode, zvon, zbis) returns varchar
-- wid: werteindizesID
-- von: datetime from
-- bis: datetime to
-- mode: aggregation function
-- zvon: time from
-- zbis: time to
-- return: aggragated value with time filter for binary data (table bwerte)

set @@character_set_client = 'utf8';

delimiter //

DROP FUNCTION IF EXISTS getbwertvbz//
create function getbwertvbz(wid integer, von datetime, bis datetime, mode varchar(12),
                            zvon time, zbis time)
                  returns varchar(255)
deterministic
sql security invoker
begin
  declare va, ba, vo, bo time;
  if TIME(zvon) < TIME(zbis)
   then
    set va = TIME(zvon);
    set ba = TIME(zbis);
    set vo = '24:00:00';
    set bo = '-00:00:01';
  else
    set va = '24:00:00';
    set ba = '-00:00:01';
    set vo = TIME(zvon);
    set bo = TIME(zbis);
  end if;
  case mode
    when 'count' then
      return (select count(*) from bwerte
              where werteindizesID = wid and von <= datum and datum <= bis
                and ((va <= TIME(datum) and TIME(datum) <= ba)
                     or vo <= TIME(datum) or TIME(datum) <= bo));
    when 'size' then
      return (select length(daten) from bwerte, daten
              where werteindizesID = wid and von <= datum and datum <= bis
                and daten.ID=datenID
                and ((va <= TIME(datum) and TIME(datum) <= ba)
                     or vo <= TIME(datum) or TIME(datum) <= bo)
              order by datum desc, bwerte.ID desc limit 1);
    when 'firstsize' then
      return (select length(daten) from bwerte, daten
              where werteindizesID = wid and von <= datum and datum <= bis
                and daten.ID=datenID
                and ((va <= TIME(datum) and TIME(datum) <= ba)
                     or vo <= TIME(datum) or TIME(datum) <= bo)
              order by datum, bwerte.ID limit 1);
    when 'lastsize' then
      return (select length(daten) from bwerte, daten
              where werteindizesID = wid and von <= datum and datum <= bis
                and daten.ID=datenID
                and ((va <= TIME(datum) and TIME(datum) <= ba)
                     or vo <= TIME(datum) or TIME(datum) <= bo)
              order by datum desc, bwerte.ID desc limit 1);
    when 'differ' then
      return (select datenID from bwerte
              where werteindizesID = wid and von <= datum and datum <= bis
                and ((va <= TIME(datum) and TIME(datum) <= ba)
                     or vo <= TIME(datum) or TIME(datum) <= bo)
              order by datum desc, ID desc limit 1)
          != (select datenID from bwerte
              where werteindizesID = wid and von <= datum and datum <= bis
                and ((va <= TIME(datum) and TIME(datum) <= ba)
                     or vo <= TIME(datum) or TIME(datum) <= bo)
              order by datum, ID limit 1);
    when 'last' then
      return (select dateiname from bwerte
              where werteindizesID = wid and von <= datum and datum <= bis
                and ((va <= TIME(datum) and TIME(datum) <= ba)
                     or vo <= TIME(datum) or TIME(datum) <= bo)
              order by datum desc, ID desc limit 1);
    else
      return '';
  end case;
end//

GRANT EXECUTE on FUNCTION getbwertvbz to logginglesen@'localhost', loggingtest@'localhost', loggingadmin@'localhost'//
