-- getcwertvb.sql
--
-- function getcwertvb(wid, von, bis, mode) returns varchar
-- wid: werteindizesID
-- von: datetime from
-- bis: datetime to
-- mode: aggregation function
-- return: aggragated value (table cwerte)

set @@character_set_client = 'utf8';

delimiter //

DROP FUNCTION IF EXISTS getcwertvb//
create function getcwertvb(wid integer, von datetime, bis datetime, mode varchar(12))
                  returns varchar(30)
deterministic
sql security invoker
begin
  declare v double;
  declare r, l varchar(20);
  declare ld datetime;
  declare f integer;
  case mode
    when 'min' then
          select min(wert+0) into r from cwerte
            where werteindizesID = wid and von <= datum and datum <= bis;
    when 'max' then
          select max(wert+0) into r from cwerte
            where werteindizesID = wid and von <= datum and datum <= bis;
    when 'avg' then
          select avg(wert+0), max(length(0+replace(wert, '.',''))) into v, f from cwerte
            where werteindizesID = wid and von <= datum and datum <= bis;
          return round(v, f + 2 - locate('.', v));
    when 'last/avg' then
          select wert+0, datum into l, ld from cwerte
            where werteindizesID = wid and von <= datum and datum <= bis
            order by datum desc limit 1;
          select avg(wert+0), max(length(0+replace(wert, '.',''))) into v, f from cwerte
            where werteindizesID = wid and von <= datum and datum < ld;
          if v != 0 then
            return round(l/v, f + 2 - locate('.', l/v));
          else
            return NULL;
          end if;
    when 'last/avg-1' then
          select wert+0, datum into l, ld from cwerte
            where werteindizesID = wid and von <= datum and datum <= bis
            order by datum desc limit 1;
          select avg(wert+0), max(length(0+replace(wert, '.',''))) into v, f from cwerte
            where werteindizesID = wid and von <= datum and datum < ld;
          if v != 0 then
            return round(l/v-1, f + 2 - locate('.', l/v-1));
          else
            return NULL;
          end if;
    when 'dev' then
          select stddev_samp(wert+0), max(length(0+replace(wert, '.',''))) + 2 - locate('.', avg(wert+0)) into v, f from cwerte
            where werteindizesID = wid and von <= datum and datum <= bis;
            return round(v, f);
    when 'sum' then
          select sum(wert+0), max(locate('.', reverse(wert))) into r, f from cwerte
            where werteindizesID = wid and von <= datum and datum <= bis;
          if f < 0 then
            set f = 0;
          end if;
          return round(r, f);
    when 'max-min' then
          select max(wert+0)-min(wert+0) into r from cwerte
          where werteindizesID = wid and von <= datum and datum <= bis;
    when 'last-first' then
          return (select wert from cwerte
                  where werteindizesID = wid and von <= datum and datum <= bis
                  order by datum desc limit 1)
               - (select wert from cwerte
                  where werteindizesID = wid and von <= datum and datum <= bis
                  order by datum limit 1);
    when 'last-first+' then
          set r = (select wert from cwerte
                  where werteindizesID = wid and von <= datum and datum <= bis
                  order by datum desc limit 1)
               - (select wert from cwerte
                  where werteindizesID = wid and von <= datum and datum <= bis
                  order by datum limit 1);
          if r < 0 then
            return NULL;
          end if;
    when 'last-first-' then
          set r = (select wert from cwerte
                  where werteindizesID = wid and von <= datum and datum <= bis
                  order by datum desc limit 1)
               - (select wert from cwerte
                  where werteindizesID = wid and von <= datum and datum <= bis
                  order by datum limit 1);
          if r > 0 then
            return NULL;
          end if;
    when 'differ' then
          return BINARY (select wert from cwerte
                  where werteindizesID = wid and von <= datum and datum <= bis
                  order by datum desc limit 1)
              != BINARY (select wert from cwerte
                  where werteindizesID = wid and von <= datum and datum <= bis
                  order by datum limit 1);
    when 'last' then
          select wert into r from cwerte
          where werteindizesID = wid and von <= datum and datum <= bis
          order by datum desc limit 1;
    when 'first' then
          select wert into r from cwerte
          where werteindizesID = wid and von <= datum and datum <= bis
          order by datum limit 1;
    when 'size' then
          select length(wert) into r from cwerte
          where werteindizesID = wid and von <= datum and datum <= bis
          order by datum desc limit 1;
    when 'firstsize' then
          select length(wert) into r from cwerte
          where werteindizesID = wid and von <= datum and datum <= bis
          order by datum limit 1;
    when 'count' then
          select count(*) into r from cwerte
          where werteindizesID = wid and von <= datum and datum <= bis;
    when 'grad' then
      return gradientcvb(wid, von, bis);
    when 'accel' then
      return accelc(wid, von, bis, '00:00:00', '24:00:00', '',
                    '', '', '', '', '', '', '', '', '', '', '');
    else
      return NULL;
  end case;
  return r;
end//

GRANT EXECUTE on FUNCTION getcwertvb to logginglesen@'localhost', loggingtest@'localhost', loggingadmin@'localhost'//
