-- getcwertvbfilter.sql
--
-- function getcwertvbfilter(wid, von, bis, mode, zvon, zbis) returns varchar
-- wid: werteindizesID
-- von: datetime from
-- bis: datetime to
-- mode: aggregation function
-- zvon: time from
-- zbis: time to
-- return: aggragated value with time filter (table cwerte)

set @@character_set_client = 'utf8';

delimiter //

DROP FUNCTION IF EXISTS getcwertvbfilter//
create function getcwertvbfilter(wid integer, von datetime, bis datetime,
  mode varchar(12), zvon time, zbis time)
                  returns varchar(30)
deterministic
sql security invoker
begin
  declare v double;
  declare r, l varchar(20);
  declare ld datetime;
  declare va, ba, vo, bo time;
  declare f integer;
  if TIME(zvon) < TIME(zbis)
   then
    set va = TIME(zvon);
    set ba = TIME(zbis);
    set vo = '00:00:00';
    set bo = '23:59:59';
  else
    set va = '00:00:00';
    set ba = '23:59:59';
    set vo = TIME(zvon);
    set bo = TIME(zbis);
  end if;
  case mode
    when 'min' then
      select min(wert+0) into r from cwerte
        where werteindizesID = wid and von <= datum and datum <= bis
          and ((va <= TIME(datum) and TIME(datum) <= ba)
               or vo <= TIME(datum) or TIME(datum) <= bo);
    when 'max' then
      select max(wert+0) into r from cwerte
        where werteindizesID = wid and von <= datum and datum <= bis
          and ((va <= TIME(datum) and TIME(datum) <= ba)
               or vo <= TIME(datum) or TIME(datum) <= bo);
    when 'avg' then
      select avg(wert+0), max(length(0+replace(wert, '.',''))) into v, f from cwerte
        where werteindizesID = wid and von <= datum and datum <= bis
          and ((va <= TIME(datum) and TIME(datum) <= ba)
               or vo <= TIME(datum) or TIME(datum) <= bo);
      return round(v, f + 2 - locate('.', v));
    when 'last/avg' then
      select wert+0, datum into l, ld from cwerte
        where werteindizesID = wid and von <= datum and datum <= bis
          and ((va <= TIME(datum) and TIME(datum) <= ba)
               or vo <= TIME(datum) or TIME(datum) <= bo)
        order by datum desc limit 1;
      select avg(wert+0) into v, f from cwerte
        where werteindizesID = wid and von <= datum and datum < ld
          and ((va <= TIME(datum) and TIME(datum) <= ba)
               or vo <= TIME(datum) or TIME(datum) <= bo);
      if v != 0 then
        return round(l/v, f + 2 - locate('.', l/v));
      else
        return NULL;
      end if;
    when 'last/avg-1' then
      select wert+0, datum into l, ld from cwerte
        where werteindizesID = wid and von <= datum and datum <= bis
          and ((va <= TIME(datum) and TIME(datum) <= ba)
               or vo <= TIME(datum) or TIME(datum) <= bo)
        order by datum desc limit 1;
      select avg(wert+0), max(length(0+replace(wert, '.',''))) into v, f from cwerte
        where werteindizesID = wid and von <= datum and datum < ld
          and ((va <= TIME(datum) and TIME(datum) <= ba)
               or vo <= TIME(datum) or TIME(datum) <= bo);
      if v != 0 then
        return round(l/v-1, f + 2 - locate('.', l/v-1));
      else
        return NULL;
      end if;
    when 'dev' then
      select stddev_samp(wert+0), max(length(0+replace(wert, '.',''))) + 2 - locate('.', avg(wert+0)) into v, f from cwerte
        where werteindizesID = wid and von <= datum and datum <= bis
          and ((va <= TIME(datum) and TIME(datum) <= ba)
               or vo <= TIME(datum) or TIME(datum) <= bo);
      return round(v, f);
    when 'sum' then
      select sum(wert+0), max(locate('.', reverse(wert))) into r, f from cwerte
        where werteindizesID = wid and von <= datum and datum <= bis
          and ((va <= TIME(datum) and TIME(datum) <= ba)
               or vo <= TIME(datum) or TIME(datum) <= bo);
      if f < 0 then
        set f = 0;
      end if;
      return round(r, f);
    when 'max-min' then
      select max(wert+0)-min(wert+0) into r from cwerte
        where werteindizesID = wid and von <= datum and datum <= bis
          and ((va <= TIME(datum) and TIME(datum) <= ba)
               or vo <= TIME(datum) or TIME(datum) <= bo);
    when 'last-first' then
      return (select wert from cwerte
              where werteindizesID = wid and von <= datum and datum <= bis
                and ((va <= TIME(datum) and TIME(datum) <= ba)
                     or vo <= TIME(datum) or TIME(datum) <= bo)
              order by datum desc limit 1)
           - (select wert from cwerte
              where werteindizesID = wid and von <= datum and datum <= bis
                and ((va <= TIME(datum) and TIME(datum) <= ba)
                     or vo <= TIME(datum) or TIME(datum) <= bo)
              order by datum limit 1);
    when 'last-first+' then
      set r = (select wert from cwerte
              where werteindizesID = wid and von <= datum and datum <= bis
                and ((va <= TIME(datum) and TIME(datum) <= ba)
                     or vo <= TIME(datum) or TIME(datum) <= bo)
              order by datum desc limit 1)
           - (select wert from cwerte
              where werteindizesID = wid and von <= datum and datum <= bis
                and ((va <= TIME(datum) and TIME(datum) <= ba)
                     or vo <= TIME(datum) or TIME(datum) <= bo)
              order by datum limit 1);
      if r < 0 then
        return NULL;
      end if;
    when 'last-first-' then
      set r = (select wert from cwerte
              where werteindizesID = wid and von <= datum and datum <= bis
                and ((va <= TIME(datum) and TIME(datum) <= ba)
                     or vo <= TIME(datum) or TIME(datum) <= bo)
              order by datum desc limit 1)
           - (select wert from cwerte
              where werteindizesID = wid and von <= datum and datum <= bis
                and ((va <= TIME(datum) and TIME(datum) <= ba)
                     or vo <= TIME(datum) or TIME(datum) <= bo)
              order by datum limit 1);
      if r > 0 then
        return NULL;
      end if;
    when 'differ' then
      return BINARY (select wert from cwerte
              where werteindizesID = wid and von <= datum and datum <= bis
                and ((va <= TIME(datum) and TIME(datum) <= ba)
                     or vo <= TIME(datum) or TIME(datum) <= bo)
              order by datum desc limit 1)
          != BINARY (select wert from cwerte
              where werteindizesID = wid and von <= datum and datum <= bis
                and ((va <= TIME(datum) and TIME(datum) <= ba)
                     or vo <= TIME(datum) or TIME(datum) <= bo)
              order by datum limit 1);
    when 'count' then
      select count(*) into r from cwerte
        where werteindizesID = wid and von <= datum and datum <= bis
          and ((va <= TIME(datum) and TIME(datum) <= ba)
               or vo <= TIME(datum) or TIME(datum) <= bo);
    when 'grad' then
      return gradientcvbfilter(wid, von, bis, zvon, zbis);
    when 'accel' then
      return accelc(wid, von, bis, zvon, zbis, '',
              '', '', '', '', '', '', '', '', '', '', '');
    when 'last' then
      select wert into r from cwerte
        where werteindizesID = wid and von <= datum and datum <= bis
          and ((va <= TIME(datum) and TIME(datum) <= ba)
               or vo <= TIME(datum) or TIME(datum) <= bo)
        order by datum desc limit 1;
    else
      return NULL;
  end case;
  return r;
end//

GRANT EXECUTE on FUNCTION getcwertvbfilter to logginglesen@'localhost', loggingtest@'localhost'//
