-- getlwertvb.sql
--
-- function getlwertvb(lwid, von, bis, mode) returns varchar
-- lwid: lwerteindizesID
-- von: datetime from
-- bis: datetime to
-- mode: aggregation function
-- return: aggragated value from compressed values table lwerte

set @@character_set_client = 'utf8';

delimiter //

DROP FUNCTION IF EXISTS getlwertvb//
create function getlwertvb(lwid integer, von datetime, bis datetime, mode varchar(12))
                  returns varchar(30)
deterministic
sql security invoker
begin
  declare r, l double;
  declare ld datetime;
  declare f integer;
  case mode
    when 'min' then
          select min(wert) into r from lwerte
            where lwerteindizesID = lwid and von <= datum and datum <= bis;
    when 'max' then
          select max(wert) into r from lwerte
            where lwerteindizesID = lwid and von <= datum and datum <= bis;
    when 'avg' then
          select avg(wert), max(length(0+replace(wert, '.',''))) into r, f from lwerte
            where lwerteindizesID = lwid and von <= datum and datum <= bis;
          return round(r, f + 2 - locate('.', r));
    when 'last/avg' then
          select wert, datum into l, ld from lwerte
            where lwerteindizesID = lwid and von <= datum and datum <= bis
            order by datum desc limit 1;
          select avg(wert), max(length(0+replace(wert, '.',''))) into r, f from lwerte
            where lwerteindizesID = lwid and von <= datum and datum < ld;
          if r != 0 then
            return round(l/r, f + 2 - locate('.', l/r));
          else
            return '';
          end if;
    when 'last/avg-1' then
          select wert, datum into l, ld from lwerte
            where lwerteindizesID = lwid and von <= datum and datum <= bis
            order by datum desc limit 1;
          select avg(wert), max(length(0+replace(wert, '.',''))) into r, f from lwerte
            where lwerteindizesID = lwid and von <= datum and datum < ld;
          if r != 0 then
            return round(l/r-1, f + 2 - locate('.', l/r-1));
          else
            return '';
          end if;
    when 'dev' then
          select stddev_samp(wert) into r from lwerte
            where lwerteindizesID = lwid and von <= datum and datum <= bis;
    when 'sum' then
          select sum(wert) into r from lwerte
            where lwerteindizesID = lwid and von <= datum and datum <= bis;
    when 'max-min' then
          select max(wert)-min(wert) into r from lwerte
            where lwerteindizesID = lwid and von <= datum and datum <= bis;
    when 'last-first' then
          return (select wert from lwerte
                  where lwerteindizesID = lwid and von <= datum and datum <= bis
                  order by datum desc limit 1)
               - (select wert from lwerte
                  where lwerteindizesID = lwid and von <= datum and datum <= bis
                  order by datum limit 1);
    when 'last-first+' then
          set r = (select wert from lwerte
                  where lwerteindizesID = lwid and von <= datum and datum <= bis
                  order by datum desc limit 1)
               - (select wert from lwerte
                  where lwerteindizesID = lwid and von <= datum and datum <= bis
                  order by datum limit 1);
          if r < 0 then
            return '';
          end if;
    when 'last-first-' then
          set r = (select wert from lwerte
                  where lwerteindizesID = lwid and von <= datum and datum <= bis
                  order by datum desc limit 1)
               - (select wert from lwerte
                  where lwerteindizesID = lwid and von <= datum and datum <= bis
                  order by datum limit 1);
          if r > 0 then
            return '';
          end if;
    when 'differ' then
          return BINARY (select wert from lwerte
                  where lwerteindizesID = lwid and von <= datum and datum <= bis
                  order by datum desc limit 1)
              != BINARY (select wert from lwerte
                  where lwerteindizesID = lwid and von <= datum and datum <= bis
                  order by datum limit 1);
    when 'last' then
          select wert into r from lwerte
            where lwerteindizesID = lwid and von <= datum and datum <= bis
            order by datum desc limit 1;
    when 'first' then
          select wert into r from lwerte
            where lwerteindizesID = lwid and von <= datum and datum <= bis
            order by datum limit 1;
    when 'size' then
          select length(wert) into r from lwerte
            where lwerteindizesID = lwid and von <= datum and datum <= bis
            order by datum desc limit 1;
    when 'firstsize' then
          select length(wert) into r from lwerte
            where lwerteindizesID = lwid and von <= datum and datum <= bis
            order by datum limit 1;
    when 'count' then
          select count(*) into r from lwerte
            where lwerteindizesID = lwid and von <= datum and datum <= bis;
    when 'grad' then
      return gradientlvbextfilter(lwid, von, bis, '00:00:00', '24:00:00', '',
                                   '', '', '', '', '', '', '', '', '', '', '');
    when 'accel' then
      return accell(lwid, von, bis, '00:00:00', '24:00:00', '',
                     '', '', '', '', '', '', '', '', '', '', '');
    else
      return '';
  end case;
  return r;
end//

GRANT EXECUTE on FUNCTION getlwertvb to logginglesen@'localhost', loggingtest@'localhost'//
