-- getlwertvbextfilter.sql
--
-- function getlwertvbextfilter(lwid, von, bis, mode, zvon, zbis, wtf, vg1, wf1, m2, vg2, wf2, m3, vg3, wf3, m4, vg4, wf4) returns varchar
-- lwid: lwerteindizesID
-- von: datetime from
-- bis: datetime to
-- mode: aggregation function
-- zvon: time from
-- zbis: time to
-- wtf: set of weekdays
-- vg1: comparevalue 1
-- wf1: comparefunction 1
-- m2: OR|AND
-- vg2: comparevalue 2
-- wf2: comparefunction 2
-- m3: OR|AND
-- vg3: comparevalue 3
-- wf3: comparefunction 3
-- m4: OR|AND
-- vg4: comparevalue 4
-- wf4: comparefunction 4
-- return: aggragated value with extended filter from compressed values (table lwerte)

set @@character_set_client = 'utf8';

delimiter //

DROP FUNCTION IF EXISTS getlwertvbextfilter//
create function getlwertvbextfilter(lwid integer, von datetime, bis datetime,
  mode varchar(12), zvon time, zbis time, wtf varchar(20),
  vg1 varchar(3), wf1 varchar(250),
  m2 varchar(3), vg2 varchar(3), wf2 varchar(250),
  m3 varchar(3), vg3 varchar(3), wf3 varchar(250),
  m4 varchar(3), vg4 varchar(3), wf4 varchar(250))
                  returns varchar(30)
deterministic
sql security invoker
begin
  declare r varchar(50);
  declare va, ba, vo, bo time;
  declare f integer;

  if TIME(zvon) < TIME(zbis)
   then
    set va = TIME(zvon);
    set ba = TIME(zbis);
    set vo = '24:00:00';
    set bo = '-00:00:01';
  else
    set va = '24:00:00';
    set ba = '-00:00:01';
    set vo = TIME(zvon);
    set bo = TIME(zbis);
  end if;
  case mode
    when 'min' then
      select min(wert+0) into r from lwerte
        where lwerteindizesID = lwid and von <= datum and datum <= bis
          and ((va <= TIME(datum) and TIME(datum) <= ba)
               or vo <= TIME(datum) or TIME(datum) <= bo)
          and (wtf = '' or find_in_set(DAYOFWEEK(datum), wtf))
          and extfilter(wert, vg1, wf1, m2, vg2, wf2, m3, vg3, wf3, m4, vg4, wf4);
    when 'max' then
      select max(wert+0) into r from lwerte
        where lwerteindizesID = lwid and von <= datum and datum <= bis
          and ((va <= TIME(datum) and TIME(datum) <= ba)
               or vo <= TIME(datum) or TIME(datum) <= bo)
          and (wtf = '' or find_in_set(DAYOFWEEK(datum), wtf))
          and extfilter(wert, vg1, wf1, m2, vg2, wf2, m3, vg3, wf3, m4, vg4, wf4);
    when 'avg' then
      select avg(wert+0), max(length(0+replace(wert, '.',''))) into r, f from lwerte
        where lwerteindizesID = lwid and von <= datum and datum <= bis
          and ((va <= TIME(datum) and TIME(datum) <= ba)
               or vo <= TIME(datum) or TIME(datum) <= bo)
          and (wtf = '' or find_in_set(DAYOFWEEK(datum), wtf))
          and extfilter(wert, vg1, wf1, m2, vg2, wf2, m3, vg3, wf3, m4, vg4, wf4);
      return round(r, f + 2 - locate('.', r));
    when 'dev' then
      select stddev_samp(wert+0) into r from lwerte
        where lwerteindizesID = lwid and von <= datum and datum <= bis
          and ((va <= TIME(datum) and TIME(datum) <= ba)
               or vo <= TIME(datum) or TIME(datum) <= bo)
          and (wtf = '' or find_in_set(DAYOFWEEK(datum), wtf))
          and extfilter(wert, vg1, wf1, m2, vg2, wf2, m3, vg3, wf3, m4, vg4, wf4);
    when 'sum' then
      select sum(wert+0) into r from lwerte
        where lwerteindizesID = lwid and von <= datum and datum <= bis
          and ((va <= TIME(datum) and TIME(datum) <= ba)
               or vo <= TIME(datum) or TIME(datum) <= bo)
          and (wtf = '' or find_in_set(DAYOFWEEK(datum), wtf))
          and extfilter(wert, vg1, wf1, m2, vg2, wf2, m3, vg3, wf3, m4, vg4, wf4);
    when 'max-min' then
      select max(wert+0)-min(wert+0) into r from lwerte
        where lwerteindizesID = lwid and von <= datum and datum <= bis
          and ((va <= TIME(datum) and TIME(datum) <= ba)
               or vo <= TIME(datum) or TIME(datum) <= bo)
          and (wtf = '' or find_in_set(DAYOFWEEK(datum), wtf))
          and extfilter(wert, vg1, wf1, m2, vg2, wf2, m3, vg3, wf3, m4, vg4, wf4);
    when 'last-first' then
      return (select wert from lwerte
              where lwerteindizesID = lwid and von <= datum and datum <= bis
                and ((va <= TIME(datum) and TIME(datum) <= ba)
                     or vo <= TIME(datum) or TIME(datum) <= bo)
                and (wtf = '' or find_in_set(DAYOFWEEK(datum), wtf))
                and extfilter(wert, vg1, wf1, m2, vg2, wf2, m3, vg3, wf3, m4, vg4, wf4)
              order by datum desc limit 1)
           - (select wert from lwerte
              where lwerteindizesID = lwid and von <= datum and datum <= bis
                and ((va <= TIME(datum) and TIME(datum) <= ba)
                     or vo <= TIME(datum) or TIME(datum) <= bo)
                and (wtf = '' or find_in_set(DAYOFWEEK(datum), wtf))
                and extfilter(wert, vg1, wf1, m2, vg2, wf2, m3, vg3, wf3, m4, vg4, wf4)
              order by datum limit 1);
    when 'last-first+' then
      set r = (select wert from lwerte
              where lwerteindizesID = lwid and von <= datum and datum <= bis
                and ((va <= TIME(datum) and TIME(datum) <= ba)
                     or vo <= TIME(datum) or TIME(datum) <= bo)
                and (wtf = '' or find_in_set(DAYOFWEEK(datum), wtf))
                and extfilter(wert, vg1, wf1, m2, vg2, wf2, m3, vg3, wf3, m4, vg4, wf4)
              order by datum desc limit 1)
           - (select wert from lwerte
              where lwerteindizesID = lwid and von <= datum and datum <= bis
                and ((va <= TIME(datum) and TIME(datum) <= ba)
                     or vo <= TIME(datum) or TIME(datum) <= bo)
                and (wtf = '' or find_in_set(DAYOFWEEK(datum), wtf))
                and extfilter(wert, vg1, wf1, m2, vg2, wf2, m3, vg3, wf3, m4, vg4, wf4)
              order by datum limit 1);
      if r < 0 then
        return '';
      end if;
    when 'last-first-' then
      set r = (select wert from lwerte
              where lwerteindizesID = lwid and von <= datum and datum <= bis
                and ((va <= TIME(datum) and TIME(datum) <= ba)
                     or vo <= TIME(datum) or TIME(datum) <= bo)
                and (wtf = '' or find_in_set(DAYOFWEEK(datum), wtf))
                and extfilter(wert, vg1, wf1, m2, vg2, wf2, m3, vg3, wf3, m4, vg4, wf4)
              order by datum desc limit 1)
           - (select wert from lwerte
              where lwerteindizesID = lwid and von <= datum and datum <= bis
                and ((va <= TIME(datum) and TIME(datum) <= ba)
                     or vo <= TIME(datum) or TIME(datum) <= bo)
                and (wtf = '' or find_in_set(DAYOFWEEK(datum), wtf))
                and extfilter(wert, vg1, wf1, m2, vg2, wf2, m3, vg3, wf3, m4, vg4, wf4)
              order by datum limit 1);
      if r > 0 then
        return '';
      end if;
    when 'first' then
      select wert into r from lwerte
        where lwerteindizesID = lwid and von <= datum and datum <= bis
          and ((va <= TIME(datum) and TIME(datum) <= ba)
               or vo <= TIME(datum) or TIME(datum) <= bo)
          and (wtf = '' or find_in_set(DAYOFWEEK(datum), wtf))
          and extfilter(wert, vg1, wf1, m2, vg2, wf2, m3, vg3, wf3, m4, vg4, wf4)
        order by datum limit 1;
    when 'differ' then
      return BINARY (select wert from lwerte
              where lwerteindizesID = lwid and von <= datum and datum <= bis
                and ((va <= TIME(datum) and TIME(datum) <= ba)
                     or vo <= TIME(datum) or TIME(datum) <= bo)
                and (wtf = '' or find_in_set(DAYOFWEEK(datum), wtf))
                and extfilter(wert, vg1, wf1, m2, vg2, wf2, m3, vg3, wf3, m4, vg4, wf4)
              order by datum desc limit 1)
          != BINARY (select wert from lwerte
              where lwerteindizesID = lwid and von <= datum and datum <= bis
                and ((va <= TIME(datum) and TIME(datum) <= ba)
                     or vo <= TIME(datum) or TIME(datum) <= bo)
                and (wtf = '' or find_in_set(DAYOFWEEK(datum), wtf))
                and extfilter(wert, vg1, wf1, m2, vg2, wf2, m3, vg3, wf3, m4, vg4, wf4)
              order by datum limit 1);
    when 'count' then
      select count(*) into r from lwerte
        where lwerteindizesID = lwid and von <= datum and datum <= bis
          and ((va <= TIME(datum) and TIME(datum) <= ba)
               or vo <= TIME(datum) or TIME(datum) <= bo)
          and (wtf = '' or find_in_set(DAYOFWEEK(datum), wtf))
          and extfilter(wert, vg1, wf1, m2, vg2, wf2, m3, vg3, wf3, m4, vg4, wf4);
    when 'grad' then
      return gradientlvbextfilter(lwid, von, bis, zvon, zbis, wtf,
              vg1, wf1, m2, vg2, wf2, m3, vg3, wf3, m4, vg4, wf4);
    when 'accel' then
      return accell(lwid, von, bis, zvon, zbis, wtf,
              vg1, wf1, m2, vg2, wf2, m3, vg3, wf3, m4, vg4, wf4);
    when 'last' then
      select wert+0 into r from lwerte
        where lwerteindizesID = lwid and von <= datum and datum <= bis
          and ((va <= TIME(datum) and TIME(datum) <= ba)
               or vo <= TIME(datum) or TIME(datum) <= bo)
          and (wtf = '' or find_in_set(DAYOFWEEK(datum), wtf))
          and extfilter(wert, vg1, wf1, m2, vg2, wf2, m3, vg3, wf3, m4, vg4, wf4)
        order by datum desc limit 1;
    else
      return '';
  end case;
  return r;
end//

GRANT EXECUTE on FUNCTION getlwertvbextfilter to logginglesen@'localhost'//
