-- getserverid.sql
--
-- function getserveridi(srvname, sip)returns integer
-- srvname: servername
-- sip: server IP address
-- return: ID of server

-- function getserveridx(srvname, x509s)returns integer
-- srvname: servername
-- x509s: x509 certificate from https connection
-- return: ID of server

set @@character_set_client = 'utf8';

delimiter //
DROP FUNCTION IF EXISTS getserveridi//
create function getserveridi(srvname varchar(200), sip varchar(39))
                returns integer
  deterministic
  sql security invoker
  begin
    declare s, ext varchar(50);
    declare si, ei, domid, scid integer unsigned;
 
    if srvname like 'DOMAIN:%' then
      set ei = locate(':', srvname, 8);
      if ei > 8 then
        set ext = substr(srvname, ei+1);
        set s = substr(srvname, 8, ei-8);
      else
        set ext = '';
        set s = substr(srvname, 8);
      end if;
      if is_ipv6(sip) then
        select verfahrenID, serverclassesID into domid, scid
          from server
          where fqdn like concat(s,'%') and (fqdn = s or servername = s) and state != 'inactive'
            and (inet6_aton(sip) = ipv6
                 or exists (select * from acceptips where serverID = server.ID and ipstart <= hex(inet6_aton(sip)) and hex(inet6_aton(sip)) <= ipend))
          order by fqdn != s, ID desc limit 1;
      else
        select verfahrenID, serverclassesID into domid, scid
          from server
          where fqdn like concat(s,'%') and (fqdn = s or servername = s) and state != 'inactive'
            and (inet_aton(sip) = server.ip or inet_aton(sip) = transferip
                 or exists (select * from acceptips where serverID = server.ID and ipstart <= hex(inet6_aton(sip)) and hex(inet6_aton(sip)) <= ipend))
          order by fqdn != s, ID desc limit 1;
      end if;
      if domid is NULL then
        return NULL;
      else
        if ei > 8 AND ext = '' then
          set ext = coalesce((select name from serverclasses where ID = scid), '');
        end if;
        return newserver(domid, ext);
      end if;
    else
      if is_ipv6(sip) then
        set si = (select server.ID
            from server
            where fqdn like concat(srvname,'%') and (fqdn = srvname or servername = srvname) and state != 'inactive'
              and (inet6_aton(sip) = ipv6
                   or exists (select * from acceptips where serverID = server.ID and ipstart <= hex(inet6_aton(sip)) and hex(inet6_aton(sip)) <= ipend))
            order by fqdn != srvname, ID desc limit 1);
      else
        set si = (select server.ID
            from server
            where fqdn like concat(srvname,'%') and (fqdn = srvname or servername = srvname) and state != 'inactive'
              and (inet_aton(sip) = server.ip or inet_aton(sip) = transferip
                   or exists (select * from acceptips where serverID = server.ID and ipstart <= hex(inet6_aton(sip)) and hex(inet6_aton(sip)) <= ipend))
            order by fqdn != srvname, ID desc limit 1);
      end if;
      if si is NULL then
        return newautoserver(srvname, sip);
      else
        return si;
      end if;
    end if;
  end//

grant EXECUTE on FUNCTION getserveridi to logging@'localhost'//


DROP FUNCTION IF EXISTS getserveridx//
create function getserveridx(srvname varchar(200), x509s varchar(250))
                returns integer
  deterministic
  sql security invoker
  begin
    declare s, ext varchar(50);
    declare ei, domid, scid integer unsigned;
 
    if srvname like 'DOMAIN:%' then
      set ei = locate(':', srvname, 8);
      if ei > 8 then
        set ext = substr(srvname, ei+1);
        set s = substr(srvname, 8, ei-8);
      else
        set ext = '';
        set s = substr(srvname, 8);
      end if;
      select verfahrenID, serverclassesID into domid, scid
        from server
        where fqdn like concat(s,'%') and (fqdn = s or servername = s) and state != 'inactive'
          and find_in_set(x509s, x509subject)
        order by fqdn != s, ID desc limit 1;
      if domid is NULL then
        return NULL;
      else
        if ei > 8 AND ext = '' then
          set ext = coalesce((select name from serverclasses where ID = scid), '');
        end if;
        return newserver(domid, ext);
      end if;
    else
      return (select server.ID
          from server
          where fqdn like concat(srvname,'%') and (fqdn = srvname or servername = srvname) and state != 'inactive'
            and find_in_set(x509s, x509subject)
          order by fqdn != srvname, ID desc limit 1);
    end if;
  end//

grant EXECUTE on FUNCTION getserveridx to logging@'localhost'//
