-- getswertvb.sql
--
-- function getswertvb(wid, von, bis, mode) returns varchar
-- wid: werteindizesID
-- von: datetime from
-- bis: datetime to
-- mode: aggregation function
-- return: aggragated value (table swerte)

set @@character_set_client = 'utf8';

delimiter //

DROP FUNCTION IF EXISTS getswertvb//
create function getswertvb(wid integer, von datetime, bis datetime, mode varchar(12))
                  returns varchar(20000)
deterministic
sql security invoker
begin
  case mode
    when 'min' then
      return (select min(wert) from swerte
              where werteindizesID = wid and von <= datum and datum <= bis);
    when 'max' then
      return (select max(wert) from swerte
              where werteindizesID = wid and von <= datum and datum <= bis);
    when 'count' then
      return (select count(*) from swerte
              where werteindizesID = wid and von <= datum and datum <= bis);
    when 'last' then
      return (select wert from swerte
              where werteindizesID = wid and von <= datum and datum <= bis
              order by datum desc, ID desc limit 1);
    when 'first' then
      return (select wert from swerte
              where werteindizesID = wid and von <= datum and datum <= bis
              order by datum, ID limit 1);
    when 'size' then
      return (select length(wert) from swerte
              where werteindizesID = wid and von <= datum and datum <= bis
              order by datum desc, ID desc limit 1);
    when 'firstsize' then
      return (select length(wert) from swerte
              where werteindizesID = wid and von <= datum and datum <= bis
              order by datum, ID limit 1);
    when 'lastsize' then
      return (select length(wert) from swerte
              where werteindizesID = wid and von <= datum and datum <= bis
              order by datum desc, ID desc limit 1);
    when 'differ' then
      return BINARY (select wert from swerte
              where werteindizesID = wid and von <= datum and datum <= bis
              order by datum desc, ID desc limit 1)
          != BINARY (select wert from swerte
              where werteindizesID = wid and von <= datum and datum <= bis
              order by datum, ID limit 1);
    else
      return (select wert from swerte
              where werteindizesID = wid and von <= datum and datum <= bis
              order by datum desc, ID desc limit 1);
  end case;
end//

GRANT EXECUTE on FUNCTION getswertvb to logginglesen@'localhost', loggingtest@'localhost', loggingadmin@'localhost'//
