-- getswertvbextfilter.sql
--
-- function getswertvbextfilter(wid, von, bis, mode, zvon, zbis, wtf, vg1, wf1, m2, vg2, wf2, m3, vg3, wf3, m4, vg4, wf4) returns varchar
-- wid: werteindizesID
-- von: datetime from
-- bis: datetime to
-- mode: aggregation function
-- zvon: time from
-- zbis: time to
-- wtf: set of weekdays
-- vg1: comparevalue 1
-- wf1: comparefunction 1
-- m2: OR|AND
-- vg2: comparevalue 2
-- wf2: comparefunction 2
-- m3: OR|AND
-- vg3: comparevalue 3
-- wf3: comparefunction 3
-- m4: OR|AND
-- vg4: comparevalue 4
-- wf4: comparefunction 4
-- return: aggragated value with extended filter (table swerte)

set @@character_set_client = 'utf8';

delimiter //

DROP FUNCTION IF EXISTS getswertvbextfilter//
create function getswertvbextfilter(wid integer, von datetime, bis datetime,
  mode varchar(12), zvon time, zbis time, wtf varchar(20),
  vg1 varchar(3), wf1 varchar(250),
  m2 varchar(3), vg2 varchar(3), wf2 varchar(250),
  m3 varchar(3), vg3 varchar(3), wf3 varchar(250),
  m4 varchar(3), vg4 varchar(3), wf4 varchar(250))
                  returns varchar(20000)
deterministic
sql security invoker
begin
  declare va, ba, vo, bo time;
  if TIME(zvon) < TIME(zbis)
   then
    set va = TIME(zvon);
    set ba = TIME(zbis);
    set vo = '24:00:00';
    set bo = '-00:00:01';
  else
    set va = '24:00:00';
    set ba = '-00:00:01';
    set vo = TIME(zvon);
    set bo = TIME(zbis);
  end if;
  case mode
    when 'count' then
      return (select count(*) from swerte
              where werteindizesID = wid and von <= datum and datum <= bis
                and ((va <= TIME(datum) and TIME(datum) <= ba)
                     or vo <= TIME(datum) or TIME(datum) <= bo)
                and (wtf = '' or find_in_set(DAYOFWEEK(datum), wtf))
                and extsfilter(wert, vg1, wf1, m2, vg2, wf2, m3, vg3, wf3, m4, vg4, wf4));
    when 'first' then
      return (select wert from swerte
              where werteindizesID = wid and von <= datum and datum <= bis
                and ((va <= TIME(datum) and TIME(datum) <= ba)
                     or vo <= TIME(datum) or TIME(datum) <= bo)
                and (wtf = '' or find_in_set(DAYOFWEEK(datum), wtf))
                and extsfilter(wert, vg1, wf1, m2, vg2, wf2, m3, vg3, wf3, m4, vg4, wf4)
              order by datum, ID limit 1);
    when 'min' then
      return (select min(wert) from swerte
              where werteindizesID = wid and von <= datum and datum <= bis
                and ((va <= TIME(datum) and TIME(datum) <= ba)
                     or vo <= TIME(datum) or TIME(datum) <= bo)
                and (wtf = '' or find_in_set(DAYOFWEEK(datum), wtf))
                and extsfilter(wert, vg1, wf1, m2, vg2, wf2, m3, vg3, wf3, m4, vg4, wf4));
    when 'max' then
      return (select max(wert) from swerte
              where werteindizesID = wid and von <= datum and datum <= bis
                and ((va <= TIME(datum) and TIME(datum) <= ba)
                     or vo <= TIME(datum) or TIME(datum) <= bo)
                and (wtf = '' or find_in_set(DAYOFWEEK(datum), wtf))
                and extsfilter(wert, vg1, wf1, m2, vg2, wf2, m3, vg3, wf3, m4, vg4, wf4));
    when 'differ' then
      return BINARY (select wert from swerte
              where werteindizesID = wid and von <= datum and datum <= bis
                and ((va <= TIME(datum) and TIME(datum) <= ba)
                     or vo <= TIME(datum) or TIME(datum) <= bo)
                and (wtf = '' or find_in_set(DAYOFWEEK(datum), wtf))
                and extsfilter(wert, vg1, wf1, m2, vg2, wf2, m3, vg3, wf3, m4, vg4, wf4)
              order by datum desc, ID desc limit 1)
          != BINARY (select wert from swerte
              where werteindizesID = wid and von <= datum and datum <= bis
                and ((va <= TIME(datum) and TIME(datum) <= ba)
                     or vo <= TIME(datum) or TIME(datum) <= bo)
                and (wtf = '' or find_in_set(DAYOFWEEK(datum), wtf))
                and extsfilter(wert, vg1, wf1, m2, vg2, wf2, m3, vg3, wf3, m4, vg4, wf4)
              order by datum, ID limit 1);
    when 'last' then
      return (select wert from swerte
              where werteindizesID = wid and von <= datum and datum <= bis
                and ((va <= TIME(datum) and TIME(datum) <= ba)
                     or vo <= TIME(datum) or TIME(datum) <= bo)
                and (wtf = '' or find_in_set(DAYOFWEEK(datum), wtf))
                and extsfilter(wert, vg1, wf1, m2, vg2, wf2, m3, vg3, wf3, m4, vg4, wf4)
              order by datum desc, ID desc limit 1);
    else
      return '';
  end case;
end//

GRANT EXECUTE on FUNCTION getswertvbextfilter to logginglesen@'localhost'//
