-- gettwertvbextfilter.sql
--
-- function gettwertvbextfilter(wid, von, bis, mode, zvon, zbis, wtf, vg1, wf1, m2, vg2, wf2, m3, vg3, wf3, m4, vg4, wf4) returns varchar
-- wid: werteindizesID
-- von: datetime from
-- bis: datetime to
-- mode: aggregation function
-- zvon: time from
-- zbis: time to
-- wtf: set of weekdays
-- vg1: comparevalue 1
-- wf1: comparefunction 1
-- m2: OR|AND
-- vg2: comparevalue 2
-- wf2: comparefunction 2
-- m3: OR|AND
-- vg3: comparevalue 3
-- wf3: comparefunction 3
-- m4: OR|AND
-- vg4: comparevalue 4
-- wf4: comparefunction 4
-- return: aggragated value with extended filter (table twerte)

set @@character_set_client = 'utf8';

delimiter //

DROP FUNCTION IF EXISTS gettwertvbextfilter//
create function gettwertvbextfilter(wid integer, von datetime(6), bis datetime(6),
  mode varchar(12), zvon time, zbis time, wtf varchar(20),
  vg1 varchar(3), wf1 varchar(250),
  m2 varchar(3), vg2 varchar(3), wf2 varchar(250),
  m3 varchar(3), vg3 varchar(3), wf3 varchar(250),
  m4 varchar(3), vg4 varchar(3), wf4 varchar(250))
                  returns varchar(20000)
deterministic
sql security invoker
begin
  declare r, l varchar(20);
  declare ld datetime;
  declare va, ba, vo, bo time;
  declare f integer;
  declare t enum('str','num','dat');

  if TIME(zvon) < TIME(zbis)
   then
    set va = TIME(zvon);
    set ba = TIME(zbis);
    set vo = '24:00:00';
    set bo = '-00:00:01';
  else
    set va = '24:00:00';
    set ba = '-00:00:01';
    set vo = TIME(zvon);
    set bo = TIME(zbis);
  end if;
  select type into t from werteindizes, teststepdetails, einheiten
      where werteindizes.ID = wid and teststepdetailsID = teststepdetails.ID
        and einheitID = einheiten.ID;
  if t = 'num' then
    case mode
      when 'min' then
        select min(wert+0) into r from twerte
          where werteindizesID = wid and von <= datum and datum <= bis
            and ((va <= TIME(datum) and TIME(datum) <= ba)
                 or vo <= TIME(datum) or TIME(datum) <= bo)
            and (wtf = '' or find_in_set(DAYOFWEEK(datum), wtf))
            and extfilter(wert, vg1, wf1, m2, vg2, wf2, m3, vg3, wf3, m4, vg4, wf4);
      when 'max' then
        select max(wert+0) into r from twerte
          where werteindizesID = wid and von <= datum and datum <= bis
            and ((va <= TIME(datum) and TIME(datum) <= ba)
                 or vo <= TIME(datum) or TIME(datum) <= bo)
            and (wtf = '' or find_in_set(DAYOFWEEK(datum), wtf))
            and extfilter(wert, vg1, wf1, m2, vg2, wf2, m3, vg3, wf3, m4, vg4, wf4);
      when 'avg' then
        select avg(wert+0), max(length(0+replace(wert, '.',''))) into r, f from twerte
          where werteindizesID = wid and von <= datum and datum <= bis
            and ((va <= TIME(datum) and TIME(datum) <= ba)
                 or vo <= TIME(datum) or TIME(datum) <= bo)
            and (wtf = '' or find_in_set(DAYOFWEEK(datum), wtf))
            and extfilter(wert, vg1, wf1, m2, vg2, wf2, m3, vg3, wf3, m4, vg4, wf4);
        return round(r, f + 2 - locate('.', r));
      when 'last/avg' then
        select wert, datum into l, ld from twerte
          where werteindizesID = wid and von <= datum and datum <= bis
            and ((va <= TIME(datum) and TIME(datum) <= ba)
                 or vo <= TIME(datum) or TIME(datum) <= bo)
            and (wtf = '' or find_in_set(DAYOFWEEK(datum), wtf))
            and extfilter(wert, vg1, wf1, m2, vg2, wf2, m3, vg3, wf3, m4, vg4, wf4)
          order by datum desc limit 1;
        select avg(wert+0), max(length(0+replace(wert, '.',''))) into r, f from twerte
          where werteindizesID = wid and von <= datum and datum < ld
            and ((va <= TIME(datum) and TIME(datum) <= ba)
                 or vo <= TIME(datum) or TIME(datum) <= bo)
            and (wtf = '' or find_in_set(DAYOFWEEK(datum), wtf))
            and extfilter(wert, vg1, wf1, m2, vg2, wf2, m3, vg3, wf3, m4, vg4, wf4);
        if r != 0 then
          return round(l/r, f + 2 - locate('.', l/r));
        else
          return NULL;
        end if;
      when 'last/avg-1' then
        select wert, datum into l, ld from twerte
          where werteindizesID = wid and von <= datum and datum <= bis
            and ((va <= TIME(datum) and TIME(datum) <= ba)
                 or vo <= TIME(datum) or TIME(datum) <= bo)
            and (wtf = '' or find_in_set(DAYOFWEEK(datum), wtf))
            and extfilter(wert, vg1, wf1, m2, vg2, wf2, m3, vg3, wf3, m4, vg4, wf4)
          order by datum desc limit 1;
        select avg(wert+0), max(length(0+replace(wert, '.',''))) into r, f from twerte
          where werteindizesID = wid and von <= datum and datum < ld
            and ((va <= TIME(datum) and TIME(datum) <= ba)
                 or vo <= TIME(datum) or TIME(datum) <= bo)
            and (wtf = '' or find_in_set(DAYOFWEEK(datum), wtf))
            and extfilter(wert, vg1, wf1, m2, vg2, wf2, m3, vg3, wf3, m4, vg4, wf4);
        if r != 0 then
          return round(l/r-1, f + 2 - locate('.', l/r-1));
        else
          return NULL;
        end if;
      when 'dev' then
        select stddev_samp(wert+0) into r from twerte
          where werteindizesID = wid and von <= datum and datum <= bis
            and ((va <= TIME(datum) and TIME(datum) <= ba)
                 or vo <= TIME(datum) or TIME(datum) <= bo)
            and (wtf = '' or find_in_set(DAYOFWEEK(datum), wtf))
            and extfilter(wert, vg1, wf1, m2, vg2, wf2, m3, vg3, wf3, m4, vg4, wf4);
      when 'sum' then
        select sum(wert+0), max(locate('.', reverse(wert))) into r, f from twerte
          where werteindizesID = wid and von <= datum and datum <= bis
            and ((va <= TIME(datum) and TIME(datum) <= ba)
                 or vo <= TIME(datum) or TIME(datum) <= bo)
            and (wtf = '' or find_in_set(DAYOFWEEK(datum), wtf))
            and extfilter(wert, vg1, wf1, m2, vg2, wf2, m3, vg3, wf3, m4, vg4, wf4);
        if f < 0 then
          set f = 0;
        end if;
        return round(r, f);
      when 'max-min' then
        select max(wert+0)-min(wert+0) into r from twerte
          where werteindizesID = wid and von <= datum and datum <= bis
            and ((va <= TIME(datum) and TIME(datum) <= ba)
                 or vo <= TIME(datum) or TIME(datum) <= bo)
            and (wtf = '' or find_in_set(DAYOFWEEK(datum), wtf))
            and extfilter(wert, vg1, wf1, m2, vg2, wf2, m3, vg3, wf3, m4, vg4, wf4);
      when 'last-first' then
        return (select wert from twerte
                where werteindizesID = wid and von <= datum and datum <= bis
                  and ((va <= TIME(datum) and TIME(datum) <= ba)
                       or vo <= TIME(datum) or TIME(datum) <= bo)
                  and (wtf = '' or find_in_set(DAYOFWEEK(datum), wtf))
                  and extfilter(wert, vg1, wf1, m2, vg2, wf2, m3, vg3, wf3, m4, vg4, wf4)
                order by datum desc limit 1)
             - (select wert from twerte
                where werteindizesID = wid and von <= datum and datum <= bis
                  and ((va <= TIME(datum) and TIME(datum) <= ba)
                       or vo <= TIME(datum) or TIME(datum) <= bo)
                  and (wtf = '' or find_in_set(DAYOFWEEK(datum), wtf))
                  and extfilter(wert, vg1, wf1, m2, vg2, wf2, m3, vg3, wf3, m4, vg4, wf4)
                order by datum limit 1);
      when 'last-first+' then
        set r = (select wert from twerte
                where werteindizesID = wid and von <= datum and datum <= bis
                  and ((va <= TIME(datum) and TIME(datum) <= ba)
                       or vo <= TIME(datum) or TIME(datum) <= bo)
                  and (wtf = '' or find_in_set(DAYOFWEEK(datum), wtf))
                  and extfilter(wert, vg1, wf1, m2, vg2, wf2, m3, vg3, wf3, m4, vg4, wf4)
                order by datum desc limit 1)
             - (select wert from twerte
                where werteindizesID = wid and von <= datum and datum <= bis
                  and ((va <= TIME(datum) and TIME(datum) <= ba)
                       or vo <= TIME(datum) or TIME(datum) <= bo)
                  and (wtf = '' or find_in_set(DAYOFWEEK(datum), wtf))
                  and extfilter(wert, vg1, wf1, m2, vg2, wf2, m3, vg3, wf3, m4, vg4, wf4)
                order by datum limit 1);
        if r < 0 then
          return NULL;
        end if;
      when 'last-first-' then
        set r = (select wert from twerte
                where werteindizesID = wid and von <= datum and datum <= bis
                  and ((va <= TIME(datum) and TIME(datum) <= ba)
                       or vo <= TIME(datum) or TIME(datum) <= bo)
                  and (wtf = '' or find_in_set(DAYOFWEEK(datum), wtf))
                  and extfilter(wert, vg1, wf1, m2, vg2, wf2, m3, vg3, wf3, m4, vg4, wf4)
                order by datum desc limit 1)
             - (select wert from twerte
                where werteindizesID = wid and von <= datum and datum <= bis
                  and ((va <= TIME(datum) and TIME(datum) <= ba)
                       or vo <= TIME(datum) or TIME(datum) <= bo)
                  and (wtf = '' or find_in_set(DAYOFWEEK(datum), wtf))
                  and extfilter(wert, vg1, wf1, m2, vg2, wf2, m3, vg3, wf3, m4, vg4, wf4)
                order by datum limit 1);
        if r > 0 then
          return NULL;
        end if;
      when 'first' then
        select wert into r from twerte
          where werteindizesID = wid and von <= datum and datum <= bis
            and ((va <= TIME(datum) and TIME(datum) <= ba)
                  or vo <= TIME(datum) or TIME(datum) <= bo)
            and (wtf = '' or find_in_set(DAYOFWEEK(datum), wtf))
                 and extfilter(wert, vg1, wf1, m2, vg2, wf2, m3, vg3, wf3, m4, vg4, wf4)
          order by datum limit 1;
      when 'differ' then
        return BINARY (select wert from twerte
                where werteindizesID = wid and von <= datum and datum <= bis
                  and ((va <= TIME(datum) and TIME(datum) <= ba)
                       or vo <= TIME(datum) or TIME(datum) <= bo)
                  and (wtf = '' or find_in_set(DAYOFWEEK(datum), wtf))
                  and extfilter(wert, vg1, wf1, m2, vg2, wf2, m3, vg3, wf3, m4, vg4, wf4)
                order by datum desc limit 1)
            != BINARY (select wert from twerte
                where werteindizesID = wid and von <= datum and datum <= bis
                  and ((va <= TIME(datum) and TIME(datum) <= ba)
                       or vo <= TIME(datum) or TIME(datum) <= bo)
                  and (wtf = '' or find_in_set(DAYOFWEEK(datum), wtf))
                  and extfilter(wert, vg1, wf1, m2, vg2, wf2, m3, vg3, wf3, m4, vg4, wf4)
                order by datum limit 1);
      when 'count' then
        select count(*) into r from twerte
          where werteindizesID = wid and von <= datum and datum <= bis
            and ((va <= TIME(datum) and TIME(datum) <= ba)
                 or vo <= TIME(datum) or TIME(datum) <= bo)
            and (wtf = '' or find_in_set(DAYOFWEEK(datum), wtf))
            and extfilter(wert, vg1, wf1, m2, vg2, wf2, m3, vg3, wf3, m4, vg4, wf4);
      when 'grad' then
        return gradienttvbextfilter(wid, von, bis, zvon, zbis, wtf,
                vg1, wf1, m2, vg2, wf2, m3, vg3, wf3, m4, vg4, wf4);
      when 'accel' then
        return accelt(wid, von, bis, zvon, zbis, wtf,
                vg1, wf1, m2, vg2, wf2, m3, vg3, wf3, m4, vg4, wf4);
      when 'last' then
        select wert into r from twerte
          where werteindizesID = wid and von <= datum and datum <= bis
            and ((va <= TIME(datum) and TIME(datum) <= ba)
                 or vo <= TIME(datum) or TIME(datum) <= bo)
            and (wtf = '' or find_in_set(DAYOFWEEK(datum), wtf))
            and extfilter(wert, vg1, wf1, m2, vg2, wf2, m3, vg3, wf3, m4, vg4, wf4)
          order by datum desc limit 1;
      else
        return NULL;
    end case;
    return r;
  else
    case mode
      when 'count' then
        return (select count(*) from twerte
                where werteindizesID = wid and von <= datum and datum <= bis
                  and ((va <= TIME(datum) and TIME(datum) <= ba)
                       or vo <= TIME(datum) or TIME(datum) <= bo)
                  and (wtf = '' or find_in_set(DAYOFWEEK(datum), wtf))
                  and extsfilter(wert, vg1, wf1, m2, vg2, wf2, m3, vg3, wf3, m4, vg4, wf4));
      when 'first' then
        return (select wert from twerte
                where werteindizesID = wid and von <= datum and datum <= bis
                  and ((va <= TIME(datum) and TIME(datum) <= ba)
                       or vo <= TIME(datum) or TIME(datum) <= bo)
                  and (wtf = '' or find_in_set(DAYOFWEEK(datum), wtf))
                  and extsfilter(wert, vg1, wf1, m2, vg2, wf2, m3, vg3, wf3, m4, vg4, wf4)
                order by datum, ID limit 1);
      when 'min' then
        return (select min(wert) from twerte
                where werteindizesID = wid and von <= datum and datum <= bis
                  and ((va <= TIME(datum) and TIME(datum) <= ba)
                       or vo <= TIME(datum) or TIME(datum) <= bo)
                  and (wtf = '' or find_in_set(DAYOFWEEK(datum), wtf))
                  and extsfilter(wert, vg1, wf1, m2, vg2, wf2, m3, vg3, wf3, m4, vg4, wf4));
      when 'max' then
        return (select max(wert) from twerte
                where werteindizesID = wid and von <= datum and datum <= bis
                  and ((va <= TIME(datum) and TIME(datum) <= ba)
                       or vo <= TIME(datum) or TIME(datum) <= bo)
                  and (wtf = '' or find_in_set(DAYOFWEEK(datum), wtf))
                  and extsfilter(wert, vg1, wf1, m2, vg2, wf2, m3, vg3, wf3, m4, vg4, wf4));
      when 'differ' then
        return BINARY (select wert from twerte
                where werteindizesID = wid and von <= datum and datum <= bis
                  and ((va <= TIME(datum) and TIME(datum) <= ba)
                       or vo <= TIME(datum) or TIME(datum) <= bo)
                  and (wtf = '' or find_in_set(DAYOFWEEK(datum), wtf))
                  and extsfilter(wert, vg1, wf1, m2, vg2, wf2, m3, vg3, wf3, m4, vg4, wf4)
                order by datum desc, ID desc limit 1)
            != BINARY (select wert from twerte
                where werteindizesID = wid and von <= datum and datum <= bis
                  and ((va <= TIME(datum) and TIME(datum) <= ba)
                       or vo <= TIME(datum) or TIME(datum) <= bo)
                  and (wtf = '' or find_in_set(DAYOFWEEK(datum), wtf))
                  and extsfilter(wert, vg1, wf1, m2, vg2, wf2, m3, vg3, wf3, m4, vg4, wf4)
                order by datum, ID limit 1);
      when 'last' then
        return (select wert from twerte
                where werteindizesID = wid and von <= datum and datum <= bis
                  and ((va <= TIME(datum) and TIME(datum) <= ba)
                       or vo <= TIME(datum) or TIME(datum) <= bo)
                  and (wtf = '' or find_in_set(DAYOFWEEK(datum), wtf))
                  and extsfilter(wert, vg1, wf1, m2, vg2, wf2, m3, vg3, wf3, m4, vg4, wf4)
                order by datum desc, ID desc limit 1);
      else
        return '';
    end case;
  end if;
end//

GRANT EXECUTE on FUNCTION gettwertvbextfilter to logginglesen@'localhost'//
