-- gettwertvbfilter.sql
--
-- function gettwertvbfilter(wid, von, bis, mode, zvon, zbis) returns varchar
-- wid: werteindizesID
-- von: datetime from
-- bis: datetime to
-- mode: aggregation function
-- zvon: time from
-- zbis: time to
-- return: aggragated value with time filter (table twerte)

set @@character_set_client = 'utf8';

delimiter //

DROP FUNCTION IF EXISTS gettwertvbfilter//
create function gettwertvbfilter(wid integer, von datetime(6), bis datetime(6),
  mode varchar(12), zvon time, zbis time)
                  returns varchar(20000)
deterministic
sql security invoker
begin
  declare v, w double;
  declare r, l varchar(20);
  declare ld datetime;
  declare va, ba, vo, bo time;
  declare c, f integer;
  declare t enum('str','num','dat');

  if TIME(zvon) < TIME(zbis)
   then
    set va = TIME(zvon);
    set ba = TIME(zbis);
    set vo = '00:00:00';
    set bo = '23:59:59';
  else
    set va = '00:00:00';
    set ba = '23:59:59';
    set vo = TIME(zvon);
    set bo = TIME(zbis);
  end if;
  select type into t from werteindizes, teststepdetails, einheiten
      where werteindizes.ID = wid and teststepdetailsID = teststepdetails.ID
        and einheitID = einheiten.ID;
  if t = 'num' then
    case mode
      when 'min' then
        select min(wert+0) into r from twerte
          where werteindizesID = wid and von <= datum and datum <= bis
            and ((va <= TIME(datum) and TIME(datum) <= ba)
                 or vo <= TIME(datum) or TIME(datum) <= bo);
      when 'max' then
        select max(wert+0) into r from twerte
          where werteindizesID = wid and von <= datum and datum <= bis
            and ((va <= TIME(datum) and TIME(datum) <= ba)
                 or vo <= TIME(datum) or TIME(datum) <= bo);
      when 'avg' then
        select avg(wert+0), max(length(0+replace(wert, '.',''))), count(*) into v, f, c from twerte
          where werteindizesID = wid and von <= datum and datum <= bis
            and ((va <= TIME(datum) and TIME(datum) <= ba)
                 or vo <= TIME(datum) or TIME(datum) <= bo);
        return round(v, f + 2 - locate('.', v));
      when 'last/avg' then
        select wert+0, datum, length(0+replace(wert, '.','')) into l, ld, f from twerte
          where werteindizesID = wid and von <= datum and datum <= bis
            and ((va <= TIME(datum) and TIME(datum) <= ba)
                 or vo <= TIME(datum) or TIME(datum) <= bo)
          order by datum desc limit 1;
        select avg(wert+0) into v from twerte
          where werteindizesID = wid and von <= datum and datum < ld
            and ((va <= TIME(datum) and TIME(datum) <= ba)
                 or vo <= TIME(datum) or TIME(datum) <= bo);
        if v != 0 then
          return round(l/v, f + 2 - locate('.', l/v));
        else
          return NULL;
        end if;
      when 'last/avg-1' then
        select wert+0, datum, length(0+replace(wert, '.','')) into l, ld, f from twerte
          where werteindizesID = wid and von <= datum and datum <= bis
            and ((va <= TIME(datum) and TIME(datum) <= ba)
                 or vo <= TIME(datum) or TIME(datum) <= bo)
          order by datum desc limit 1;
        select avg(wert+0) into v from twerte
          where werteindizesID = wid and von <= datum and datum < ld
            and ((va <= TIME(datum) and TIME(datum) <= ba)
                 or vo <= TIME(datum) or TIME(datum) <= bo);
        if v != 0 then
          return round(l/v-1, f + 2 - locate('.', l/v-1));
        else
          return NULL;
        end if;
      when 'dev' then
        select stddev_samp(wert+0), avg(wert+0), max(length(0+replace(wert, '.',''))), count(*) into v, w, f, c from twerte
          where werteindizesID = wid and von <= datum and datum <= bis
            and ((va <= TIME(datum) and TIME(datum) <= ba)
                 or vo <= TIME(datum) or TIME(datum) <= bo);
        return round(v, f + 2 - locate('.', w));
      when 'sum' then
        select sum(wert+0), max(locate('.', reverse(wert))) into r, f from twerte
          where werteindizesID = wid and von <= datum and datum <= bis
            and ((va <= TIME(datum) and TIME(datum) <= ba)
                 or vo <= TIME(datum) or TIME(datum) <= bo);
        if f < 0 then
          set f = 0;
        end if;
        return round(r, f);
      when 'max-min' then
        select max(wert+0)-min(wert+0) into r from twerte
          where werteindizesID = wid and von <= datum and datum <= bis
            and ((va <= TIME(datum) and TIME(datum) <= ba)
                 or vo <= TIME(datum) or TIME(datum) <= bo);
      when 'last-first' then
        return (select wert from twerte
                where werteindizesID = wid and von <= datum and datum <= bis
                  and ((va <= TIME(datum) and TIME(datum) <= ba)
                       or vo <= TIME(datum) or TIME(datum) <= bo)
                order by datum desc limit 1)
             - (select wert from twerte
                where werteindizesID = wid and von <= datum and datum <= bis
                  and ((va <= TIME(datum) and TIME(datum) <= ba)
                       or vo <= TIME(datum) or TIME(datum) <= bo)
                order by datum limit 1);
      when 'last-first+' then
        set r = (select wert from twerte
                where werteindizesID = wid and von <= datum and datum <= bis
                  and ((va <= TIME(datum) and TIME(datum) <= ba)
                       or vo <= TIME(datum) or TIME(datum) <= bo)
                order by datum desc limit 1)
             - (select wert from twerte
                where werteindizesID = wid and von <= datum and datum <= bis
                  and ((va <= TIME(datum) and TIME(datum) <= ba)
                       or vo <= TIME(datum) or TIME(datum) <= bo)
                order by datum limit 1);
        if r < 0 then
          return NULL;
        end if;
      when 'last-first-' then
        set r = (select wert from twerte
                where werteindizesID = wid and von <= datum and datum <= bis
                  and ((va <= TIME(datum) and TIME(datum) <= ba)
                       or vo <= TIME(datum) or TIME(datum) <= bo)
                order by datum desc limit 1)
             - (select wert from twerte
                where werteindizesID = wid and von <= datum and datum <= bis
                  and ((va <= TIME(datum) and TIME(datum) <= ba)
                       or vo <= TIME(datum) or TIME(datum) <= bo)
                order by datum limit 1);
        if r > 0 then
          return NULL;
        end if;
      when 'differ' then
        return BINARY (select wert from twerte
                where werteindizesID = wid and von <= datum and datum <= bis
                  and ((va <= TIME(datum) and TIME(datum) <= ba)
                       or vo <= TIME(datum) or TIME(datum) <= bo)
                order by datum desc limit 1)
            != BINARY (select wert from twerte
                where werteindizesID = wid and von <= datum and datum <= bis
                  and ((va <= TIME(datum) and TIME(datum) <= ba)
                       or vo <= TIME(datum) or TIME(datum) <= bo)
                order by datum limit 1);
      when 'count' then
        select count(*) into r from twerte
          where werteindizesID = wid and von <= datum and datum <= bis
            and ((va <= TIME(datum) and TIME(datum) <= ba)
                 or vo <= TIME(datum) or TIME(datum) <= bo);
      when 'grad' then
        return gradienttvbfilter(wid, von, bis, zvon, zbis);
      when 'accel' then
        return accelt(wid, von, bis, zvon, zbis, '',
                '', '', '', '', '', '', '', '', '', '', '');
      when 'last' then
        select wert into r from twerte
          where werteindizesID = wid and von <= datum and datum <= bis
            and ((va <= TIME(datum) and TIME(datum) <= ba)
                 or vo <= TIME(datum) or TIME(datum) <= bo)
          order by datum desc limit 1;
      else
        return NULL;
    end case;
    return r;
  else
    case mode
      when 'count' then
        return (select count(*) from twerte
                where werteindizesID = wid and von <= datum and datum <= bis
                  and ((va <= TIME(datum) and TIME(datum) <= ba)
                  or vo <= TIME(datum) or TIME(datum) <= bo));
      else
        return (select wert from twerte
                where werteindizesID = wid and von <= datum and datum <= bis
                  and ((va <= TIME(datum) and TIME(datum) <= ba)
                  or vo <= TIME(datum) or TIME(datum) <= bo)
                order by datum desc limit 1);
    end case;
  end if;
end//

GRANT EXECUTE on FUNCTION gettwertvbfilter to logginglesen@'localhost', loggingtest@'localhost'//
