-- grafana_query.sql
--
-- procedure grafana_query(data json)
--

set @@character_set_client = 'utf8';

delimiter //

DROP PROCEDURE IF EXISTS grafana_query//
create procedure grafana_query(jquery json, usrd integer unsigned)
deterministic
sql security invoker
begin
  declare z, wid, alen, a, blen, b, srvid, secid, tstid integer unsigned;
  declare dtfrom, dtto datetime;
  declare wt, tp varchar(30);
  declare target varchar(1000);
  declare srv, sec, det varchar(100);
  declare tst varchar(330);
  declare servers, sections, teststeps, details varchar(20000);
  declare srvs, secs, tsts, dets varchar(20000);
  declare selcur cursor for
    select ID, vtype, wtab from loggingtmp.cols;
  declare continue handler for not found set z = 0;

  set dtfrom = convert_tz(json_unquote(replace(json_extract(jquery, '$.range.from'), 'Z', '')), '+00:00', 'SYSTEM');
  set dtto = convert_tz(json_unquote(replace(json_extract(jquery, '$.range.to'), 'Z', '')), '+00:00', 'SYSTEM');

  create temporary table loggingtmp.jsons (ret json);

  set alen = json_length(jquery, '$.targets');
  set a = 0;
  while a < alen do
    set target = json_unquote(json_extract(jquery, concat('$.targets[',a,'].target')));
    set srv = substring_index(target, '.', 1);
    set sec = substring_index(substring_index(target, '.', 2), '.', -1);
    set tst = substring_index(substring_index(target, '.', 3), '.', -1);

    if json_unquote(json_extract(jquery, concat('$.targets[',a,'].type'))) = 'timeseries' then
      set det = substring_index(target, '.', -1);
      select werteindizes.ID, wtab, serverID, sectionID, teststepID
        into wid, wt, srvid, secid, tstid
        from werteindizes
        join server on server.ID = serverID and servername = srv
        join uvaccess on uvaccess.userID = usrd and uvaccess.verfahrenID = server.verfahrenID
        join section on section.ID = sectionID and section.name = sec
        join teststep on teststep.ID = teststepID and teststep.name = tst
        join teststepdetails on teststepdetails.ID = teststepdetailsID and teststepdetails.name = det
        join einheiten on einheiten.ID = einheitID and type = 'num'
        order by werteindizes.ID desc
        limit 1;
      case wt when 'cwerte' then
        insert into loggingtmp.jsons
        select concat('{"target":"', target, '","datapoints":[', group_concat(concat('[',if(wert = '', '""', 0+wert),',',round(unix_timestamp(datum)*1000,0),']') separator ','), ']}')
          from cwerte where werteindizesID = wid and dtfrom <= datum and datum <= dtto;
      when 'pwerte' then
        insert into loggingtmp.jsons
        select concat('{"target":"', target, '","datapoints":[', group_concat(concat('[',if(wert = '', '""', 0+wert),',',round(unix_timestamp(datum)*1000,0),']') separator ','), ']}')
          from pwerte where werteindizesID = wid and dtfrom <= datum and datum <= dtto;
      when 'twerte' then
        insert into loggingtmp.jsons
        select concat('{"target":"', target, '","datapoints":[', group_concat(concat('[',if(wert = '', '""', 0+wert),',',round(unix_timestamp(datum)*1000,0),']') separator ','), ']}')
          from twerte where werteindizesID = wid and dtfrom <= datum and datum <= dtto;
      else
        insert into loggingtmp.jsons
        select concat('{"target":"', target, '","datapoints":[', group_concat(concat('[',if(wert = '', '""', 0+wert),',',round(unix_timestamp(datum)*1000,0),']') separator ','), ']}')
          from zwerte where werteindizesID = wid and dtfrom <= datum and datum <= dtto;
      end case;
    else
      set servers = json_extract(jquery, '$.scopedVars.Servers.value');
      set sections = json_extract(jquery, '$.scopedVars.Sections.value');
      set teststeps = json_extract(jquery, '$.scopedVars.Teststeps.value');
      set details = json_extract(jquery, '$.scopedVars.Details.value');
      if servers is NULL and sections is NULL and teststeps is NULL and details is NULL then
        create temporary table loggingtmp.cols (primary key(ID))
          select werteindizes.ID, d.name, if(e.name = '[E]', 'str', e.type) as vtype, e.wtab
        from werteindizes
        join server on server.ID = serverID and servername = srv
        join uvaccess on uvaccess.userID = usrd and uvaccess.verfahrenID = server.verfahrenID
        join section on section.ID = sectionID and section.name = sec
        join teststep on teststep.ID = teststepID and teststep.name = tst
        join teststepdetails as d on d.ID = teststepdetailsID
        join einheiten as e on e.ID = einheitID and type in ('str','num','dat')
        group by werteindizes.ID;
      else
        if servers is NULL OR servers = '' then
          set srvs = srv;
        else
          set srvs = json_unquote(json_extract(servers, '$[0]'));
          set blen = json_length(servers);
          set b = 1;
          while b < blen do
            set srvs = concat(srvs, ',', json_unquote(json_extract(servers, concat('$[', b, ']'))));
            set b = b + 1;
          end while;
        end if;
        if sections is NULL OR sections = '' then
          set secs = sec;
        else
          set secs = json_unquote(json_extract(sections, '$[0]'));
          set blen = json_length(sections);
          set b = 1;
          while b < blen do
            set secs = concat(secs, ',', json_unquote(json_extract(sections, concat('$[', b, ']'))));
            set b = b + 1;
          end while;
        end if;
        if teststeps is NULL or teststeps = '' then
          set tsts = tst;
        else
          set tsts = json_unquote(json_extract(teststeps, '$[0]'));
          set blen = json_length(teststeps);
          set b = 1;
          while b < blen do
            set tsts = concat(tsts, ',', json_unquote(json_extract(teststeps, concat('$[', b, ']'))));
            set b = b + 1;
          end while;
        end if;
        if details is NULL OR details = '' then
          set dets = det;
        else
          set dets = json_unquote(json_extract(details, '$[0]'));
          set blen = json_length(details);
          set b = 1;
          while b < blen do
            set dets = concat(dets, ',', json_unquote(json_extract(details, concat('$[', b, ']'))));
            set b = b + 1;
          end while;
        end if;
        create temporary table loggingtmp.cols (primary key(ID))
          select werteindizes.ID,
            concat(if(locate(',', srvs), concat(servername, '.'), ''), if(locate(',', secs), concat(section.name, '.'), ''),
                   if(locate(',', tst), concat(teststep.name, '.'), ''), d.name) as name,
            if(e.name = '[E]', 'str', e.type) as vtype, e.wtab
          from werteindizes
          join server on server.ID = serverID and find_in_set(servername, srvs)
          join uvaccess on uvaccess.userID = usrd and uvaccess.verfahrenID = server.verfahrenID
          join section on section.ID = sectionID and find_in_set(section.name, secs)
          join teststep on teststep.ID = teststepID and find_in_set(teststep.name, tsts)
          join teststepdetails as d on d.ID = teststepdetailsID and find_in_set(d.name, dets)
          join einheiten as e on e.ID = einheitID and type in ('str','num','dat')
          group by werteindizes.ID;
      end if;

      create temporary table loggingtmp.coldata (
        TS bigint unsigned not null,
        ID integer unsigned not null,
        val varchar(3000) not null,
        key ts(TS,ID));
      open selcur;
      set z = 1;
      fetch selcur into wid, tp, wt;
      while z do
        case wt when 'cwerte' then
          insert into loggingtmp.coldata (TS, ID, val) select round(unix_timestamp(datum)*1000,0), wid, if(wert = '', '""', 0+wert)
            from cwerte where werteindizesID = wid and dtfrom <= datum and datum <= dtto;
        when 'pwerte' then
          insert into loggingtmp.coldata (TS, ID, val) select round(unix_timestamp(datum)*1000,0), wid, if(wert = '', '""', 0+wert)
            from pwerte where werteindizesID = wid and dtfrom <= datum and datum <= dtto;
        when 'zwerte' then
          insert into loggingtmp.coldata (TS, ID, val) select round(unix_timestamp(datum)*1000,0), wid, if(wert = '', '""', 0+wert)
            from zwerte where werteindizesID = wid and dtfrom <= datum and datum <= dtto;
        when 'twerte' then
          if tp = 'num' then
            insert into loggingtmp.coldata (TS, ID, val) select round(unix_timestamp(datum)*1000,0), wid, if(wert = '', '""', 0+wert)
              from twerte where werteindizesID = wid and dtfrom <= datum and datum <= dtto;
          else
          insert into loggingtmp.coldata (TS, ID, val) select round(unix_timestamp(datum)*1000,0), wid, json_quote(wert)
            from twerte where werteindizesID = wid and dtfrom <= datum and datum <= dtto;
          end if;
        else
          insert into loggingtmp.coldata (TS, ID, val) select round(unix_timestamp(datum)*1000,0), wid, json_quote(wert)
            from swerte where werteindizesID = wid and dtfrom <= datum and datum <= dtto;
        end case;
        fetch selcur into wid, tp, wt;
      end while;
      close selcur;

      insert into loggingtmp.jsons values
        (concat('{"columns":[{"text":"Timestamp","type":"time"},',
               (select group_concat(concat('{"text":', json_quote(name), ',"type":"', if(vtype = 'num', 'number', 'string'), '"}') order by name, ID separator ',') from loggingtmp.cols),
               '],"rows":[',
         (select group_concat((select concat('[',t.TS,',',
                                             group_concat(coalesce((select val from loggingtmp.coldata where coldata.ID = cols.ID and coldata.TS = t.TS limit 1), '""')
                                                          order by cols.name, cols.ID separator ','),']')
                               from loggingtmp.cols) order by t.TS separator ',')
            from (select distinct TS from loggingtmp.coldata) as t),
         '],"type":"table"}'));
      drop temporary table loggingtmp.cols, loggingtmp.coldata;
    end if;
    set a = a + 1;
  end while;

  select concat('[', coalesce((select group_concat(ret separator ',') from loggingtmp.jsons), ''), ']') as c;
  drop temporary table loggingtmp.jsons;

end//
