-- grafana_search.sql
--
-- procedure grafana_search(data json)
--

set @@character_set_client = 'utf8';

delimiter //

DROP PROCEDURE IF EXISTS grafana_search//
create procedure grafana_search(data json, usrd integer unsigned)
deterministic
sql security invoker
begin
  declare target, srvreg, secreg, tstreg varchar(1000);
  declare srv, sec, det varchar(100);
  declare tst varchar(330);
  declare l, m integer unsigned;

  set srvreg = coalesce((select wert from systemvars where name = 'grafanaseverreg'), '');
  set secreg = coalesce((select wert from systemvars where name = 'grafanasectionreg'), '');
  set tstreg = coalesce((select wert from systemvars where name = 'grafanateststepreg'), '');
  set det = NULL;

  set target = json_unquote(json_extract(data, '$.target'));
  if target like '/%' then
    set target = replace(replace(substr(target, 2), '*', '%'), '?', '_');
    set l = locate('.', target);
    if l > 0 then
      set srv = left(target, l-1);
      create temporary table loggingtmp.srv (primary key(serverID, accessID))
        select server.ID as serverID, accessID, servername
        from server
        join userrechte as ur on ur.userID = usrd and ur.verfahrenID = server.verfahrenID
        where servername = srv and servername regexp srvreg
        union distinct
        select server.ID as serverID, accessID, servername
        from usergruppen as ug
        join gruppenrechte as gr using(gruppenID)
        join server on server.verfahrenID = gr.verfahrenID and servername like srv and servername regexp srvreg
        where ug.userID = usrd
        group by server.ID, accessID;
      set m = locate('.', target, l+1);
      if m > 0 then
        set sec = substr(target, l+1, m-l-1);
        create temporary table loggingtmp.sec (primary key(sectionID))
          select ID as sectionID, name from section where name like sec and name regexp secreg;
        set l = locate('.', target, m + 1);
        if l > 0 then
          set tst = substr(target, m+1, l-m-1);
          set det = substr(target, l+1);
          create temporary table loggingtmp.tst (primary key(teststepID))
            select ID as teststepID, name from teststep where name like tst and name regexp tstreg;
          select json_arrayagg(s) as j
          from
          (select concat(servername, '.', sec.name, '.', tst.name, '.', teststepdetails.name) as s
           from werteindizes
           join loggingtmp.srv using(serverID, accessID)
           join loggingtmp.sec using(sectionID)
           join loggingtmp.tst using(teststepID)
           join teststepdetails on teststepdetails.ID = teststepdetailsID and teststepdetails.name like det
           join einheiten on einheiten.ID = einheitID and type = 'num'
           order by servername, sec.name, tst.name, teststepdetails.name limit 100000) as s;
        else
          set tst = substr(target, m+1);
          create temporary table loggingtmp.tst (primary key(teststepID))
            select ID as teststepID, name from teststep where name like tst and name regexp tstreg;
          select json_arrayagg(s) as j
          from
          (select concat(servername, '.', sec.name, '.', tst.name) as s
           from werteindizes
           join loggingtmp.srv using(serverID, accessID)
           join loggingtmp.sec using(sectionID)
           join loggingtmp.tst using(teststepID)
           group by serverID, sectionID, teststepID
           order by servername, sec.name, tst.name limit 100000) as s;
        end if;
      else
        set sec = substr(target, l+1);
        create temporary table loggingtmp.sec (primary key(sectionID))
          select ID as sectionID, name from section where name like sec and name regexp secreg;
        create temporary table loggingtmp.tst (primary key(teststepID))
          select ID as teststepID, name from teststep where name regexp tstreg;
        select json_arrayagg(s) as j
        from
        (select concat(servername, '.', sec.name) as s
         from werteindizes
         join loggingtmp.srv using(serverID, accessID)
         join loggingtmp.sec using(sectionID)
         join loggingtmp.tst using(teststepID)
         group by serverID, sectionID
         order by servername, sec.name limit 100000) as s;
      end if;
    else
      create temporary table loggingtmp.srv (primary key(serverID, accessID))
        select server.ID as serverID, accessID, servername
        from server
        join userrechte as ur on ur.userID = usrd and ur.verfahrenID = server.verfahrenID
        where servername like target and servername regexp srvreg
        union distinct
        select server.ID as serverID, accessID, servername
        from usergruppen as ug
        join gruppenrechte as gr using(gruppenID)
        join server on server.verfahrenID = gr.verfahrenID and servername like target and servername regexp srvreg
        where ug.userID = usrd
        group by server.ID, accessID;
      create temporary table loggingtmp.sec (primary key(sectionID))
        select ID as sectionID, name from section where name regexp secreg;
      create temporary table loggingtmp.tst (primary key(teststepID))
        select ID as teststepID, name from teststep where name regexp tstreg;
      select json_arrayagg(s) as j
      from
      (select servername as s
       from werteindizes
       join loggingtmp.srv using(serverID, accessID)
       join loggingtmp.sec using(sectionID)
       join loggingtmp.tst using(teststepID)
       group by serverID
       order by servername limit 100000) as s;
    end if;
  else
    set l = locate('.', target);
    if l > 0 then
      set srv = left(target, l-1);
      create temporary table loggingtmp.srv (primary key(serverID, accessID))
        select server.ID as serverID, accessID, servername
        from server
        join userrechte as ur on ur.userID = usrd and ur.verfahrenID = server.verfahrenID
        where servername = srv and servername regexp srvreg
        union distinct
        select server.ID as serverID, accessID, servername
        from usergruppen as ug
        join gruppenrechte as gr using(gruppenID)
        join server on server.verfahrenID = gr.verfahrenID and servername = srv and servername regexp srvreg
        where ug.userID = usrd
        group by server.ID, accessID;
      set m = locate('.', target, l+1);
      if m > 0 then
        set sec = substr(target, l+1, m-l-1);
        create temporary table loggingtmp.sec (primary key(sectionID))
          select ID as sectionID, name from section where name = sec and name regexp secreg;
        set l = locate('.', target, m + 1);
        if l > 0 then
          set tst = substr(target, m+1, l-m-1);
          set det = concat(substr(target, l+1), '%');
          create temporary table loggingtmp.tst (primary key(teststepID))
            select ID as teststepID, name from teststep where name = tst and name regexp tstreg;
        else
          set tst = concat(substr(target, m+1), '%');
          create temporary table loggingtmp.tst (primary key(teststepID))
            select ID as teststepID, name from teststep where name like tst and name regexp tstreg;
        end if;
      else
        set sec = concat(substr(target, l+1), '%');
        create temporary table loggingtmp.sec (primary key(sectionID))
          select ID as sectionID, name from section where name like sec and name regexp secreg;
        create temporary table loggingtmp.tst (primary key(teststepID))
          select ID as teststepID, name from teststep where name regexp tstreg;
      end if;
    else
      set srv = concat(target, '%');
      create temporary table loggingtmp.srv (primary key(serverID, accessID))
        select server.ID as serverID, accessID, servername
        from server
        join userrechte as ur on ur.userID = usrd and ur.verfahrenID = server.verfahrenID
        where servername like srv and servername regexp srvreg
        union distinct
        select server.ID as serverID, accessID, servername
        from usergruppen as ug
        join gruppenrechte as gr using(gruppenID)
        join server on server.verfahrenID = gr.verfahrenID and servername like srv and servername regexp srvreg
        where ug.userID = usrd
        group by server.ID, accessID;
      create temporary table loggingtmp.sec (primary key(sectionID))
        select ID as sectionID, name from section where name regexp secreg;
      create temporary table loggingtmp.tst (primary key(teststepID))
        select ID as teststepID, name from teststep where name regexp tstreg;
    end if;

    if json_unquote(json_extract(data, '$.type')) = 'table' then
      select json_arrayagg(s) as j
      from
      (select concat(servername, '.', sec.name, '.', tst.name) as s
       from werteindizes
       join loggingtmp.srv using(serverID, accessID)
       join loggingtmp.sec using(sectionID)
       join loggingtmp.tst using(teststepID)
       group by serverID, sectionID, teststepID
       order by servername, sec.name, tst.name limit 100000) as s;
    elseif det is NULL then
      select json_arrayagg(s) as j
      from
      (select concat(servername, '.', sec.name, '.', tst.name, '.', teststepdetails.name) as s
       from werteindizes
       join loggingtmp.srv using(serverID, accessID)
       join loggingtmp.sec using(sectionID)
       join loggingtmp.tst using(teststepID)
       join teststepdetails on teststepdetails.ID = teststepdetailsID
       join einheiten on einheiten.ID = einheitID and type = 'num'
       order by servername, sec.name, tst.name, teststepdetails.name limit 100000) as s;
    else
      select json_arrayagg(s) as j
      from
      (select concat(servername, '.', sec.name, '.', tst.name, '.', teststepdetails.name) as s
       from werteindizes
       join loggingtmp.srv using(serverID, accessID)
       join loggingtmp.sec using(sectionID)
       join loggingtmp.tst using(teststepID)
       join teststepdetails on teststepdetails.ID = teststepdetailsID and teststepdetails.name like det
       join einheiten on einheiten.ID = einheitID and type = 'num'
       order by servername, sec.name, tst.name, teststepdetails.name limit 100000) as s;
    end if;
  end if;
  drop temporary table loggingtmp.srv, loggingtmp.sec, loggingtmp.tst;

end//
