set @@character_set_client = 'utf8';

delimiter //

drop trigger if exists groupsuspensions_au//
create trigger groupsuspensions_au after update on groupsuspensions
  for each row 
begin
  insert into changeprot
    (groupID, accessID, changedtable, tableID, userID, username, action,
     domain, classes, server, section, teststep, details, unit, newvalues)
  select NEW.groupID, NEW.accessID, 'groupsuspensions', NEW.ID,
    coalesce(@userid,0), coalesce(@username,@srvname,'SYSTEM'), 'update',
    showreg(NEW.domain, NEW.domain_v), showreg(NEW.classes, NEW.classes_v), showreg(NEW.server, NEW.server_v),
    showreg(NEW.section, NEW.section_v), showreg(NEW.teststep, NEW.teststep_v), showreg(NEW.details, NEW.details_v),
    showreg(NEW.einheit, NEW.einheit_v),
    concat('datefrom=', NEW.datefrom,
           '\ndateto=', NEW.dateto,
           if(NEW.weekdays = '0,1,2,3,4,5,6', '', concat('\nweekdays=', weekdaynames(NEW.weekdays))),
           if(NEW.mdays = '', '', concat('\nmdays=', getrange(NEW.mdays))),
           if(NEW.timefrom = '00:00:00' AND NEW.timeto = '23:59:59', '',
              concat('\ntimefrom=', NEW.timefrom, '\ntimeto=', coalesce(@suspensiontimeto,NEW.timeto))),
           '\nremark=', REPLACE(REPLACE(NEW.remark, '\\', '\\\\'), '\n', '\\n'))
  from groupsuspensions
  where groupsuspensions.ID = NEW.ID and NEW.nextdayID = 0 and NEW.TS = groupsuspensions.TS;
end//


drop trigger if exists groupsuspensions_ai//
create trigger groupsuspensions_ai after insert on groupsuspensions
  for each row 
begin
  insert into changeprot
    (groupID, accessID, changedtable, tableID, userID, username, action,
     domain, classes, server, section, teststep, details, unit, newvalues)
  select NEW.groupID, NEW.accessID, 'groupsuspensions', NEW.ID,
    coalesce(@userid,0), coalesce(@username,@srvname,'SYSTEM'), 'insert',
    showreg(NEW.domain, NEW.domain_v), showreg(NEW.classes, NEW.classes_v), showreg(NEW.server, NEW.server_v),
    showreg(NEW.section, NEW.section_v), showreg(NEW.teststep, NEW.teststep_v), showreg(NEW.details, NEW.details_v),
    showreg(NEW.einheit, NEW.einheit_v),
    concat('datefrom=', NEW.datefrom,
           '\ndateto=', NEW.dateto,
           if(NEW.weekdays = '0,1,2,3,4,5,6', '', concat('\nweekdays=', weekdaynames(NEW.weekdays))),
           if(NEW.mdays = '', '', concat('\nmdays=', getrange(NEW.mdays))),
           if(NEW.timefrom = '00:00:00' AND NEW.timeto = '23:59:59', '',
              concat('\ntimefrom=', NEW.timefrom, '\ntimeto=', coalesce(@suspensiontimeto,NEW.timeto))),
           '\nremark=', REPLACE(REPLACE(NEW.remark, '\\', '\\\\'), '\n', '\\n'))
  from dual
  where NEW.nextdayID = 0;
end//


drop trigger if exists groupsuspensions_bd//
create trigger groupsuspensions_bd before delete on groupsuspensions
  for each row 
begin
  insert into changeprot
    (groupID, accessID, changedtable, tableID, userID, username, action,
     domain, classes, server, section, teststep, details, unit, newvalues)
  select OLD.groupID, OLD.accessID, 'groupsuspensions', OLD.ID,
    coalesce(@userid,0), coalesce(@username,@srvname,'SYSTEM'), 'delete',
    showreg(OLD.domain, OLD.domain_v), showreg(OLD.classes, OLD.classes_v), showreg(OLD.server, OLD.server_v),
    showreg(OLD.section, OLD.section_v), showreg(OLD.teststep, OLD.teststep_v), showreg(OLD.details, OLD.details_v),
    showreg(OLD.einheit, OLD.einheit_v),
    coalesce(if(OLD.remark = @suspensionremark, '', concat('remark=', REPLACE(REPLACE(@suspensionremark, '\\', '\\\\'), '\n', '\\n'))),'')
    from dual
    where OLD.nextdayID = 0;
end//

delimiter ;
