set @@character_set_client = 'utf8';

delimiter //

drop trigger if exists isalive_au//
create trigger isalive_au after update on isalive
  for each row
begin
  insert into changeprot
    (serverID, accessID, changedtable, tableID, domautoID, groupautoID,
     userID, username, action, server, section, teststep, details, unit, newvalues)
  select serverID, accessID, 'isalive', isalive.ID, autoisaliveID,
    groupautoID, coalesce(@userid,0), coalesce(@username,'SYSTEM'), 'update',
    servername, section.name, teststep.name, teststepdetails.name, einheiten.name,
    concat('period=', maxperiods, '\nperiodunit=', intervalle.name, '\nmsglevel=', msglevel,
           '\nnotifymode=', notifymode, '\nautoclose=', autoclose, '\nmessage=', REPLACE(REPLACE(meldung, '\\', '\\\\'), '\n', '\\n'),
           '\ndestination1=', notifdestname(meldungszielID), '\ndestination2=', notifdestname(meldungsziel2ID),
           '\nremark=', REPLACE(REPLACE(bemerkungen, '\\', '\\\\'), '\n', '\\n'), '\nactive=', aktiv, if(hidden, '\n[hidden]', ''),
           if(groupautoID > OLD.groupautoID, '\nconverted to group-auto', ''),
           if(autoisaliveID > OLD.autoisaliveID, '\nconverted to domain-auto', ''))
  from isalive
  join werteindizes on werteindizes.ID = werteindizesID
  join server on server.ID = werteindizes.serverID
  join section on section.ID = werteindizes.sectionID
  join teststep on teststep.ID = werteindizes.teststepID
  join teststepdetails on teststepdetails.ID = teststepdetailsID
  join einheiten on einheiten.ID = teststepdetails.einheitID
  join intervalle on intervalle.ID = intervalleID
  where isalive.ID = NEW.ID and isalive.TS = NEW.TS;
end//


drop trigger if exists isalive_ai//
create trigger isalive_ai after insert on isalive
  for each row
begin
  insert into tisalive (ID) select NEW.ID;
  insert into changeprot
    (serverID, accessID, changedtable, tableID, domautoID, groupautoID,
     userID, username, action, server, section, teststep, details, unit, newvalues)
  select serverID, accessID, 'isalive', NEW.ID, NEW.autoisaliveID,
    NEW.groupautoID, coalesce(@userid,0), coalesce(@username,'SYSTEM'), 'insert',
    servername, section.name, teststep.name, teststepdetails.name, einheiten.name,
    concat('period=', NEW.maxperiods, '\nperiodunit=', intervalle.name, '\nmsglevel=', NEW.msglevel,
           '\nnotifymode=', NEW.notifymode, '\nautoclose=', NEW.autoclose, '\nmessage=', REPLACE(REPLACE(NEW.meldung, '\\', '\\\\'), '\n', '\\n'),
           '\ndestination1=', notifdestname(NEW.meldungszielID), '\ndestination2=', notifdestname(NEW.meldungsziel2ID),
           '\nremark=', REPLACE(REPLACE(NEW.bemerkungen, '\\', '\\\\'), '\n', '\\n'), '\nactive=', NEW.aktiv,
           if(NEW.hidden, '\n[hidden]', ''))
  from werteindizes
  join server on server.ID = werteindizes.serverID
  join section on section.ID = werteindizes.sectionID
  join teststep on teststep.ID = werteindizes.teststepID
  join teststepdetails on teststepdetails.ID = teststepdetailsID
  join einheiten on einheiten.ID = teststepdetails.einheitID
  join intervalle on intervalle.ID = NEW.intervalleID
  where werteindizes.ID = NEW.werteindizesID;
end//


drop trigger if exists isalive_bd//
create trigger isalive_bd before delete on isalive
  for each row
begin
  delete from tisalive where tisalive.ID = OLD.ID;
  insert into changeprot
    (serverID, accessID, changedtable, tableID, domautoID, groupautoID,  userID, username, action,
     server, section, teststep, details, unit, newvalues)
  select serverID, accessID, 'isalive', OLD.ID, OLD.autoisaliveID,
    OLD.groupautoID, coalesce(@userid,0), coalesce(@username,'SYSTEM'), 'delete',
    servername, section.name, teststep.name, teststepdetails.name, einheiten.name,
    coalesce(if(OLD.bemerkungen = @isaliveremark, '', concat('remark=', REPLACE(REPLACE(@isaliveremark, '\\', '\\\\'), '\n', '\\n'))),'')
  from werteindizes
  join server on server.ID = werteindizes.serverID
  join section on section.ID = werteindizes.sectionID
  join teststep on teststep.ID = werteindizes.teststepID
  join teststepdetails on teststepdetails.ID = teststepdetailsID
  join einheiten on einheiten.ID = teststepdetails.einheitID
  where werteindizes.ID = OLD.werteindizesID;
end//

delimiter ;
