-- kautos.sql
--
-- procedure kautos(lwi, cls, srv, sec, tst, gfid, per, det, eh, vid, aid)
-- lwi: lwerteindizesID
-- cls: class
-- srv: servername
-- sec: section
-- tst: teststep
-- gfid: groupfunctionID
-- per: hour|day|week|month
-- det: detail
-- eh: unit
-- vid: domainID
-- aid: accessID
--
-- create compression from [group]auto-compression on new values


-- lwertelimits anlegen aus autollimits
-- und gautoclimits
--
-- Voraussetzung: loggingtmp.grights
--                  groupID, slavegroupID

set @@character_set_client = 'utf8';

delimiter //

DROP procedure IF EXISTS kautos//
create procedure kautos(lwi integer unsigned, cls varchar(50), srv varchar(50),
                        sec varchar(100), tst varchar(600),
                        gfid integer unsigned, per varchar(10),
                        det varchar(100), eh varchar(100),
                        vid integer unsigned, aid integer unsigned)
  deterministic
  sql security invoker
 begin
  declare mt, gmt integer unsigned;

  -- Domain Limits
  insert into lwertelimits
    (lwerteindizesID, autollimitsID, infowert, warnwert, errorwert, notifymode, vergleich,
     groupfunctionsID, anzmittel, meldung, meldungszielID, ziel1level,
     meldungsziel2ID, ziel2level, bemerkungen)
    select lwi, a.ID, a.infowert, a.warnwert, a.errorwert, a.notifymode, a.vergleich,
      a.groupfunctionsID, a.anzmittel, a.meldung, a.meldungszielID, a.ziel1level,
      a.meldungsziel2ID, a.ziel2level, a.bemerkungen
    from autollimits as a
    left join lwertelimits
      on lwertelimits.autollimitsID = a.ID and lwertelimits.lwerteindizesID=lwi
    where verfahrenID = vid and a.aktiv = '1' and testreg(cls, classes, classes_v)
      and testreg(srv, server, server_v) and testreg(sec, section, section_v)
      and testreg(tst, teststep, teststep_v)
      and testreg(concat((select code from groupfunctions where ID=gfid),
                         '[', left(per, 1), '](', det, ')'),
                  details, details_v) and testreg(eh, einheit, einheit_v)
      and a.accessID = aid and lwertelimits.autollimitsID IS NULL;

  -- Group Limits
  insert into lwertelimits
    (lwerteindizesID, groupautoID, infowert, warnwert, errorwert, notifymode, vergleich,
     groupfunctionsID, anzmittel, meldung, meldungszielID, ziel1level,
     meldungsziel2ID, ziel2level, bemerkungen, slavegroupID)
    select lwi, ga.ID, ga.infowert, ga.warnwert, ga.errorwert, ga.notifymode, ga.vergleich,
      ga.groupfunctionsID, ga.anzmittel, ga.meldung, ga.meldungszielID, ga.ziel1level,
      ga.meldungsziel2ID, ga.ziel2level, ga.bemerkungen, gr.slavegroupID
    from gautoclimits as ga
    join loggingtmp.grights as gr using(groupID)
    left join lwertelimits
      on lwertelimits.groupautoID = ga.ID and lwertelimits.lwerteindizesID = lwi
        and lwertelimits.slavegroupID <=> gr.slavegroupID
    where ga.accessID = aid and testreg(cls, classes, classes_v)
      and testreg(srv, server, server_v) and testreg(sec, section, section_v)
      and testreg(tst, teststep, teststep_v)
      and testreg(concat((select code from groupfunctions where ID=gfid),
                         '[', left(per, 1), '](', det, ')'),
                  details, details_v) and testreg(eh, einheit, einheit_v)
      and lwertelimits.groupautoID IS NULL;

  -- hidden on no-other
  if 0 < (select count(*)
          from lwertelimits as l
          left join gautoclimits as ga on ga.ID = l.groupautoID and not ga.nootherflag
          where l.lwerteindizesID = lwi and (ga.ID is not NULL OR autollimitsID > 0)) then
    update lwertelimits as l
      join gautoclimits as ga on ga.ID = l.groupautoID and ga.nootherflag
      set l.hidden = true
      where l.lwerteindizesID = lwi;
  end if;

  -- Hidden on overridemaster
  if 0 < (select count(*)
          from lwertelimits as l
          where l.lwerteindizesID = lwi and l.slavegroupID is not NULL) then
    update lwertelimits as l
      join gautoclimits as ga on ga.ID = l.groupautoID
      join (select gai.groupID
            from lwertelimits as li
            join gautoclimits as gai
              on gai.ID = li.groupautoID and gai.overridemaster
            where li.lwerteindizesID = lwi) as sg
        on sg.groupID = l.slavegroupID
      set l.hidden = true
      where l.lwerteindizesID = lwi;
  end if;

end//

GRANT EXECUTE on PROCEDURE kautos to logging@'localhost'//
GRANT EXECUTE on PROCEDURE kautos to logginglesen@'localhost'//

