-- limgrpstat.sql
--
-- procedure limgrpstat(limgrpid)
-- limgrpid: limitgroupID
--
-- update status of limitgroup limitgroupID

-- Status-Tabellen der Limitgruppen updaten
--
-- Bedeutung level:
--  0: new
--  1: none
--  2: preinfo
--  3: suspended
--  4-6: info - error
--
--
-- Beteiligte Tabellen
--
--  limgrpstat       Liste der letzten Aktualisierungen der Limitgruppen Stati
--    limitgroupID:  ID der Limitgruppe
--    TS:            Zeitpunkt der Aktualisierung
--    workerid:      connection_id der Verbindung bei Aktualisierung, sonst 0
--  Werte der Limitgruppe limitgroupID werden maximal alle 2 Minuten aktualisiert.
--  Wenn workerid für mehr als 3 Minunte gesetzt ist, dann wird trotzdem aktualisiert.
--
--  limgrpstats      aktuelle Level pro Limitgruppe, Verfahren, Klasse
--    limitgroupID:  ID der Limitgruppe
--    verfahrenID:   ID der Domain
--    classID:       ID der Serverklassen
--    level:         aktueller Level
--    lasttestdate:  Zeitpunkt des letzten Tests
--
--  limgrpelemstats  Stati pro Limitgruppe, Element (oder Limits eines Masters), Verfahren, Klasse
--    limitgroupID:  ID der Limitgruppe
--    limigrpelemID: ID des Limitgruppenelements
--    limtype:       Limittyp
--    groupautoID:   zugeordnetes Gruppenlimit
--    verfahrenID:   ID der Domain
--    classID:       ID der Serverklassen
--    level:         aktueller Level
--    lasttestdate:  Zeitpunkt des letzten Tests

set @@character_set_client = 'utf8';

delimiter //

DROP procedure IF EXISTS limgrpstat//
create procedure limgrpstat(limgrpid integer unsigned)
  deterministic
  sql security invoker
 begin

  if not exists (select * from limgrpstat where limgrpstat.limitgroupID = limgrpid) then
    insert ignore into limgrpstat (limitgroupID, workerid, TS) values (limgrpid, CONNECTION_ID(), UTC_TIMESTAMP());
  else
    update limgrpstat set workerid = CONNECTION_ID(), TS = UTC_TIMESTAMP()
      where limitgroupID = limgrpid
        and ((workerid = 0 AND UTC_TIMESTAMP() > TS + interval 2 minute) OR UTC_TIMESTAMP() > TS + interval 3 minute);
  end if;

  if (select workerid from limgrpstat where limgrpstat.limitgroupID = limgrpid) = CONNECTION_ID() then

    create temporary table loggingtmp.newlimgrpstats (primary key(verfahrenID, classID))
      select v.ID as verfahrenID, c.ID as classID, 0 as level, now() as grptestdate, limitgroup.groupID, v.name as domname
      from limitgroup
      join gruppenrechte as gr on gr.gruppenID = limitgroup.groupID and gr.accessID = 1 and gr.admin = '1'
      join verfahren as v on v.ID = gr.verfahrenID and testreg(v.name, domain, domain_v)
      join serverclasses as c
      where limitgroup.ID = limgrpid;

    create temporary table loggingtmp.limgrpelement
        (primary key (ID, limtype, groupautoID))
      select ID, limtype, groupautoID
      from limitgroupelement
      where limitgroupID = limgrpid and active and limtype != 'M';
    -- bei limtype = 'M' ist limitgroupelement.groupautoID die Gruppen-ID der Vorlagengruppe
    insert into loggingtmp.limgrpelement
      (ID, limtype, groupautoID)
      select limitgroupelement.ID, 'L' as limtype, ga.ID as groupautoID
      from limitgroupelement
      join gautolimits as ga
        on ga.groupID = limitgroupelement.groupautoID and ga.aktiv = '1'
      where limitgroupID = limgrpid and limitgroupelement.active and limtype = 'M';
    insert into loggingtmp.limgrpelement
      (ID, limtype, groupautoID)
      select limitgroupelement.ID, 'I' as limtype, ga.ID as groupautoID
      from limitgroupelement
      join gautoisalive as ga
        on ga.groupID = limitgroupelement.groupautoID and ga.aktiv = '1'
      where limitgroupID = limgrpid and limitgroupelement.active and limtype = 'M';
    insert into loggingtmp.limgrpelement
      (ID, limtype, groupautoID)
      select limitgroupelement.ID, 'C' as limtype, ga.ID as groupautoID
      from limitgroupelement
      join gautocombilimits as ga
        on ga.groupID = limitgroupelement.groupautoID and ga.aktiv = '1'
      where limitgroupID = limgrpid and limitgroupelement.active and limtype = 'M';

    create temporary table loggingtmp.newlimgrpelemstats
        (primary key (limgrpelemID, limtype, groupautoID, verfahrenID, classID))
      select limgrpelemID, limtype, groupautoID, verfahrenID, classID,
        case when mingood > 0 then
          case when nok >= mingood then if(warngood > 0 AND nok < warngood, 2, 1)
               when nok+suspensions >= mingood then 3
               when nok+suspensions+nil >= mingood then 4
               when nok+suspensions+nil+nwl >= mingood then 5
               else 6 end
        when nel > 0 then 6
        when nwl > 0 then 5
        when nil > 0 then 4
        when suspensions > 0 then 3
        when nok > 0 then 1
        else 0 end as level, lasttestdate
      from
      ( select lgt.ID as limgrpelemID, lgt.limtype, lgt.groupautoID, nlgs.verfahrenID, nlgs.classID, lge.mingood, lge.warngood,
          sum(up.verfahrenID is NULL and gsp.groupID is NULL and twl.msglevel = 'none') as nok,
          sum(up.verfahrenID is NULL and gsp.groupID is NULL and twl.msglevel = 'info') as nil,
          sum(up.verfahrenID is NULL and gsp.groupID is NULL and twl.msglevel = 'warn') as nwl,
          sum(up.verfahrenID is NULL and gsp.groupID is NULL and twl.msglevel = 'error') as nel,
          sum(up.verfahrenID is not NULL OR gsp.groupID is not NULL) as suspensions,
          max(lasttestdate) as lasttestdate
        from loggingtmp.newlimgrpstats as nlgs
        join loggingtmp.limgrpelement as lgt on lgt.limtype = 'L'
        join limitgroupelement as lge on lge.ID = lgt.ID
        join gautolimits as ga on ga.ID = lgt.groupautoID and ga.aktiv = '1'
        join wertelimits as wl on wl.groupautoID = ga.ID and wl.aktiv = '1' and wl.vergleich != '' and not wl.hidden
        join twertelimits as twl on twl.ID = wl.ID
        join werteindizes as w on w.ID = wl.werteindizesID
        join server as s
          on s.verfahrenID = nlgs.verfahrenID and s.serverclassesID = nlgs.classID
            and s.ID = w.serverID and s.state = 'active'
            and testreg(s.servername, lge.server, lge.server_v)
        join serverclasses as c on c.ID = serverclassesID and testreg(c.name, lge.classes, lge.classes_v)
        join verfahren as v on v.ID = s.verfahrenID and testreg(v.name, lge.domain, lge.domain_v)
        join section as e on e.ID = w.sectionID and testreg(e.name, lge.section, lge.section_v)
        join teststep as t on t.ID = w.teststepID and testreg(t.name, lge.teststep, lge.teststep_v)
        join teststepdetails as d on d.ID = teststepdetailsID
        join einheiten as u on u.ID = einheitID
        left join ueberwachungspausen as up
          on up.verfahrenID = s.verfahrenID and up.accessID = w.accessID
            and testreg(c.name, up.classes, up.classes_v)
            and testreg(s.servername, up.server, up.server_v)
            and testreg(e.name, up.section, up.section_v)
            and testreg(t.name, up.teststep, up.teststep_v)
            and testreg(d.name, up.details, up.details_v)
            and testreg(u.name, up.einheit, up.einheit_v)
            and up.datumvon <= now() and now() <= up.datumbis
            and find_in_set(weekday(curdate()), up.weekdays)
            and up.zeitvon <= curtime() and curtime() <= zeitbis
            and (up.mdays = '' OR find_in_set(day(curdate()), up.mdays))
        left join groupsuspensions as gsp
          on gsp.groupID = nlgs.groupID and gsp.accessID = w.accessID
            and testreg(nlgs.domname, gsp.domain, gsp.domain_v)
            and testreg(c.name, gsp.classes, gsp.classes_v)
            and testreg(s.servername, gsp.server, gsp.server_v)
            and testreg(e.name, gsp.section, gsp.section_v)
            and testreg(t.name, gsp.teststep, gsp.teststep_v)
            and testreg(d.name, gsp.details, gsp.details_v)
            and testreg(u.name, gsp.einheit, gsp.einheit_v)
            and gsp.datefrom <= now() and now() <= gsp.dateto
            and find_in_set(weekday(curdate()), gsp.weekdays)
            and gsp.timefrom <= curtime() and curtime() <= gsp.timeto
            and (gsp.mdays = '' OR find_in_set(day(curdate()), gsp.mdays))
        where gsp.groupID is NULL OR not gsp.hidesuspended
        group by limgrpelemID, limtype, groupautoID, verfahrenID, classID) as n;

    insert into loggingtmp.newlimgrpelemstats (limgrpelemID, limtype, groupautoID, verfahrenID, classID, level, lasttestdate)
      select limgrpelemID, limtype, groupautoID, verfahrenID, classID,
        case when mingood > 0 then
          case when nok >= mingood then if(warngood > 0 AND nok < warngood, 2, 1)
               when nok+suspensions >= mingood then 3
               when nok+suspensions+nil >= mingood then 4
               when nok+suspensions+nil+nwl >= mingood then 5
               else 6 end
        when nel > 0 then 6
        when nwl > 0 then 5
        when nil > 0 then 4
        when suspensions > 0 then 3
        when nok > 0 then 1
        else 0 end as level, lasttestdate
      from
      ( select lgt.ID as limgrpelemID, lgt.limtype, lgt.groupautoID, nlgs.verfahrenID, nlgs.classID, lge.mingood, lge.warngood,
          sum(up.verfahrenID is NULL and tcmbl.msglevel = 'none') as nok,
          sum(up.verfahrenID is NULL and tcmbl.msglevel = 'info') as nil,
          sum(up.verfahrenID is NULL and tcmbl.msglevel = 'warn') as nwl,
          sum(up.verfahrenID is NULL and tcmbl.msglevel = 'error') as nel,
          sum(up.verfahrenID is not NULL) as suspensions,
          max(lasttestdate) as lasttestdate
        from loggingtmp.newlimgrpstats as nlgs
        join loggingtmp.limgrpelement as lgt on lgt.limtype = 'C'
        join limitgroupelement as lge on lge.ID = lgt.ID
        join gautocombilimits as ga on ga.ID = lgt.groupautoID
        join kombilimits as cmbl on cmbl.groupautoID = ga.ID and cmbl.aktiv = '1' and cmbl.relation != '' and not cmbl.hidden
        join tkombilimits as tcmbl on tcmbl.ID = cmbl.ID
        join server as s
          on s.verfahrenID = nlgs.verfahrenID and s.serverclassesID = nlgs.classID
            and s.ID = cmbl.serverID and s.state = 'active'
            and testreg(s.servername, lge.server, lge.server_v)
        join serverclasses as c on c.ID = serverclassesID and testreg(c.name, lge.classes, lge.classes_v)
        join verfahren as v on v.ID = s.verfahrenID and testreg(v.name, lge.domain, lge.domain_v)
        join section as e on e.ID = cmbl.sectionID and testreg(e.name, lge.section, lge.section_v)
        join teststep as t on t.ID = cmbl.teststepID and testreg(t.name, lge.teststep, lge.teststep_v)
        left join ueberwachungspausen as up
          on up.verfahrenID = s.verfahrenID and up.accessID = cmbl.accessID
            and testreg(c.name, up.classes, up.classes_v)
            and testreg(s.servername, up.server, up.server_v)
            and testreg(e.name, up.section, up.section_v)
            and testreg(t.name, up.teststep, up.teststep_v)
            and datumvon <= now() and now() <= datumbis
            and find_in_set(weekday(curdate()), up.weekdays)
            and zeitvon <= curtime() and curtime() <= zeitbis
            and (up.mdays = '' OR find_in_set(day(curdate()), up.mdays))
        left join groupsuspensions as gsp
          on gsp.groupID = nlgs.groupID and gsp.accessID = cmbl.accessID
            and testreg(nlgs.domname, gsp.domain, gsp.domain_v)
            and testreg(c.name, gsp.classes, gsp.classes_v)
            and testreg(s.servername, gsp.server, gsp.server_v)
            and testreg(e.name, gsp.section, gsp.section_v)
            and testreg(t.name, gsp.teststep, gsp.teststep_v)
            and gsp.datefrom <= now() and now() <= gsp.dateto
            and find_in_set(weekday(curdate()), gsp.weekdays)
            and gsp.timefrom <= curtime() and curtime() <= gsp.timeto
            and (gsp.mdays = '' OR find_in_set(day(curdate()), gsp.mdays))
        where gsp.groupID is NULL OR not gsp.hidesuspended
        group by limgrpelemID, limtype, groupautoID, verfahrenID, classID) as n;

    insert into loggingtmp.newlimgrpelemstats (limgrpelemID, limtype, groupautoID, verfahrenID, classID, level, lasttestdate)
      select limgrpelemID, limtype, groupautoID, verfahrenID, classID,
        case when mingood > 0 then
          case when nok >= mingood then if(warngood > 0 AND nok < warngood, 2, 1)
               when nok+suspensions >= mingood then 3
               when nok+suspensions+nil >= mingood then 4
               when nok+suspensions+nil+nwl >= mingood then 5
               else 6 end
        when nel > 0 then 6
        when nwl > 0 then 5
        when nil > 0 then 4
        when suspensions > 0 then 3
        else 1 end as level, lasttestdate
      from
      ( select lgt.ID as limgrpelemID, lgt.limtype, lgt.groupautoID, nlgs.verfahrenID, nlgs.classID, lge.mingood, lge.warngood,
          sum(up.verfahrenID is NULL and msgdate <= greatest(lastdatum,seen)) as nok,
          sum(up.verfahrenID is NULL and il.msglevel = 'info' and msgdate > greatest(lastdatum, seen)) as nil,
          sum(up.verfahrenID is NULL and il.msglevel = 'warn' and msgdate > greatest(lastdatum, seen)) as nwl,
          sum(up.verfahrenID is NULL and il.msglevel = 'error' and msgdate > greatest(lastdatum, seen)) as nel,
          sum(up.verfahrenID is not NULL) as suspensions, max(lastdatum) as lasttestdate
        from loggingtmp.newlimgrpstats as nlgs
        join loggingtmp.limgrpelement as lgt on lgt.limtype = 'I'
        join limitgroupelement as lge on lge.ID = lgt.ID
        join gautoisalive as ga on ga.ID = lgt.groupautoID and ga.aktiv = '1'
        join isalive as il on il.groupautoID = ga.ID and il.aktiv = '1' and not il.hidden
        join tisalive as til on til.ID = il.ID
        join werteindizes as w on w.ID = il.werteindizesID
        join letztewerte as lw on lw.ID = w.ID
        join server as s
          on s.verfahrenID = nlgs.verfahrenID and s.serverclassesID = nlgs.classID
            and s.ID = w.serverID and s.state = 'active'
            and testreg(s.servername, lge.server, lge.server_v)
        join serverclasses as c on c.ID = serverclassesID and testreg(c.name, lge.classes, lge.classes_v)
        join verfahren as v on v.ID = s.verfahrenID and testreg(v.name, lge.domain, lge.domain_v)
        join section as e on e.ID = w.sectionID and testreg(e.name, lge.section, lge.section_v)
        join teststep as t on t.ID = w.teststepID and testreg(t.name, lge.teststep, lge.teststep_v)
        join teststepdetails as d on d.ID = teststepdetailsID
        join einheiten as u on u.ID = einheitID
        left join ueberwachungspausen as up
          on up.verfahrenID = s.verfahrenID and up.accessID = w.accessID
            and testreg(c.name, up.classes, up.classes_v)
            and testreg(s.servername, up.server, up.server_v)
            and testreg(e.name, up.section, up.section_v)
            and testreg(t.name, up.teststep, up.teststep_v)
            and testreg(d.name, up.details, up.details_v)
            and testreg(u.name, up.einheit, up.einheit_v)
            and datumvon <= now() and now() <= datumbis
            and find_in_set(weekday(curdate()), up.weekdays)
            and zeitvon <= curtime() and curtime() <= zeitbis
            and (up.mdays = '' OR find_in_set(day(curdate()), up.mdays))
        left join groupsuspensions as gsp
          on gsp.groupID = nlgs.groupID and gsp.accessID = w.accessID
            and testreg(nlgs.domname, gsp.domain, gsp.domain_v)
            and testreg(c.name, gsp.classes, gsp.classes_v)
            and testreg(s.servername, gsp.server, gsp.server_v)
            and testreg(e.name, gsp.section, gsp.section_v)
            and testreg(t.name, gsp.teststep, gsp.teststep_v)
            and testreg(d.name, gsp.details, gsp.details_v)
            and testreg(u.name, gsp.einheit, gsp.einheit_v)
            and gsp.datefrom <= now() and now() <= gsp.dateto
            and find_in_set(weekday(curdate()), gsp.weekdays)
            and gsp.timefrom <= curtime() and curtime() <= gsp.timeto
            and (gsp.mdays = '' OR find_in_set(day(curdate()), gsp.mdays))
        where gsp.groupID is NULL OR not gsp.hidesuspended
        group by limgrpelemID, limtype, groupautoID, verfahrenID, classID) as n;
    
    delete limgrpelemstats
      from limgrpelemstats
      left join loggingtmp.newlimgrpelemstats as n
        on n.limgrpelemID = limgrpelemstats.limgrpelemID and n.limtype = limgrpelemstats.limtype
          and n.groupautoID = limgrpelemstats.groupautoID and n.verfahrenID = limgrpelemstats.verfahrenID
          and n.classID = limgrpelemstats.classID
      where limgrpelemstats.limitgroupID = limgrpid and n.limgrpelemID is NULL;
    update limgrpelemstats as lg
      join loggingtmp.newlimgrpelemstats as n
        on n.limgrpelemID = lg.limgrpelemID and n.limtype = lg.limtype and n.groupautoID = lg.groupautoID
          and n.verfahrenID = lg.verfahrenID and n.classID = lg.classID
      set lg.level = n.level, lg.lasttestdate = n.lasttestdate
      where lg.limitgroupID = limgrpid;
    insert into limgrpelemstats (limitgroupID, limgrpelemID, limtype, groupautoID, verfahrenID, classID, level, lasttestdate)
      select limgrpid, n.limgrpelemID, n.limtype, n.groupautoID, n.verfahrenID, n.classID, n.level, n.lasttestdate
      from loggingtmp.newlimgrpelemstats as n
      left join limgrpelemstats as lg
        on lg.limgrpelemID = n.limgrpelemID and n.limtype = lg.limtype and n.groupautoID = lg.groupautoID
          and lg.verfahrenID = n.verfahrenID and lg.classID = n.classID
      where lg.limitgroupID is NULL;

    delete loggingtmp.newlimgrpstats
    from loggingtmp.newlimgrpstats
    left join (select distinct verfahrenID, classID
               from loggingtmp.newlimgrpelemstats) as n
      using (verfahrenID, classID)
    where n.verfahrenID is NULL;

    delete limgrpstats
      from limgrpstats
      left join loggingtmp.newlimgrpstats using(verfahrenID, classID)
      where limgrpstats.limitgroupID = limgrpid and newlimgrpstats.verfahrenID is NULL;
    update loggingtmp.newlimgrpstats
      join (select verfahrenID, classID, max(level) as level, max(lasttestdate) as lasttestdate
            from loggingtmp.newlimgrpelemstats
            group by verfahrenID, classID) as ml
        using(verfahrenID, classID)
      set newlimgrpstats.level = ml.level, newlimgrpstats.grptestdate = ml.lasttestdate;
    update limgrpstats
      join loggingtmp.newlimgrpstats as n using(verfahrenID, classID)
      set limgrpstats.level = n.level, limgrpstats.lasttestdate = n.grptestdate
      where limgrpstats.limitgroupID = limgrpid;
    insert into limgrpstats (limitgroupID, verfahrenID, classID, level, lasttestdate)
      select limgrpid, n.verfahrenID, n.classID, n.level, n.grptestdate
      from loggingtmp.newlimgrpstats as n
      left join limgrpstats as lg
        on lg.limitgroupID = limgrpid and lg.verfahrenID = n.verfahrenID and lg.classID = n.classID
      where lg.verfahrenID is NULL;

    drop temporary table loggingtmp.newlimgrpstats, loggingtmp.newlimgrpelemstats, loggingtmp.limgrpelement;

    update limgrpstat set workerid = 0, TS = UTC_TIMESTAMP()
      where limgrpstat.limitgroupID = limgrpid and workerid = CONNECTION_ID();
  else
    if (select workerid from limgrpstat where limitgroupID = limgrpid) > 0 then
      select sleep(3) into @dummy;
      if (select workerid from limgrpstat where limitgroupID = limgrpid) > 0 then
        select sleep(4) into @dummy;
        if (select workerid from limgrpstat where limitgroupID = limgrpid) > 0 then
          select sleep(6) into @dummy;
        end if;
      end if;
    end if;
  end if;
end//
