-- missingautos.sql
--
-- procedure missingautos()
--
-- check for autocombilimits, grouptautocombilimits, autodetails or groupautodetails
-- with missing details from other tables

-- Wenn sich ein kombilimit/detail auf ein Detail bezieht, das noch nicht vorhanden ist,
-- dann kann es nicht angelegt werden. Missingautos prüft alle zurückgestellten
-- Kombilimits/details
--

set @@character_set_client = 'utf8';

delimiter //

DROP procedure IF EXISTS missingautos//
create procedure missingautos()
  deterministic
  sql security invoker
 begin

  declare dtype, ctype enum('num','str','dat','blob');
  declare si, ei, ti, aid, fdwid, akid, slvgrpid integer unsigned;
  declare z integer;
  declare q, fkt varchar(10000);
  declare cnstvars, uwids varchar(3000);
  declare climcur cursor for
    select a.ID, a.expression, m.serverID, m.sectionID, m.teststepID, m.accessID
    from autocombilimitsmissing as m
    join autokombilimits as a on a.ID = m.autokombilimitsID;
  declare gclimcur cursor for
    select ga.ID, ga.expression, gr.slavegroupID, m.serverID, m.sectionID, m.teststepID, m.accessID
    from gautocombilimitsmissing as m
    join gautocombilimits as ga on ga.ID = m.groupautoID
    join loggingtmp.grights as gr using(groupID);
  declare cdetcur cursor for
    select a.ID, a.formel, einheiten.type, m.serverID, m.sectionID, m.teststepID, m.accessID
    from autocombidetailsmissing as m
    join autokombidetails as a on a.ID = m.autokombidetailsID
    join teststepdetails as td on td.ID = a.teststepdetailsID
    left join einheiten on einheiten.ID = einheitID and einheiten.type != 'str';
  declare gcdetcur cursor for
    select ga.ID, ga.formel, einheiten.type, m.serverID, m.sectionID, m.teststepID, m.accessID
    from gautocombidetailsmissing as m
    join gautocombidetails as ga on ga.ID = m.groupautoID
    join server as s on s.ID = m.serverID
    join gruppenrechte as gr
      on gr.verfahrenID = s.verfahrenID and gr.accessID = m.accessID and gr.gruppenID = ga.groupID
    join teststepdetails as td on td.ID = ga.teststepdetailsID
    left join einheiten on einheiten.ID = einheitID and einheiten.type != 'str';
  declare continue handler for not found set z = 1;

  -- Domain combilimits
  -- inzwichen angelegte oder deaktivierte Kombilimits löschen
  delete autocombilimitsmissing from autocombilimitsmissing
    join kombilimits using(serverID, sectionID, teststepID, accessID, autokombilimitsID);
  delete m from autocombilimitsmissing as m
    left join autokombilimits as a on a.ID = m.autokombilimitsID
    where coalesce(a.aktiv, '0') = '0';
  delete m from autocombilimitsmissing as m
    left join werteindizes as w using(serverID, sectionID, teststepID, accessID)
    where w.serverID is NULL;

  open climcur;
  set z = 0;
  fetch climcur into akid, fkt, si, ei, ti, aid;
  while z != 1 do
    call convert_combi(si, ei, ti, aid, fkt, '', '', q, fdwid, ctype, cnstvars, uwids);
    if q != '' then
      insert into kombilimits 
        (serverID, sectionID, teststepID, expression, expressionsql, relation, infolevel,
         warnlevel, errorlevel, notifymode, meldung, meldungszielID, dest1level,
         meldungsziel2ID, dest2level, aktiv, autokombilimitsID, bemerkungen, accessID, usedwIDs)
        select si, ei, ti, expression, concat('select ', q), relation, infolevel, warnlevel,
          errorlevel, notifymode, meldung, meldungszielID, dest1level, meldungsziel2ID,
          dest2level, aktiv, ID, bemerkungen, accessID, trim(leading ',' from uwids)
        from autokombilimits
        where ID = akid and not exists
          (select * from kombilimits as k
           where k.serverID = si and k.sectionID = ei and k.teststepID = ti
             and k.autokombilimitsID = autokombilimits.ID);
      delete from autocombilimitsmissing
        where serverID = si and sectionID = ei and teststepID = ti and accessID = aid and autokombilimitsID = akid;
    end if;
    set z = 0;
    fetch climcur into akid, fkt, si, ei, ti, aid;
  end while;
  close climcur;

  -- Group combilimits
  -- inzwichen angelegte oder deaktivierte Gruppenkombilimits löschen
  delete gautocombilimitsmissing from gautocombilimitsmissing
    join kombilimits using(serverID, sectionID, teststepID, accessID, groupautoID);
  delete m from gautocombilimitsmissing as m
    left join gautocombilimits as a on a.ID = m.groupautoID
    where coalesce(a.aktiv, '0') = '0';
  delete m from gautocombilimitsmissing as m
    left join werteindizes as w using(serverID, sectionID, teststepID, accessID)
    where w.serverID is NULL;

  if (select count(*) from gautocombilimitsmissing) > 0 then
    create temporary table loggingtmp.grights (unique key uk(verfahrenID, groupID, accessID, slavegroupID))
        select g.verfahrenID, g.gruppenID as groupID, g.accessID, NULL as slavegroupID
        from gruppenrechte as g
        where g.admin = '1'
      union all
        select g.verfahrenID, gm.mastergroupID as groupID, g.accessID, g.gruppenID as slavegroupID
        from gruppenrechte as g
        join masterslavegroups as gm on gm.slavegroupID = g.gruppenID
        where g.admin = '1';
    delete m from gautocombilimitsmissing as m
      join gautocombilimits as ga on ga.ID = m.groupautoID
      left join loggingtmp.grights as gr using(groupID)
      where gr.groupID is NULL;
    open gclimcur;
    set z = 0;
    fetch gclimcur into akid, fkt, slvgrpid, si, ei, ti, aid;
    while z != 1 do
      call convert_combi(si, ei, ti, aid, fkt, '', '', q, fdwid, ctype, cnstvars, uwids);
      if q != '' then
        insert into kombilimits
          (serverID, sectionID, teststepID, expression, expressionsql, relation, infolevel, warnlevel,
           errorlevel, notifymode, meldung, meldungszielID, dest1level, meldungsziel2ID,
           dest2level, aktiv, groupautoID, bemerkungen, accessID, usedwIDs, slavegroupID)
          select si, ei, ti, expression, concat('select ', q), relation, infolevel, warnlevel,
            errorlevel, notifymode, meldung, meldungszielID, dest1level, meldungsziel2ID,
            dest2level, aktiv, ID, bemerkungen, accessID, trim(leading ',' from uwids), slvgrpid
          from gautocombilimits
          where ID = akid and not exists
            (select * from kombilimits as k
             where k.serverID = si and k.sectionID = ei and k.teststepID = ti
               and k.groupautoID = gautocombilimits.ID);
        delete from gautocombilimitsmissing
          where serverID = si and sectionID = ei and teststepID = ti and accessID = aid and groupautoID = akid;
      end if;
      set z = 0;
      fetch gclimcur into akid, fkt, slvgrpid, si, ei, ti, aid;
    end while;
    close gclimcur;
    drop temporary table loggingtmp.grights;
  end if;

  -- Hidden on-other
  if 0 < (select count(*)
          from kombilimits as kl
          left join gautocombilimits as ga on ga.ID = kl.groupautoID and not ga.nootherflag
          where kl.serverID = si and kl.sectionID = ei and kl.teststepID = ti
            and (ga.ID is not NULL OR kl.autokombilimitsID > 0)) then
    update kombilimits as kl
      join gautocombilimits as ga on ga.ID = kl.groupautoID and ga.nootherflag
      set kl.hidden = true
      where kl.serverID = si and kl.sectionID = ei and kl.teststepID = ti;
  end if;

  -- Hidden on overridemaster
  if 0 < (select count(*)
          from kombilimits as kl
          where kl.serverID = si and kl.sectionID = ei and kl.teststepID = ti and kl.slavegroupID is not NULL) then
    update kombilimits as kl
      join gautocombilimits as ga on ga.ID = kl.groupautoID
      join (select gai.groupID
            from kombilimits as kli
            join gautocombilimits as gai
              on gai.ID = kli.groupautoID and gai.overridemaster
            where kli.serverID = si and kli.sectionID = ei and kli.teststepID = ti) as sg
        on sg.groupID = kl.slavegroupID
      set kl.hidden = true
      where kl.serverID = si and kl.sectionID = ei and kl.teststepID = ti;
  end if;

  -- Domain combidetails
  -- inzwichen angelegte oder deaktivierte Kombidetails löschen
  delete autocombidetailsmissing from autocombidetailsmissing
    join kombidetails using(serverID, sectionID, teststepID, accessID, autokombidetailsID);
  delete m from autocombidetailsmissing as m
    join autokombidetails as a on a.ID = m.autokombidetailsID and a.aktiv = '0';
  delete m from autocombidetailsmissing as m
    left join werteindizes as w using(serverID, sectionID, teststepID, accessID)
    where w.serverID is NULL;

  open cdetcur;
  set z = 0;
  fetch cdetcur into akid, fkt, dtype, si, ei, ti, aid;
  while z != 1 do
    call convert_combi(si, ei, ti, aid, fkt, 'b', ',datum', q, fdwid, ctype, cnstvars, uwids);
    if q != '' then
      insert into kombidetails
        (serverID, sectionID, teststepID, teststepdetailsID, formel, formelsql,
         constvars, firstdetwiID, autokombidetailsID, bemerkungen, stringflag,
         aktiv, accessID, usedwIDs)
      select si, ei, ti, teststepdetailsID, formel, q,
        cnstvars, fdwid, ID, bemerkungen, coalesce(dtype, ctype) != 'num', '1', accessID, trim(leading ',' from uwids)
      from autokombidetails
      where ID = akid and not exists
        (select * from kombidetails as k
         where k.serverID = si and k.sectionID = ei and k.teststepID = ti
           and k.autokombidetailsID = autokombidetails.ID);
      delete from autocombidetailsmissing
        where serverID = si and sectionID = ei and teststepID = ti and accessID = aid and autokombidetailsID = akid;
    end if;
    set z = 0;
    fetch cdetcur into akid, fkt, dtype, si, ei, ti, aid;
  end while;
  close cdetcur;

  -- Group combidetails
  -- inzwichen angelegte oder deaktivierte Gruppenkombidetails löschen
  delete gautocombidetailsmissing from gautocombidetailsmissing
    join kombidetails using(serverID, sectionID, teststepID, accessID, groupautoID);
  delete m from gautocombidetailsmissing as m
    left join gautocombidetails as a on a.ID = m.groupautoID
    where coalesce(a.aktiv, '0') = '0';
  delete m from gautocombidetailsmissing as m
    left join werteindizes as w using(serverID, sectionID, teststepID, accessID)
    where w.serverID is NULL;
  delete m from gautocombidetailsmissing as m
    join gautocombidetails as ga on ga.ID = m.groupautoID
    join server as s on s.ID = m.serverID
    left join gruppenrechte as gr
      on gr.verfahrenID = s.verfahrenID and gr.accessID = m.accessID and gr.gruppenID = ga.groupID
    where gr.verfahrenID is NULL;

  open gcdetcur;
  set z = 0;
  fetch gcdetcur into akid, fkt, dtype, si, ei, ti, aid;
  while z != 1 do
    call convert_combi(si, ei, ti, aid, fkt, 'b', ',datum', q, fdwid, ctype, cnstvars, uwids);
    if q != '' then
      insert into kombidetails
        (serverID, sectionID, teststepID, teststepdetailsID, formel, formelsql,
         constvars, firstdetwiID, groupautoID, bemerkungen, stringflag,
         aktiv, accessID, usedwIDs)
      select si, ei, ti, teststepdetailsID, formel, q,
        cnstvars, fdwid, ID, bemerkungen, coalesce(dtype, ctype) != 'num', '1', accessID, trim(leading ',' from uwids)
      from gautocombidetails
      where ID = akid and not exists
        (select * from kombidetails as k
         where k.serverID = si and k.sectionID = ei and k.teststepID = ti
           and k.groupautoID = gautocombidetails.ID);
      delete from gautocombidetailsmissing
        where serverID = si and sectionID = ei and teststepID = ti and accessID = aid and groupautoID = akid;
    end if;
    set z = 0;
    fetch gcdetcur into akid, fkt, dtype, si, ei, ti, aid;
  end while;
  close gcdetcur;

end//

GRANT EXECUTE on PROCEDURE missingautos to loggingadmin@'localhost'//
