-- modelwert.sql
--
-- function modelwert(lwid, anz, mode) returns varchar
-- lwid: lwerteinidzesID
-- anz: number of values
-- mode: aggregation mode
-- return: aggregated value of the last "anz" values before "bis" with werteindizesID == "wid" (table mwerte)

-- combined aggregation details

set @@character_set_client = 'utf8';

delimiter //

DROP FUNCTION IF EXISTS modelwert//
create function modelwert(lwid integer, anz integer, mode varchar(12))
    returns varchar(30)
  deterministic
  sql security invoker
  begin
    declare wl, f, counter, z integer;
    declare w, wsum2, wsum, wmin, wmax, wlast double;
    declare lastdate, d datetime;
    declare selcur cursor for select wert, datum, length(0+replace(wert,'.','')) from lwerte
      where lwerteindizesID = lwid order by datum desc;
    declare continue handler for not found set z = 1;

    if anz = 1 OR mode = 'last' then
      return (select wert from lwerte
              where lwerteindizesID=lwid order by datum desc limit 1);
    end if;

    set counter = 0;
    set z = 0;
    open selcur;
    repeat
      fetch selcur into w, d, wl;
      if z != 1
       then
        if counter = 0 then
          set wmin = w;
          set wmax = w;
          set wsum = w;
          set wsum2 = w*w;
          set wlast = w;
          set f = wl;
        elseif counter = 1 && mode like 'last/avg%' then
          set wsum = w;
          set anz = anz + 1;
        else
          set wsum = wsum + w;
          set wsum2 = wsum2 + w*w;
          if wl > f then
            set f = wl;
          end if;
          if w < wmin then
            set wmin = w;
          end if;
          if w > wmax then
            set wmax = w;
          end if;
        end if;
        set counter = counter + 1;
      end if;
      until (counter >= anz || z = 1)
    end repeat;
    close selcur;
    if counter >= anz && counter > 0
     then
      case mode
        when 'count' then
          return counter;
        when 'avg' then
          set wsum = wsum / counter;
          return round(wsum, f + 2 - locate('.', wsum));
        when 'dev' then
          if counter > 1 then
            set wsum = sqrt((wsum2 - wsum*wsum/counter)/(counter-1));
            return round(wsum, f + 2 - locate('.', wsum));
          else
            return "";
          end if;
        when 'last/avg' then
          if wsum != 0 then
            set wsum = wlast / wsum * (counter-1);
            return cast(round(wsum, f + 2 - locate('.', wsum)) as char(30));
          else
            return "";
          end if;
        when 'last/avg-1' then
          if wsum != 0 AND counter > 1 then
            set wsum = wlast / wsum * (counter-1) - 1;
            return cast(round(wsum, f + 2 - locate('.', wsum)) as char(30));
          else
            return '';
          end if;
        when 'sum' then
          return round(wsum, f + 1 - locate('.', wsum));
        when 'min' then
          return wmin;
        when 'max' then
          return wmax;
        when 'max-min' then
          set wsum = wmax - wmin;
          return round(wsum, f + 1 - locate('.', wsum));
        when 'last-first' then
          set wsum = wlast - w;
          return round(wsum, f + 1 - locate('.', wsum));
        when 'last-first+' then
          set wsum = wlast - w;
          if wsum >= 0 then
            return round(wsum, f + 1 - locate('.', wsum));
          else
            return '';
          end if;
        when 'last-first-' then
          set wsum = wlast - w;
          if wsum <= 0 then
            return round(wsum, f + 1 - locate('.', wsum));
          else
            return '';
          end if;
        when 'first' then
          return w;
        when 'differ' then
          return w != wlast;
        when 'firstsize' then
          return length(w);
        when 'grad' then
          return gradientlvbextfilter(lwid, d, lastdate, '00:00:00', '24:00:00', '',
                        '', '', '', '', '', '', '', '', '', '', '');
        when 'accel' then
          return accell(lwid, d, lastdate, '00:00:00', '24:00:00', '',
                        '', '', '', '', '', '', '', '', '', '', '');
        else
          return "";
      end case;
    else
      return "";
    end if;
  end//

GRANT EXECUTE on FUNCTION modelwert to logginglesen@'localhost', loggingtest@'localhost'//
