-- modelwertb.sql
--
-- function modelwertb(lwid, bis, anz, mode) returns varchar
-- lwid: lwerteindizesID
-- bis: up to datetime
-- anz: number of values
-- mode: aggregation mode
-- return: aggregated value of the last "anz" values before "bis" with werteindizesID == "wid" (table lwerte)

-- get grouped value from lwerte - fixed number of values
--
-- call from
-- combined aggregation details

set @@character_set_client = 'utf8';

delimiter //

DROP FUNCTION IF EXISTS modelwertb//
create function modelwertb(lwid integer, bis datetime, anz integer, mode varchar(12))
    returns varchar(30)
  deterministic
  sql security invoker
  begin
    declare wl, f, counter, z integer;
    declare w, wmin, wmax, wlast varchar(20);
    declare wsum2, wsum double;
    declare selcur cursor for select wert, length(0+replace(wert,'.','')) from lwerte
      where lwerteindizesID = lwid and datum <= bis order by datum desc;
    declare continue handler for not found set z = 1;

    set counter = 0;
    set z = 0;
    if mode = 'size' or mode = 'lastsize' then
      set anz = 1;
      set mode = 'firstsize';
    elseif mode = 'last' then
      set anz = 1;
      set mode = 'first';
    end if;
    open selcur;
    repeat
      fetch selcur into w, wl;
      if z != 1
       then
        if counter = 0 then
          set wmin = w;
          set wmax = w;
          set wsum = w;
          set wsum2 = w*w;
          set wlast = w;
          set f = wl;
        elseif counter = 1 && mode like 'last/avg%' then
          set wsum = w;
          set anz = anz + 1;
        else
          set wsum = wsum + w;
          set wsum2 = wsum2 + w*w;
          if wl > f then
            set f = wl;
          end if;
          if 0+w < 0+wmin then
            set wmin = w;
          end if;
          if 0+w > 0+wmax then
            set wmax = w;
          end if;
        end if;
        set counter = counter + 1;
      end if;
      until (counter >= anz || z = 1)
    end repeat;
    close selcur;
    if counter >= anz && counter > 0
     then
      case mode
        when 'count' then
          return counter;
        when 'avg' then
          set wsum = wsum / counter;
          return round(wsum, f + 2 - locate('.', wsum));
        when 'dev' then
          if counter > 1 then
            set wsum = sqrt((wsum2 - wsum*wsum/counter)/(counter-1));
            return round(wsum, f + 2 - locate('.', wsum));
          else
            return "";
          end if;
        when 'last/avg' then
          if wsum != 0 then
            set wsum = wlast / wsum * (counter-1);
            return cast(round(wsum, f + 2 - locate('.', wsum)) as char(30));
          else
            return '';
          end if;
        when 'last/avg-1' then
          if wsum != 0 AND counter > 1 then
            set wsum = wlast / wsum * (counter-1) - 1;
            return cast(round(wsum, f + 2 - locate('.', wsum)) as char(30));
          else
            return '';
          end if;
        when 'sum' then
          return round(wsum, f + 1 - locate('.', wsum));
        when 'min' then
          return wmin;
        when 'max' then
          return wmax;
        when 'max-min' then
          set wsum = wmax - wmin;
          return round(wsum, f + 1 - locate('.', wsum));
        when 'last-first' then
          set wsum = wlast - w;
          return round(wsum, f + 1 - locate('.', wsum));
        when 'last-first+' then
          set wsum = wlast - w;
          if wsum >= 0 then
            return round(wsum, f + 1 - locate('.', wsum));
          else
            return '';
          end if;
        when 'last-first-' then
          set wsum = wlast - w;
          if wsum <= 0 then
            return round(wsum, f + 1 - locate('.', wsum));
          else
            return '';
          end if;
        when 'first' then
          return w;
        when 'differ' then
          return w != wlast;
        when 'firstsize' then
          return length(w);
        else
          return '';
      end case;
    else
      return '';
    end if;
  end//

GRANT EXECUTE on FUNCTION modelwertb to logginglesen@'localhost', loggingtest@'localhost'//
