-- newcomplimits.sql
--
-- procedure newcomplimits()
--
-- create new compressed values and
-- check compressed autolimits/groupautolimits for compressed values

-- Autos prüfen für neue lwerteinidzes seit der letzten Prüfung:
--  - lwertelimits anlegen
--  - autoclimits, gautoclimits -> kautos.sql
--  - autocclimits, gautocclimits
--  - autoccdetails, gautoccetails

set @@character_set_client = 'utf8';

delimiter //

DROP procedure IF EXISTS newcomplimits//
create procedure newcomplimits()
  deterministic
  sql security invoker
 begin

  declare dom varchar(50);
  declare srv, cls varchar(50);
  declare sec varchar(100);
  declare tst varchar(600);
  declare det, un varchar(100);
  declare newlastwid, lastwid, si, ei, ti, di, aid, vid integer unsigned;
  declare dtype, ctype, wtype enum('num','str','dat','blob');
  declare fdwid, akid, slvgrpid integer unsigned;
  declare z integer;
  declare q, fkt varchar(10000);
  declare cnstvars, uwids varchar(3000);
  declare cclimcur cursor for
    select a.ID, a.expression
    from autocclimits as a
    left join cclimits as k
      on k.serverID = si and k.sectionID = ei and k.teststepID = ti
        and k.accessID = aid and k.domautoID = a.ID
    where a.verfahrenID = vid and a.aktiv and k.ID is NULL and a.accessID = aid
      and testreg(cls, classes, classes_v)
      and testreg(srv, server, server_v) and testreg(sec, section, section_v)
      and testreg(tst, teststep, teststep_v)
      and exists (select * from loggingtmp.ldets where find_in_set(ldets.detail, a.details));
  declare gcclimcur cursor for
    select ga.ID, ga.expression, gr.slavegroupID
    from gautocclimits as ga
    join loggingtmp.grights as gr on gr.verfahrenID = vid and gr.groupID = ga.groupID and gr.accessID = aid
    left join cclimits as k
      on k.serverID = si and k.sectionID = ei and k.teststepID = ti
        and k.accessID = aid and k.groupautoID = ga.ID
    where ga.aktiv and ga.accessID = aid and k.ID is NULL
      and testreg(cls, classes, classes_v)
      and testreg(dom, domain, domain_v) and testreg(srv, server, server_v)
      and testreg(sec, section, section_v) and testreg(tst, teststep, teststep_v)
      and exists (select * from loggingtmp.ldets where find_in_set(ldets.detail, ga.details));
  declare ccdetcur cursor for
    select a.ID, a.formel, einheiten.type
    from autoccdetails as a
    join teststepdetails as td on td.ID = a.teststepdetailsID
    left join einheiten on einheiten.ID = einheitID and einheiten.type != 'str'
    left join ccdetails as k
      on k.serverID = si and k.sectionID = ei and k.teststepID = ti
        and k.accessID = aid and k.domautoID = a.ID
    where a.verfahrenID = vid and a.aktiv = '1' and k.ID is NULL and a.accessID = aid
      and testreg(srv, server, server_v) and testreg(sec, section, section_v)
      and testreg(tst, teststep, teststep_v)
      and exists (select * from loggingtmp.ldets where find_in_set(ldets.detail, a.details));
  declare gccdetcur cursor for
    select ga.ID, ga.formel, einheiten.type
    from gautoccdetails as ga
    join gruppenrechte as gr
      on gr.gruppenID = ga.groupID and gr.verfahrenID = vid and gr.accessID = aid
    join teststepdetails as td on td.ID = ga.teststepdetailsID
    left join einheiten on einheiten.ID = einheitID and einheiten.type != 'str'
    left join ccdetails as k
      on k.serverID = si and k.sectionID = ei and k.teststepID = ti
        and k.accessID = aid and k.groupautoID = ga.ID
    where ga.aktiv = '1' and ga.accessID=aid and k.ID is NULL
      and testreg(dom, domain, domain_v) and testreg(srv, server, server_v)
      and testreg(sec, section, section_v) and testreg(tst, teststep, teststep_v)
      and exists (select * from loggingtmp.ldets where find_in_set(ldets.detail, ga.details));
  declare widscur cursor for
    select verfahrenID, serverID, sectionID, teststepID, teststepdetailsID,
      domain, classname, servername, secname, tstname, detname, unit, `type`, accessID
    from loggingtmp.widsnames
    where compexist
    order by verfahrenID;
  declare cwidscur cursor for
    select verfahrenID, serverID, sectionID, teststepID,
      domain, classname, servername, secname, tstname, accessID
    from loggingtmp.widsnames
    join lwerteindizes using(serverID, sectionID, teststepID, accessID)
    where limexist
    group by serverID, sectionID, teststepID, accessID
    order by verfahrenID;
  declare continue handler for not found set z = 1;

  set newlastwid = (select max(ID) as mid from werteindizes);
  set lastwid = (select wert from systemvars where name = 'lastnewcomplimitsID');

  create temporary table loggingtmp.comps (key domainID(verfahrenID, accessID))
    select distinct verfahrenID, accessID, server, server_v, section, section_v, teststep, teststep_v,
        details, details_v, einheit, einheit_v
      from autokomprimierung where aktiv = '1'
    union distinct
    select distinct gruppenrechte.verfahrenID, gruppenrechte.accessID, server, server_v, section, section_v, teststep, teststep_v,
        '.*' as details, '' as details_v, '.*' as einheit, '' as einheit_v
      from gautocompression
      join gruppenrechte on gruppenrechte.gruppenID = gautocompression.groupID and gruppenrechte.accessID = gautocompression.accessID
      join verfahren on verfahren.ID = gruppenrechte.verfahrenID and testreg(verfahren.name, domain, domain_v)
      where aktiv = '1';

  create temporary table loggingtmp.wilds (key domainID(verfahrenID, accessID))
    select distinct verfahrenID, accessID, server, server_v, section, section_v, teststep, teststep_v
      from autocclimits where aktiv
    union distinct
    select distinct verfahrenID, accessID, server, server_v, section, section_v, teststep, teststep_v
      from autoccdetails where aktiv = '1'
    union distinct
    select distinct gruppenrechte.verfahrenID, gruppenrechte.accessID, server, server_v, section, section_v, teststep, teststep_v
      from gautocclimits
      join gruppenrechte on gruppenrechte.gruppenID = gautocclimits.groupID and gruppenrechte.accessID = gautocclimits.accessID
      join verfahren on verfahren.ID = gruppenrechte.verfahrenID and testreg(verfahren.name, domain, domain_v)
      where aktiv
    union distinct
    select distinct gruppenrechte.verfahrenID, gruppenrechte.accessID, server, server_v, section, section_v, teststep, teststep_v
      from gautoccdetails
      join gruppenrechte on gruppenrechte.gruppenID = gautoccdetails.groupID and gruppenrechte.accessID = gautoccdetails.accessID
      join verfahren on verfahren.ID = gruppenrechte.verfahrenID and testreg(verfahren.name, domain, domain_v)
      where aktiv = '1';

  create temporary table loggingtmp.widsnames (key domainID(verfahrenID, serverID))
    select verfahrenID, serverID, sectionID, teststepID, teststepdetailsID, einheitID,
      verfahren.name as domain, coalesce(serverclasses.name, '') as classname,
      servername, section.name as secname, teststep.name as tstname,
      teststepdetails.name as detname, einheiten.name as unit, einheiten.`type`, accessID, werteindizes.ID as wid,
      exists (select * from loggingtmp.wilds as w
            where w.verfahrenID = verfahren.ID and w.accessID = werteindizes.accessID
              and testreg(servername, server, server_v) and testreg(section.name, section, section_v)
              and testreg(teststep.name, teststep, teststep_v)) as limexist,
      exists (select * from loggingtmp.comps as c
            where c.verfahrenID = verfahren.ID and c.accessID = werteindizes.accessID
              and testreg(servername, server, server_v) and testreg(section.name, section, section_v)
              and testreg(teststep.name, teststep, teststep_v) and testreg(teststepdetails.name, details, details_v)
              and testreg(einheiten.name, einheit, einheit_v)) as compexist
    from werteindizes
    join server on server.ID = serverID
    join verfahren on verfahren.ID = verfahrenID
    join section on section.ID = sectionID
    join teststep on teststep.ID = teststepID
    join teststepdetails on teststepdetails.ID = teststepdetailsID
    join einheiten on einheiten.ID = einheitID
    left join serverclasses on serverclasses.ID = server.serverclassesID
    where werteindizes.ID > lastwid
    having limexist or compexist;

  if row_count() > 0 then

    create temporary table loggingtmp.grights (unique key uk(verfahrenID, groupID, accessID, slavegroupID))
        select g.verfahrenID, g.gruppenID as groupID, g.accessID, NULL as slavegroupID
        from gruppenrechte as g
        where g.admin = '1'
      union all
        select g.verfahrenID, gm.mastergroupID as groupID, g.accessID, g.gruppenID as slavegroupID
        from gruppenrechte as g
        join masterslavegroups as gm on gm.slavegroupID = g.gruppenID
        where g.admin = '1';

    open widscur;
    set z = 0;
    fetch widscur into vid, si, ei, ti, di, dom, cls, srv, sec, tst, det, un, wtype, aid;
    while z = 0 do

      -- Domain compression
      if (select count(*) from autokomprimierung
          where verfahrenID = vid and accessID = aid
            and testreg(srv, server, server_v) and testreg(sec, section, section_v)
            and testreg(tst, teststep, teststep_v) and testreg(det, details, details_v)
            and testreg(un, einheit, einheit_v)) > 0 then
        call newkompautos(si, ei, ti, di, cls, srv, sec, tst, det, un, vid, aid, wtype);
      end if;

      -- Group compression
      if (select count(*)
          from gautocompression as ga
          join loggingtmp.grights as gr using(groupID)
          where ga.accessID = aid and testreg(dom, domain, domain_v)
            and testreg(srv, server, server_v) and testreg(sec, section, section_v)
            and testreg(tst, teststep, teststep_v) and testreg(det, details, details_v)
            and testreg(un, einheit, einheit_v)) > 0 then
        call newgcompautos(si, ei, ti, di, cls, srv, sec, tst, det, un, vid, aid, wtype);
      end if;

      set z = 0;
      fetch widscur into vid, si, ei, ti, di, dom, cls, srv, sec, tst, det, un, wtype, aid;
    end while;
    close widscur;

    open cwidscur;
    set z = 0;
    fetch cwidscur into vid, si, ei, ti, dom, cls, srv, sec, tst, aid;
    while z = 0 do

    -- compressed details
      create temporary table loggingtmp.ldets
        select concat(gf.code, '[', left(period, 1), '](', teststepdetails.name, ')') as detail
        from lwerteindizes
        join groupfunctions as gf on gf.ID = lwerteindizes.groupfunctionsID
        join teststepdetails on teststepdetails.ID = teststepdetailsID
        where serverID = si and sectionID = ei and teststepID = ti and accessID = aid;

      -- ---------------------------------------
      -- Domain combilimits
      set z = 0;
      open cclimcur;
      fetch cclimcur into akid, fkt;
      while z != 1 do
        call convert_ccombi(si, ei, ti, aid, fkt, '', '', q, fdwid, ctype, cnstvars, uwids);
        if q != '' then
          insert into cclimits 
            (serverID, sectionID, teststepID, expression, expressionsql, relation, infolevel,
             warnlevel, errorlevel, notifymode, meldung, meldungszielID, dest1level,
             meldungsziel2ID, dest2level, aktiv, domautoID, bemerkungen, accessID, usedwIDs)
            select si, ei, ti, expression, concat('select ', q), relation, infolevel, warnlevel,
              errorlevel, notifymode, meldung, meldungszielID, dest1level, meldungsziel2ID,
              dest2level, aktiv, ID, bemerkungen, accessID, trim(leading ',' from uwids)
            from autocclimits
            where ID = akid and not exists
              (select * from cclimits as k
               where k.serverID = si and k.sectionID = ei and k.teststepID = ti
                 and k.autokombilimitsID = domautocclimits.ID);
        end if;
        set z = 0;
        fetch cclimcur into akid, fkt;
      end while;
      close cclimcur;

      -- Group combilimits
      set z = 0;
      open gcclimcur;
      fetch gcclimcur into akid, fkt, slvgrpid;
      while z != 1 do
        call convert_ccombi(si, ei, ti, aid, fkt, '', '', q, fdwid, ctype, cnstvars, uwids);
        if q != '' then
          insert into cclimits
            (serverID, sectionID, teststepID, expression, expressionsql, relation, infolevel, warnlevel,
             errorlevel, notifymode, meldung, meldungszielID, dest1level, meldungsziel2ID,
             dest2level, aktiv, groupautoID, bemerkungen, accessID, usedwIDs, slavegroupID)
            select si, ei, ti, expression, concat('select ', q), relation, infolevel, warnlevel,
              errorlevel, notifymode, meldung, meldungszielID, dest1level, meldungsziel2ID,
              dest2level, aktiv, ID, bemerkungen, accessID, trim(leading ',' from uwids), slvgrpid
            from gautocclimits
            where ID = akid and not exists
              (select * from cclimits as k
               where k.serverID = si and k.sectionID = ei and k.teststepID = ti
                 and k.groupautoID = gautocclimits.ID);
        end if;
        set z = 0;
        fetch gcclimcur into akid, fkt, slvgrpid;
      end while;
      close gcclimcur;

      -- Hidden on-other
      if 0 < (select count(*)
              from cclimits as kl
              left join gautocclimits as ga on ga.ID = kl.groupautoID and not ga.nootherflag
              where kl.serverID = si and kl.sectionID = ei and kl.teststepID = ti
                and (ga.ID is not NULL OR kl.domautoID > 0)) then
        update cclimits as kl
          join gautocclimits as ga on ga.ID = kl.groupautoID and ga.nootherflag
          set kl.hidden = true
          where kl.serverID = si and kl.sectionID = ei and kl.teststepID = ti;
      end if;

      -- Hidden on overridemaster
      if 0 < (select count(*)
              from cclimits as kl
              where kl.serverID = si and kl.sectionID = ei and kl.teststepID = ti and kl.slavegroupID is not NULL) then
        update cclimits as kl
          join gautocclimits as ga on ga.ID = kl.groupautoID
          join (select gai.groupID
                from cclimits as kli
                join gautocclimits as gai
                  on gai.ID = kli.groupautoID and gai.overridemaster
                where kli.serverID = si and kli.sectionID = ei and kli.teststepID = ti) as sg
            on sg.groupID = kl.slavegroupID
          set kl.hidden = true
          where kl.serverID = si and kl.sectionID = ei and kl.teststepID = ti;
      end if;

      -- Domain compressed details
      set z = 0;
      open ccdetcur;
      fetch ccdetcur into akid, fkt, dtype;
      while z != 1 do
        call convert_ccombi(si, ei, ti, aid, fkt, 'b', ',datum', q, fdwid, ctype, cnstvars, uwids);
        if q != '' then
          insert into ccdetails
            (serverID, sectionID, teststepID, teststepdetailsID, formel, formelsql,
             constvars, firstdetwiID, domautoID, bemerkungen, stringflag,
             aktiv, accessID, usedwIDs)
          select si, ei, ti, teststepdetailsID, formel, q,
            cnstvars, fdwid, ID, bemerkungen, coalesce(dtype, ctype) != 'num', '1', accessID, trim(leading ',' from uwids)
          from autoccdetails
          where ID = akid and not exists
            (select * from ccdetails as c
             where c.serverID = si and c.sectionID = ei and c.teststepID = ti
               and c.domautoID = autoccdetails.ID);
        end if;
        set z = 0;
        fetch ccdetcur into akid, fkt, dtype;
      end while;
      close ccdetcur;
    
      -- Group compressed details
      set z = 0;
      open gccdetcur;
      fetch gccdetcur into akid, fkt, dtype;
      while z != 1 do
        call convert_ccombi(si, ei, ti, aid, fkt, 'b', ',datum', q, fdwid, ctype, cnstvars, uwids);
        if q != '' then
          insert into ccdetails
            (serverID, sectionID, teststepID, teststepdetailsID, formel, formelsql,
             constvars, firstdetwiID, groupautoID, bemerkungen, stringflag,
             aktiv, accessID, usedwIDs)
          select si, ei, ti, teststepdetailsID, formel, q,
            cnstvars, fdwid, ID, bemerkungen, coalesce(dtype, ctype) != 'num', '1', accessID, trim(leading ',' from uwids)
          from gautoccdetails
          where ID = akid and not exists
            (select * from ccdetails as c
             where c.serverID = si and c.sectionID = ei and c.teststepID = ti
               and c.groupautoID = gautoccdetails.ID);
        end if;
        set z = 0;
        fetch gccdetcur into akid, fkt, dtype;
      end while;
      close gccdetcur;

      drop temporary table loggingtmp.ldets;

      set z = 0;
      fetch cwidscur into vid, si, ei, ti, dom, cls, srv, sec, tst, aid;
    end while;
    close cwidscur;

    if newlastwid > lastwid then
      update systemvars set wert = newlastwid where name = 'lastnewcomplimitsID';
    end if;

    drop temporary table loggingtmp.grights;

  end if;
  drop temporary table loggingtmp.widsnames, loggingtmp.wilds, loggingtmp.comps;

end//

GRANT EXECUTE on PROCEDURE newcomplimits to loggingadmin@'localhost'//
