-- newgcompautos.sql
--
-- procedure newgcompautos(si, ei, ti, di, cls, srv, sec, tst, det, eh, vid, aid, wtype)
-- si:  serverID
-- ei:  sectionID
-- ti:  teststepID
-- di:  detailID
-- cls: class
-- srv: servername
-- sec: section
-- tst: teststep
-- det: detail
-- eh: unit
-- vid: domainID
-- aid: accessID
-- wtype: value type
--
-- create compression from group-auto-compression on new values

-- Komprimierungen aus Gruppen-Auto-Komprimierungen einrichten
--
-- wird aufgerufen, wenn ein neuer Werteindex angelegt wurde
--
-- Voraussetzung: loggingtmp.grights
--                  groupID, slavegroupID

set @@character_set_client = 'utf8';

delimiter //

DROP procedure IF EXISTS newgcompautos//
create procedure newgcompautos(si integer unsigned, ei integer unsigned, ti integer unsigned,
                   di integer unsigned, cls varchar(50), srv varchar(50),
                   sec varchar(100), tst varchar(600), det varchar(100),
                   eh varchar(100), vid integer unsigned, aid integer unsigned,
                   wtype enum('num','str','dat','blob'))
  deterministic
  sql security invoker
 begin

  declare i, z, nlwi integer;
  declare mgf, akid, lwi, slgrid integer unsigned;
  declare cmode enum('Hour','Day','Week','Month','special');
  declare cids varchar(100);
  declare gcompcur cursor for
    select ID, gr.slavegroupID, cperiod, hourIDs
    from
     ( select ID, groupID, 'Hour' as cperiod, hourIDs
       from gautocompression as ga
       where aktiv = '1' and ga.accessID = aid
         and testreg(srv, server, server_v) and testreg(sec, section, section_v)
         and testreg(tst, teststep, teststep_v) and testreg(det, details, details_v)
         and testreg(eh, einheit, einheit_v)
         and hourIDs != ''
      union all
       select ID, groupID, 'Day' as cperiod, dayIDs
       from gautocompression as ga
       where aktiv = '1' and ga.accessID = aid
         and testreg(srv, server, server_v) and testreg(sec, section, section_v)
         and testreg(tst, teststep, teststep_v) and testreg(det, details, details_v)
         and testreg(eh, einheit, einheit_v)
         and dayIDs != ''
      union all
       select ID, groupID, 'Week' as cperiod, weekIDs
       from gautocompression as ga
       where aktiv = '1' and ga.accessID = aid
         and testreg(srv, server, server_v) and testreg(sec, section, section_v)
         and testreg(tst, teststep, teststep_v) and testreg(det, details, details_v)
         and testreg(eh, einheit, einheit_v)
         and weekIDs != ''
      union all
       select ID, groupID, 'Month' as cperiod, monthIDs
       from gautocompression as ga
       where aktiv = '1' and ga.accessID = aid
         and testreg(srv, server, server_v) and testreg(sec, section, section_v)
         and testreg(tst, teststep, teststep_v) and testreg(det, details, details_v)
         and testreg(eh, einheit, einheit_v)
         and monthIDs != ''
      union all
       select ID, groupID, 'special' as cperiod, specialIDs
       from gautocompression as ga
       where aktiv = '1' and ga.accessID = aid
         and testreg(srv, server, server_v) and testreg(sec, section, section_v)
         and testreg(tst, teststep, teststep_v) and testreg(det, details, details_v)
         and testreg(eh, einheit, einheit_v)
         and specialIDs != ''
     ) as c
    join loggingtmp.grights as gr using(groupID);
  declare continue handler for not found set z = 1;

  select max(ID) into mgf from groupfunctions
      where aggregationtypes != '' OR aggregationspecialtypes != '';

  open gcompcur;
  set z = 0;
  fetch gcompcur into akid, slgrid, cmode, cids;
  while z != 1 do
    set i = 1;
    while i <= mgf do
      if find_in_set(i, cids)
          and exists (select * from groupfunctions
                      where ID = i and find_in_set(wtype, aggregationtypes)) then
        insert ignore into lwerteindizes
          (serverID, sectionID, teststepID, teststepdetailsID, accessID, groupfunctionsID, period)
          values (si, ei, ti, di, aid, i, cmode);
        set nlwi = row_count();
        set lwi = (select lwerteindizes.ID from lwerteindizes
                   where serverID = si and sectionID = ei and teststepID = ti and teststepdetailsID = di and accessID = aid
                     and groupfunctionsID = i and period = cmode);
        insert ignore into komprimierungids
          (autokomprimierungID, groupautoID, slavegroupID, lwerteindizesID) values
          (0, akid, slgrid, lwi);
        if nlwi > 0 then
          call kautos(lwi, cls, srv, sec, tst, i, cmode, det, eh, vid, aid);
        end if;
      end if;
      set i = i + 1;
    end while;
    fetch gcompcur into akid, slgrid, cmode, cids;
  end while;
  close gcompcur;

end//

GRANT EXECUTE on PROCEDURE newgcompautos to logging@'localhost', loggingadmin@'localhost'//
