-- rogroupautos.sql
--
-- procedure grosstids(domids, grpid, aid, dom, domv, s, sv, e, ev, t, tv, d)
-- domids: set of domain IDs
-- grpid: groupID
-- aid: accessID
-- dom: domain regexp
-- domv: domain not regexp
-- s: server regexp
-- sv: server not regexp
-- e: section regexp
-- ev: section not regexp
-- t: teststep regexp
-- tv: teststep not regexp
-- d: set of teststepdetails
--
-- create temporary table loggingtmp.gsstids with matching serverID, sectionID, teststepID for online values

-- procedure grocsstids(domids, grpid, aid, dom, domv, s, sv, e, ev, t, tv, d)
-- domids: set of domain IDs
-- grpid: groupID
-- aid: accessID
-- dom: domain regexp
-- domv: domain not regexp
-- s: server regexp
-- sv: server not regexp
-- e: section regexp
-- ev: section not regexp
-- t: teststep regexp
-- tv: teststep not regexp
-- d: set of teststepdetails
--
-- create temporary table loggingtmp.gcsstids with matching serverID, sectionID, teststepID for compressed values

-- procedure rogroupautos(groupids, domids, aid)
-- groupids: set of group IDs
-- domids: set of domain IDs
-- aid: accessID
--
-- update auto/gauto*limits for groupids and domids with accessID aid and only read only right

--
-- Wird aufgerufen aus Admin-Menue, wenn
-- einer Gruppe Verfahren hinzugefuegt
-- werden oder einem Verfahren Gruppen
-- hinzugefuegt werden
--

set @@character_set_client = 'utf8';

delimiter //

DROP procedure IF EXISTS grosstids//
create procedure grosstids(domids varchar(20000), grpid integer unsigned, aid integer unsigned,
                   dom varchar(500), domv varchar(300), s varchar(500), sv varchar(300),
                   e varchar(500), ev varchar(300), t varchar(500), tv varchar(300),
                   d varchar(500))
  deterministic
  sql security invoker
 begin

  create temporary table loggingtmp.srvids (primary key(ID))
    select distinct s.ID
    from verfahren as v
    join gruppenrechte
      on gruppenrechte.verfahrenID = v.ID
        and gruppenrechte.gruppenID = grpid and accessID = aid
    join server as s on s.verfahrenID = v.ID
    where find_in_set(v.ID, domids) and testreg(v.name, dom, domv) and testreg(s.servername, s, sv);

  create temporary table loggingtmp.secids (primary key(ID))
    select ID from section
    where testreg(section.name, e, ev);

  create temporary table loggingtmp.tstids (primary key(ID))
    select ID from teststep
    where testreg(teststep.name, t, tv);

  create temporary table loggingtmp.detids (primary key(ID))
    select teststepdetails.ID
    from teststepdetails
    where find_in_set(teststepdetails.name, d);

  create temporary table loggingtmp.gsstids (primary key(serverID,sectionID,teststepID))
    select distinct w.serverID, w.sectionID, w.teststepID
    from werteindizes as w
    where w.serverID in (select ID from loggingtmp.srvids)
      and w.sectionID in (select ID from loggingtmp.secids)
      and w.teststepID in (select ID from loggingtmp.tstids)
      and w.teststepdetailsID in (select ID from loggingtmp.detids)
      and w.accessID = aid;

  drop temporary table if exists loggingtmp.srvids, loggingtmp.secids,
    loggingtmp.tstids, loggingtmp.detids;

end//


DROP procedure IF EXISTS grocsstids//
create procedure grocsstids(domids varchar(20000), grpid integer unsigned, aid integer unsigned,
                   dom varchar(500), domv varchar(300), s varchar(500), sv varchar(300),
                   e varchar(500), ev varchar(300), t varchar(500), tv varchar(300),
                   d varchar(500))
  deterministic
  sql security invoker
 begin

  create temporary table loggingtmp.srvids (primary key(ID))
    select distinct s.ID
    from verfahren as v
    join gruppenrechte
      on gruppenrechte.verfahrenID = v.ID
        and gruppenrechte.gruppenID = grpid and accessID = aid
    join server as s on s.verfahrenID = v.ID
    where find_in_set(v.ID, domids) and testreg(v.name, dom, domv) and testreg(s.servername, s, sv);

  create temporary table loggingtmp.secids (primary key(ID))
    select ID from section
    where testreg(section.name, e, ev);

  create temporary table loggingtmp.tstids (primary key(ID))
    select ID from teststep
    where testreg(teststep.name, t, tv);

  create temporary table loggingtmp.gcsstids (primary key(serverID,sectionID,teststepID))
    select distinct lw.serverID, lw.sectionID, lw.teststepID
    from lwerteindizes as lw
    join loggingtmp.srvids on srvids.ID = lw.serverID
    join loggingtmp.secids on secids.ID = lw.sectionID
    join loggingtmp.tstids on tstids.ID = lw.teststepID
    join teststepdetails as d on d.ID = lw.teststepdetailsID
    join groupfunctions as gf on gf.ID = lw.groupfunctionsID
    where find_in_set(concat(gf.code, '[', left(period, 1), '](', d.name, ')'), d);

  drop temporary table if exists loggingtmp.srvids, loggingtmp.secids,
    loggingtmp.tstids;

end//



DROP procedure IF EXISTS rogroupautos//
create procedure rogroupautos(groupids varchar(20000), domids varchar(20000),
                              aid integer unsigned)
  deterministic
  sql security invoker
 begin
  declare dom, s, e, t, d varchar(500);
  declare domv, sv, ev, tv varchar(300);
  declare q, fkt varchar(10000);
  declare cnstvars, uwids varchar(3000);
  declare i, si, ci, ti, gaid, grpid, fdwid, z integer unsigned;
  declare dtype, ctype enum('num','str','dat','blob');
  declare gsstcur cursor for
    select serverID, sectionID, teststepID
    from loggingtmp.gsstids;
  declare gcsstcur cursor for
    select serverID, sectionID, teststepID
    from loggingtmp.gcsstids;
  declare gcdetcur cursor for
    select gautocombidetails.ID, domain, domain_v, server, server_v, section, section_v,
      teststep, teststep_v, details, formel, einheiten.type, gautocombidetails.groupID
    from gautocombidetails
    join teststepdetails as td on td.ID = teststepdetailsID
    left join einheiten on einheiten.ID = td.einheitID and einheiten.type != 'str'
    where aktiv = '1' and find_in_set(groupID, groupids);
  declare gccdetcur cursor for
    select gautoccdetails.ID, domain, domain_v, server, server_v, section, section_v,
      teststep, teststep_v, details, formel, einheiten.type, gautoccdetails.groupID
    from gautoccdetails
    join teststepdetails as td on td.ID = teststepdetailsID
    left join einheiten on einheiten.ID = td.einheitID and einheiten.type != 'str'
    where aktiv = '1' and find_in_set(groupID, groupids);
  declare continue handler for not found set z = 1;


-- Kombidetails

  open gcdetcur;
  set z = 0;
  fetch gcdetcur into gaid, dom, domv, s, sv, e, ev, t, tv, d, fkt, dtype, grpid;
  while z != 1 do
    call grosstids(domids, grpid, aid, dom, domv, s, sv, e, ev, t, tv, d);
    delete loggingtmp.gsstids from loggingtmp.gsstids
      join kombidetails as kd
        on kd.groupautoID = gaid and kd.serverID = gsstids.serverID
          and kd.sectionID = gsstids.sectionID and kd.teststepID = gsstids.teststepID;
    open gsstcur;
    set z = 0;
    fetch gsstcur into si, ci, ti;
    while z != 1 do
      call convert_combi(si, ci, ti, aid, fkt, 'b', ',datum', q, fdwid, ctype, cnstvars, uwids);
      if q != '' then
        insert into kombidetails
          (serverID, sectionID, teststepID, teststepdetailsID, formel, formelsql,
           constvars, firstdetwiID, groupautoID, bemerkungen, stringflag,
           aktiv, accessID, usedwIDs)
        select si, ci, ti, teststepdetailsID, formel, q,
          cnstvars, fdwid, ID, bemerkungen, coalesce(dtype, ctype) != 'num', '1', accessID, trim(leading ',' from uwids)
        from gautocombidetails
        where ID = gaid;
      end if;
      set z = 0;
      fetch gsstcur into si, ci, ti;
    end while;
    close gsstcur;
    drop temporary table loggingtmp.gsstids;
    set z = 0;
    fetch gcdetcur into gaid, dom, domv, s, sv, e, ev, t, tv, d, fkt, dtype, grpid;
  end while;
  close gcdetcur;


-- komprimierte Kombidetails

  open gccdetcur;
  set z = 0;
  fetch gccdetcur into gaid, dom, domv, s, sv, e, ev, t, tv, d, fkt, dtype, grpid;
  while z != 1 do
    call grocsstids(domids, grpid, aid, dom, domv, s, sv, e, ev, t, tv, d);
    delete loggingtmp.gcsstids from loggingtmp.gcsstids
      join ccdetails as ccd
        on ccd.groupautoID = gaid and ccd.serverID = gcsstids.serverID
          and ccd.sectionID = gcsstids.sectionID and ccd.teststepID = gcsstids.teststepID;
    open gcsstcur;
    set z = 0;
    fetch gcsstcur into si, ci, ti;
    while z != 1 do
      call convert_ccombi(si, ci, ti, aid, fkt, 'b', ',datum', q, fdwid, ctype, cnstvars, uwids);
      if q != '' then
        insert into ccdetails
          (serverID, sectionID, teststepID, teststepdetailsID, formel, formelsql,
           constvars, firstdetwiID, groupautoID, bemerkungen, stringflag,
           aktiv, accessID, usedwIDs)
        select si, ci, ti, teststepdetailsID, formel, q, cnstvars, fdwid, ID, bemerkungen,
          coalesce(dtype, ctype) != 'num', '1', accessID, trim(leading ',' from uwids)
        from gautoccdetails
        where ID = gaid;
      end if;
      set z = 0;
      fetch gcsstcur into si, ci, ti;
    end while;
    close gcsstcur;
    drop temporary table loggingtmp.gcsstids;
    set z = 0;
    fetch gccdetcur into gaid, dom, domv, s, sv, e, ev, t, tv, d, fkt, dtype, grpid;
  end while;
  close gccdetcur;

end//
