-- test_clientsuspension.sql
--
-- procedure test_clientsuspension(srvid, accid, e, t, d, u, startdate, inout enddate)
-- srvid: Server-ID
-- accid: Access-ID
-- e: section
-- t: teststep
-- d: detail
-- u: unit
-- startdate: startdate of suspension
-- enddate: enddate of suspension

-- Testet vom Client angeforderte Ueberwachungspause
-- Return: Datum/Zeit, bis zu der die Pause erlaubt ist oder
--         '0000-00-00 00:00:00' bei nicht erlaubt
--
-- Aufrufparameter
--   srvid: Server-ID
--   accessID: AccessID
--   e: angeforderte Sektion
--   t: angeforderte Teststep
--   d: angeforderte Detail
--   u: angeforderte Unit
--   startdate: angeforderte Startzeit
--   enddate:   angeforderte Endezeit
--
-- Ergebnis
--   enddate:   erlaubte Endezeit oder '0000-00-00 00:00:00', wenn nicht erlaubt 
--
-- Variablen
--   tes:  Initial: (e == '' ? '*' : e)
--   te:
--
--   ttts: (t == '' ? '*' : t)
--   tts:  noch nicht geprüfter Teil des ttts
--   tt:   zu prüfender Teil des tts, vor dem ertsen ','
--
--   ttds: (d == '' ? '*' : d)
--   tds:  noch nicht geprüfter Teil des ttds
--   td:   zu prüfender Teil des tds, vor dem ertsen ','
--
--   ttus: (u == '' ? '*' : u)
--   tus:  noch nicht geprüfter Teil des ttus
--   tu:   zu prüfender Teil des tus, vor dem ertsen ','

set @@character_set_client = 'utf8';

delimiter //

DROP PROCEDURE IF EXISTS test_clientsuspension//
create procedure test_clientsuspension(srvid integer unsigned, accid integer unsigned,
                                       e varchar(500), t varchar(500), d varchar(500), u varchar(200),
                                       INOUT startdate datetime, INOUT enddate datetime)
  deterministic
  sql security invoker
  begin
    declare i integer;
    declare domid integer unsigned;
    declare tes, te, ttts, tts, tt, ttds, tds, td varchar(500);
    declare ttus, tus, tu varchar(200);
    declare srvname, srvclass varchar(50);
    declare allowedstart, allowedend, newstartdate, newenddate datetime;

    -- prüfen, ob im angeforderten Zeitraum eine Pause über alles erlaubt ist
    select verfahrenID, servername, coalesce(sc.name, '')
      into domid, srvname, srvclass
      from server
      left join serverclasses as sc on sc.ID = server.serverclassesID
      where server.ID = srvid;
    
    if (select count(*) from clientsuspensions
        where domainID = domid and accessID = accid
          and testreg(srvname, server, server_v) and testreg(srvclass, classes, classes_v)
          and (sections = '.*' or sections = '^.*$') and sections_v = ''
          and (teststeps = '.*' or teststeps = '^.*$') and teststeps_v = ''
          and (details = '.*' or details = '^.*$') and details_v = ''
          and (units = '.*' or units = '^.*$') and units_v = ''
          and ((starttime = '00:00:00' and endtime = '23:59:59'
                and weekdays = '0,1,2,3,4,5,6' and mdays = '')
               OR (date(startdate) = date(enddate)
                   and find_in_set(weekday(startdate), weekdays)
                   and (mdays = '' OR find_in_set(day(startdate), mdays))
                   and starttime <= time(startdate) and time(startdate) <= endtime))) = 0 then
      -- keine generelle Pause, dann alle Elemente der angeforderten sections, teststeps, details und units prüfen
      set allowedend = enddate;

      if e = '' then
        set tes = '*';
      else
        set tes = e;
      end if;
      if t = '' then
        set ttts = '*';
      else
        set ttts = t;
      end if;
      if d = '' then
        set ttds = '*';
      else
        set ttds = d;
      end if;
      if u = '' then
        set ttus = '*';
      else
        set ttus = u;
      end if;
      -- Schleife über Sections
      while tes != '' do
        set i = instr(tes, ',');
        if i = 0 then
          set te = tes;
          set tes = '';
        else
          set te = left(tes, i-1);
          set tes = substring(tes, i+1);
        end if;
      
        set tts = ttts;
        -- Schleife über Teststeps
        while tts != '' do
          set i = instr(tts, ',');
          if i = 0 then
            set tt = tts;
            set tts = '';
          else
            set tt = left(tts, i-1);
            set tts = substring(tts, i+1);
          end if;
      
          set tds = ttds;
          -- Schleife über Details
          while tds != '' do
            set i = instr(tds, ',');
            if i = 0 then
              set td = tds;
              set tds = '';
            else
              set td = left(tds, i-1);
              set tds = substring(tds, i+1);
            end if;
      
            set tus = ttus;
            -- Schleife über Units
            while tus != '' do
              set i = instr(tus, ',');
              if i = 0 then
                set tu = tus;
                set tus = '';
              else
                set tu = left(tus, i-1);
                set tus = substring(tus, i+1);
              end if;

              -- generelle Pause über das einzelne Section/Teststep/Detail/Unit oder Zeitraum an einem Tag
              if (select count(*) from clientsuspensions
                  where domainID = domid and accessID = accid
                    and testreg(srvname, server, server_v) and testreg(srvclass, classes, classes_v)
                    and testreg(te, sections, sections_v) and testreg(tt, teststeps, teststeps_v)
                    and testreg(td, details, details_v) and testreg(tu, units, units_v)
                    and ((starttime = '00:00:00' and endtime = '23:59:59'
                          and weekdays = '0,1,2,3,4,5,6' and mdays = '')
                         OR (date(startdate) = date(enddate)
                             and find_in_set(weekday(startdate), weekdays)
                             and (mdays = '' OR find_in_set(day(startdate), mdays))
                             and starttime <= time(startdate) and time(startdate) <= endtime))) = 0 then
                -- keine generelle Pause erlaubt
                set newenddate = NULL;
                set newstartdate = NULL;
                select max(case when endtime < starttime then
                            addtime(date(startdate), endtime + interval 24 hour)
                          when starttime = '00:00:00' AND endtime = '23:59:59' then
                            allowed_range(startdate, enddate, mdays, weekdays)
                          else addtime(date(startdate), endtime)
                        end) as ed
                  into newenddate
                  from clientsuspensions
                  where domainID = domid and accessID = accid
                    and testreg(srvname, server, server_v) and testreg(srvclass, classes, classes_v)
                    and testreg(te, sections, sections_v) and testreg(tt, teststeps, teststeps_v)
                    and testreg(td, details, details_v) and testreg(tu, units, units_v)
                    and find_in_set(weekday(startdate), weekdays)
                    and (mdays = '' OR find_in_set(day(startdate), mdays))
                    and starttime <= time(startdate) and (endtime < starttime OR time(startdate) <= endtime);
      
                if newenddate is NULL then
                  set allowedend = '0000-00-00 00:00:00';
                  set tus = '';
                  set tds = '';
                  set tts = '';
                  set tes = '';
                else
                  if newenddate < allowedend then
                    set allowedend = newenddate;
                  end if;
                end if;
              end if;
            end while;
          end while;
        end while;
      end while;
      if startdate > allowedend then
        set enddate = '0000-00-00 00:00:00';
      else
        set enddate = allowedend;
      end if;
    end if;
  end//
