-- update_maintenance.sql
--
-- procedure update_maintenance()
-- 
-- update allowed server suspensions from maintenance window

set @@character_set_client = 'utf8';

delimiter //
DROP PROCEDURE IF EXISTS update_maintenance//
create procedure update_maintenance()
  deterministic
  sql security invoker
  begin
    declare odid, did, z integer unsigned;
    declare osn, sn, flstn, olstn, lstn varchar(50);
    declare ostrtm, strtm, oetm, etm varchar(8);
    declare omds, mds, orem, rem varchar(100);
    declare owd, wd varchar(1);
    declare maintcur cursor for
      select domainID, left(servername, char_length(servername) - char_length(lastnum)) as srvstart,
        lastnum, weekdays, mdays, starttime,
        if(starttime > '20:00:00', subtime(starttime, '20:00:01'), addtime(starttime, '03:59:59')) as endtime,
        remark
      from loggingtmp.maint
      order by domainID, servername;
    declare continue handler for not found set z = 1;

    create temporary table loggingtmp.wiids (primary key(ID))
      select distinct serverID as ID from werteindizes where accessID = 1;

    create temporary table loggingtmp.maint (primary key(domainID, servername))
      select verfahrenID as domainID, servername, substr(maintenancewindow, 6, 1)-1 as weekdays,
        concat(substr(maintenancewindow, 11, 2), ':', substr(maintenancewindow, 13, 2), ':00') as starttime,
        setrange(concat(substr(maintenancewindow, 4, 1)*7-6, '-', substr(maintenancewindow, 4, 1)*7), 1, 31) as mdays,
        substr(maintenancewindow, 1, 14) as remark,
        strlnum(servername) as lastnum
      from server
      join loggingtmp.wiids using(ID)
      where state = 'active' and maintenancewindow regexp '^MW_[1-4][.][1-7]_[A-Z][a-z]_[0-2][0-9][0-5][0-9]'
      union all
      select verfahrenID as domainID, servername, substr(maintenancewindow, 6, 1)-1 as weekdays,
        concat(substr(maintenancewindow, 11, 5), ':00') as starttime,
        setrange(concat(substr(maintenancewindow, 4, 1)*7-6, '-', substr(maintenancewindow, 4, 1)*7), 1, 31) as mdays,
        concat(substr(maintenancewindow, 1, 12), substr(maintenancewindow, 14, 2)) as remark,
        strlnum(servername) as lastnum
      from server
      join loggingtmp.wiids using(ID)
      where state = 'active' and maintenancewindow regexp '^MW_[1-4][.][1-7]_[A-Z][a-z]_[0-2][0-9]:[0-5][0-9]';

    create temporary table loggingtmp.maints (
        domainID int unsigned not null default '0',
        weekdays varchar(1) not null default '',
        starttime time NOT NULL DEFAULT '00:00:00',
        endtime time NOT NULL DEFAULT '23:59:59',
        mdays varchar(90) NOT NULL DEFAULT '',
        servername varchar(2000) NOT NULL DEFAULT '',
        remark varchar(255) NOT NULL,
        key domid(domainID, weekdays, starttime, mdays)
      );

    open maintcur;
    fetch maintcur into odid, osn, olstn, owd, omds, ostrtm, oetm, orem;
    set flstn = olstn;
    set z = 0;
    fetch maintcur into did, sn, lstn, wd, mds, strtm, etm, rem;
    while z = 0 do
      if odid != did or osn != sn or orem != rem or char_length(olstn) != char_length(lstn) or olstn+1 != lstn+0 then
        if flstn != olstn then
          insert into loggingtmp.maints (domainID, weekdays, starttime, endtime, mdays, servername, remark) values
            (odid, owd, ostrtm, oetm, omds, genreg(concat(osn, '{', flstn, '-', olstn, '}')), orem);
        else
          insert into loggingtmp.maints (domainID, weekdays, starttime, endtime, mdays, servername, remark) values
            (odid, owd, ostrtm, oetm, omds, genreg(concat(osn, olstn)), orem);
        end if;
        set odid = did;
        set osn = sn;
        set olstn = lstn;
        set owd = wd;
        set omds = mds;
        set ostrtm = strtm;
        set oetm = etm;
        set orem = rem;
        set flstn = olstn;
      else
        set olstn = lstn;
      end if;
      set z = 0;
      fetch maintcur into did, sn, lstn, wd, mds, strtm, etm, rem;
    end while;
    close maintcur;
    if osn != sn or olstn != lstn then
      if flstn != olstn then
        insert into loggingtmp.maintg (domainID, weekdays, starttime, endtime, mdays, servername, remark) values
          (odid, owd, ostrtm, oetm, omds, genreg(concat(osn, '{', flstn, '-', olstn, '}')), orem);
      else
        insert into loggingtmp.maintg (domainID, weekdays, starttime, endtime, mdays, servername, remark) values
          (odid, owd, ostrtm, oetm, omds, genreg(concat(osn, olstn)), orem);
      end if;
    end if;

    create temporary table loggingtmp.maintg (
        domainID int unsigned not null default '0',
        weekdays varchar(1) not null default '',
        starttime time NOT NULL DEFAULT '00:00:00',
        endtime time NOT NULL DEFAULT '23:59:59',
        mdays varchar(90) NOT NULL DEFAULT '',
        servernames varchar(20000) NOT NULL DEFAULT '',
        remark varchar(255) NOT NULL,
        primary key (domainID, weekdays, starttime, mdays)
      );

    insert into loggingtmp.maintg (domainID, weekdays, starttime, endtime, mdays, servernames, remark)
      select domainID, weekdays, starttime, endtime, mdays,
        group_concat(servername order by servername separator '|') as servernames,
        concat(remark, '\nInserted by Maintenance don\'t delete or change') as remark
      from loggingtmp.maints
      group by domainID, weekdays, starttime, mdays;
  
    delete clientsuspensions
      from clientsuspensions
      left join loggingtmp.maintg using(domainID, weekdays, starttime, mdays)
      where maintg.domainID is NULL
       and clientsuspensions.remark like 'MW_%\nInserted by Maintenance don\'t delete or change';

    update clientsuspensions as c
      join loggingtmp.maintg as g using(domainID, weekdays, starttime, mdays)
      set
        c.classes = '.*', c.classes_v = '',
        c.server = g.servernames, c.server_v = '',
        c.sections = '.*', c.sections_v = '',
        c.teststeps = '.*', c.teststeps_v = '',
        c.details = '.*', c.details_v = '',
        c.units = '.*', c.units_v = '',
        c.endtime = g.endtime
      where c.remark like 'MW_%\nInserted by Maintenance don\'t delete or change';

    insert into clientsuspensions
      (domainID, weekdays, starttime, mdays, classes, classes_v, server, server_v,
       sections, sections_v, teststeps, teststeps_v, details, details_v,
       units, units_v, endtime, remark)
      select g.domainID, g.weekdays, g.starttime, g.mdays, '.*', '', g.servernames, '',
        '.*', '', '.*', '', '.*', '', '.*', '', g.endtime, g.remark
      from loggingtmp.maintg as g
      left join clientsuspensions using(domainID, weekdays, starttime, mdays, remark)
      where clientsuspensions.domainID is NULL;

    drop temporary table loggingtmp.wiids, loggingtmp.maint, loggingtmp.maints, loggingtmp.maintg;
end //
