-- updateacceptips.sql
--
-- procedure createtmpacceptips(ipset)
-- ipset: set of ip-addressranges
--
-- create temporary table loggingtmp.aips with ip-addressranges from ipset

--
-- procedure updateacceptips(srvid integer, ipset)
-- srvid: serverID 
-- ipset: set of ip-addressranges
--
-- update allowed ip-addressranges in table acceptips

set @@character_set_client = 'utf8';

delimiter //
DROP PROCEDURE IF EXISTS createtmpacceptips//
create procedure createtmpacceptips(ipset varchar(1500))
  deterministic
  sql security invoker
  begin
    declare i, b integer;
    declare tips varchar(1500);
    declare tip varchar(50);
    declare htip varchar(32);
    declare hts, hte varchar(16);
    declare ht bigint unsigned;

    create temporary table loggingtmp.aips (
        `ipstart` varchar(32) NOT NULL DEFAULT '',
        `ipend` varchar(32) NOT NULL DEFAULT '',
        PRIMARY KEY (`ipstart`)
      );

    set tips = replace(ipset, ' ', '');
    while tips != '' do
      set i = instr(tips, ',');
      if i = 0 then
        set tip = tips;
        set tips = '';
      else
        set tip = left(tips, i-1);
        set tips = substring(tips, i+1);
      end if;
      if tip != '' then
        set i = instr(tip, '/');
        if i = 0 then
          if inet6_aton(tip) is not null then
            insert into loggingtmp.aips (ipstart, ipend) values
              (hex(inet6_aton(tip)), hex(inet6_aton(tip)));
          end if;
        else
          set b = substring(tip, i+1);
          set tip = left(tip, i-1);
          if is_ipv6(tip) then
            set htip = hex(inet6_aton(tip));
            if b > 64 then
              set ht = conv(substr(htip, 17), 16, 10);
              set hts = conv(ht & (-1 << 128-b), 10, 16);
              set hte = conv(ht | ~(-1 << 128-b), 10, 16);
              insert into loggingtmp.aips (ipstart, ipend) values
                (concat(left(htip, 16), concat(repeat('0', 16 - length(hts)), hts)),
                 concat(left(htip, 16), concat(repeat('0', 16 - length(hte)), hte)));
            else
              set ht = conv(left(htip, 16), 16, 10);
              set hts = conv(ht & (-1 << 64-b), 10, 16);
              set hte = conv(ht | ~(-1 << 64-b), 10, 16);
              insert into loggingtmp.aips (ipstart, ipend) values
                (concat(concat(repeat('0', 16 - length(hts)), hts), '0000000000000000'),
                 concat(concat(repeat('0', 16 - length(hte)), hte), 'FFFFFFFFFFFFFFFF'));
            end if;
          else
            set hts = conv(inet_aton(tip) & (-1 << 32-b), 10, 16);
            if hts is not null then
              set hte = conv(inet_aton(tip) | ~(-1 << 32-b), 10, 16);
              insert into loggingtmp.aips (ipstart, ipend) values
                (concat(repeat('0', 8 - length(hts)), hts),
                 concat(repeat('0', 8 - length(hte)), hte));
            end if;
          end if;
        end if;
      end if;
    end while;
  end//


DROP PROCEDURE IF EXISTS updateacceptips//
create procedure updateacceptips(srvid integer unsigned, ipset varchar(1500))
  deterministic
  sql security invoker
  begin

    call createtmpacceptips(ipset);

    delete acceptips from acceptips left join loggingtmp.aips using(ipstart)
      where serverID = srvid and aips.ipstart is NULL;
    update acceptips join loggingtmp.aips using(ipstart)
      set acceptips.ipend = aips.ipend
      where serverID = srvid;
    insert into acceptips (serverID, ipstart, ipend)
      select srvid, aips.ipstart, aips.ipend
        from loggingtmp.aips
        left join acceptips on serverID = srvid and acceptips.ipstart = aips.ipstart
        where acceptips.ipstart is NULL;

    drop temporary table loggingtmp.aips;
  end//
