-- weekdays.sql
--
-- function n2weekday(v) returns varchar
-- v: list of weekday numbers 1 == su -> 7 == sa
-- return: list of weekday names
--
-- function weekday2n(v) returns varchar
-- v: list of weekday names
-- return: list of weekday numbers 1 == su -> 7 == sa
--
-- function weekdaynames(v) returns varchar
-- v: list of weekday numbers 0 == mo -> 6 == so
-- return: list of weekday names

set @@character_set_client = 'utf8';

delimiter //

-- n2weekday, weekday2n - extendet Autofilter

DROP FUNCTION IF EXISTS n2weekday//
create function n2weekday(v varchar(14)) returns varchar(22)
  deterministic
  sql security invoker
  begin
    return coalesce(
      (select group_concat(w separator ',')
        from (select 'su' w, 1 c union select 'mo' w, 2 c union select 'tu' w, 3 c
              union select 'we' w, 4 c union select 'th' w, 5 c union select 'fr' w, 6 c
              union select 'sa' w, 7 c) wd
        where find_in_set(c, v)), '');
  end//

GRANT EXECUTE on FUNCTION n2weekday to logginglesen@'localhost'//


DROP FUNCTION IF EXISTS weekday2n//
create function weekday2n(v varchar(22)) returns varchar(14)
  deterministic
  sql security invoker
  begin
    return coalesce(
      (select group_concat(c separator ',')
        from (select 'su' w, 1 c union select 'mo' w, 2 c union select 'tu' w, 3 c
              union select 'we' w, 4 c union select 'th' w, 5 c union select 'fr' w, 6 c
              union select 'sa' w, 7 c) wd
        where find_in_set(w, v)), '');
  end//

GRANT EXECUTE on FUNCTION weekday2n to logginglesen@'localhost'//


-- weekdayname - ueberwachungspausen, mailreports

DROP FUNCTION IF EXISTS weekdaynames//
create function weekdaynames(v varchar(14)) returns varchar(22)
  deterministic
  sql security invoker
  begin
    return coalesce(
      (select group_concat(w separator ',')
        from (select 'su' w, 6 c union select 'mo' w, 0 c union select 'tu' w, 1 c
              union select 'we' w, 2 c union select 'th' w, 3 c union select 'fr' w, 4 c
              union select 'sa' w, 5 c) wd
        where find_in_set(c, v)), '');
  end//

GRANT EXECUTE on FUNCTION weekdaynames to logginglesen@'localhost'//
GRANT EXECUTE on FUNCTION weekdaynames to loggingadmin@'localhost'//
