set @@character_set_client = 'utf8';

alter table server
  modify ip int unsigned NOT NULL DEFAULT '0',
  modify transferip int unsigned NOT NULL DEFAULT '0',
  add ipv6 binary(16) null default NULL after ip;

create table acceptips (
  serverID int unsigned NOT NULL default '0',
  ipstart varchar(32) not null default '',
  ipend varchar(32) not null default '',
  primary key(serverID, ipstart));

delimiter //
DROP PROCEDURE IF EXISTS updateacceptips//
create procedure updateacceptips(srvid integer unsigned, ipset varchar(1500))
  deterministic
  sql security invoker
  begin
    declare i, b integer;
    declare tips varchar(1500);
    declare tip varchar(50);
    declare htip varchar(32);
    declare hts, hte varchar(16);
    declare ht bigint unsigned;

    create temporary table loggingtmp.aips like acceptips;
    set tips = replace(ipset, ' ', '');
    while tips != '' do
      set i = instr(tips, ',');
      if i = 0 then
        set tip = tips;
        set tips = '';
      else
        set tip = left(tips, i-1);
        set tips = substring(tips, i+1);
      end if;
      if tip != '' then
        set i = instr(tip, '/');
        if i = 0 then
          if inet6_aton(tip) is not null then
            insert into loggingtmp.aips (serverID, ipstart, ipend) values
              (srvid, hex(inet6_aton(tip)), hex(inet6_aton(tip)));
          end if;
        else
          set b = substring(tip, i+1);
          set tip = left(tip, i-1);
          if is_ipv6(tip) then
            set htip = hex(inet6_aton(tip));
            if b > 64 then
              set ht = conv(substr(htip, 17), 16, 10);
              set hts = conv(ht & (-1 << 128-b), 10, 16);
              set hte = conv(ht | ~(-1 << 128-b), 10, 16);
              insert into loggingtmp.aips (serverID, ipstart, ipend) values
                (srvid, concat(left(htip, 16), concat(repeat('0', 16 - length(hts)), hts)),
                 concat(left(htip, 16), concat(repeat('0', 16 - length(hte)), hte)));
            else
              set ht = conv(left(htip, 16), 16, 10);
              set hts = conv(ht & (-1 << 64-b), 10, 16);
              set hte = conv(ht | ~(-1 << 64-b), 10, 16);
              insert into loggingtmp.aips (serverID, ipstart, ipend) values
                (srvid, concat(concat(repeat('0', 16 - length(hts)), hts), '0000000000000000'),
                 concat(concat(repeat('0', 16 - length(hte)), hte), 'FFFFFFFFFFFFFFFF'));
            end if;
          else
            set hts = conv(inet_aton(tip) & (-1 << 32-b), 10, 16);
            if hts is not null then
              set hte = conv(inet_aton(tip) | ~(-1 << 32-b), 10, 16);
              insert into loggingtmp.aips (serverID, ipstart, ipend) values
                (srvid, concat(repeat('0', 8 - length(hts)), hts),
                 concat(repeat('0', 8 - length(hte)), hte));
            end if;
          end if;
        end if;
      end if;
    end while;
    delete acceptips from acceptips left join loggingtmp.aips using(serverID, ipstart)
      where serverID = srvid and loggingtmp.aips.serverID is NULL;
    update acceptips join loggingtmp.aips using(serverID, ipstart)
      set acceptips.ipend = aips.ipend;
    insert into acceptips
      select aips.* from loggingtmp.aips left join acceptips using(serverID, ipstart) where acceptips.ipend is NULL;
    drop temporary table loggingtmp.aips;
  end//

DROP PROCEDURE IF EXISTS insertacceptips//
create procedure insertacceptips()
  deterministic
  sql security invoker
  begin
    declare z, srvid integer unsigned;
    declare ipset varchar(1500);
    declare srvcur cursor for
      select ID, uls_werte_ips from server where uls_werte_ips != '';
    declare continue handler for not found set z = 1;

    set z = 0;
    open srvcur;
    fetch srvcur into srvid, ipset;
    while z = 0 do
      call updateacceptips(srvid, ipset);
      fetch srvcur into srvid, ipset;
    end while;
end //

delimiter ;
call insertacceptips();
DROP PROCEDURE insertacceptips;
